/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * activate.c
 */

#include "defs.h"


/*
 * standard logistic function
 */
float
logistic(float f)

{

    extern double exp();  
    
    return((float) (1.0 / (1.0 + exp(0.0 - f))));
 
}


/*
 * activate
 *
 * allow elements in one layer to activate another.
 * it gets sending layer pointer, sending layer size, receving layer pointer,
 * receiving layer size, weights pointer, receiving layer bias pointer
 */
void
act(float *b_actp, int b_size, float *t_actp, int t_size, float *t_weightp, float *t_biasp)

{
	float net;
	float *bap;
	int t;
	int tt;

	/*
	 * Activate Propagation Up Network
	 */
	for (t = 0; t < t_size; t++, t_actp++, t_biasp++) 
               {
		net = (float) 0.0;
		for (tt=0,bap = b_actp;tt < b_size;tt++,t_weightp++,bap++)
                     net += *t_weightp * *bap;
		net += *t_biasp;
		if (net < -10.0)
		    net = (float) -10.0;
	          else 
                    if (net > 10.0)
			net = (float) 10.0;		    
		*t_actp = logistic(net);
	      }
}




/*
 * propagate the activation layer by layer
 */
void
activate_net(struct  individual *pind)
{

	int i;
	float **wp;
	float **bp;
	float **lp;
	int   *ldp;

	wp  = pind->weightp;
	lp  = pind->layerp;
	ldp = pind->layer_descp;
	bp  = pind->biasp;

	for (i = 0; i < (pind->nlayers - 1); i++) 
	     {          
	       act(*lp, *ldp, *(lp + 1), *(ldp + 1), *wp, *(bp + 1));
	       ldp++;
	       lp++;
	       wp++;
	       bp++;
	      }
	

}


/*
 * same of activate_net for auto-teaching network
 */
void
auto_activate_net(struct individual *pind)
{

	register int i;
	float **wp;
	float **bp;
	float **lp;
	int   *ldp;

	wp  = pind->aweightp;
	lp  = pind->alayerp;
	ldp = pind->layer_descp;
	bp  = pind->abiasp;

	for (i = 0; i < (pind->nlayers - 1); i++) 
	     {
	       act(*lp, *ldp, *(lp + 1), *(ldp + 1), *wp, *(bp + 1));
	       ldp++;
	       lp++;
	       wp++;
	       bp++;
	      }
	
}


