/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * config.c
 */

#include "defs.h"

/*
 * read a word from a string
 */
int 
getword(char *word, char *line, char stop) 
{
    int x;
    int y;
    int l;

    x=0;
    for(x=0,l=0;((line[x]) && (line[x] != stop && line[x] != '\n'));x++,l++)
        word[x] = line[x];

    word[x] = '\0';
    if(line[x]) ++x;
    y=0;

    while(line[y++] = line[x++]);

    return(l);
}

/*
 * remove the first space characters from a string
 */
void 
clear_from_space(char *stringa) 
{

    while(*stringa == ' ')
       stringa++;

}


/* 
 * load the .cf file 
 * return -1 if it fail to open the file
 */
int
load_parameters(char *fileroot)

{

	extern  double atof();
	extern  int atoi();
	FILE	*fopen();
	FILE   *cfp;
	int     n;
	char    commands[128];
	char    comm[128];
	char    *c1,*c2;
	int     c,nc,enc;
	char	error[128];
        struct  individual *pind;

	cfp = fopen(fileroot, "r");
	if (cfp == NULL)
          return(-1);
	 /* 
	  * General parameters
	  */
	 npopulations = 0;
	 bestfathers  = 0;
	 nkids        = 0;
	 dpopulation  = 0;
	 fgets(commands,128,cfp);
	 c1 = commands;
	 while(*c1 == ' ')
               c1++;
	 while (*c1 != '\0' && *c1 != '\n')
	   {
             nc = getword(comm,c1,' ');
	     c = *comm;
	     c2 = comm;
	     c2++;
	     while(*c1 == ' ')
               c1++;
 	     switch (c) 
		{
		case 'N':
			npopulations = atoi(c2);
			enc = 2;
			break;	
		case 'B':
			bestfathers = atoi(c2);
			enc = 2;
			break;
		case 'C':
			nkids = atoi(c2);
			enc = 2;
			break;
		case 'O':
			opp_from_prev = atoi(c2);
			enc = 2;
			break;
		case 'd':
			dpopulation = atoi(c2);
			enc = 2;
			break;
		case '.':
			enc = 1;
			break;
		default:
			sprintf(error,"general:unknown key <%c> in %s\n",c, fileroot);
			display_error(error);
			break;

		}
	     if (enc == 1 && nc > 1)
	        {
		  sprintf(error,"general:unexpected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	     if (enc == 2 && nc < 2)
	        {
		  sprintf(error,"general:expected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	  }

        //create_poppointer();
	// init_parameters();
	/*
	 * for each population 
	 */
	for (n=0,pind=ind; n<npopulations; n++,pind++)
	 {
          /*
	   * population parameters (1 line for each population)
	   */
	  fgets(commands,128,cfp);
	  c1 = commands;
	  while(*c1 == ' ')
               c1++;
	  while (*c1 != '\0' && *c1 != '\n')
	    {
             nc = getword(comm,c1,' ');
	     c = *comm;
	     c2 = comm;
	     c2++;
	     while(*c1 == ' ')
               c1++;
 	     switch (c) 
		{
		/* general */
		case 'F':
			pind->ffitness = atoi(c2);
			enc = 2;
			break;	
		case 'H':
			pind->nhiddens = atoi(c2);
			enc = 2;
			break;
		case 'I':
			pind->wrange = (float) atof(c2);
			enc = 2;
			break;
		case 'L':
			//pind->nlayers = atoi(c2); we leave the flag for compatibility
			enc = 2;
			break;
		case 'P':
			pind->input_outputs = atoi(c2);
			enc = 2;
			break;
		/* sensors */
		case 'a':
                        pind->newcamerafive = 1;
			enc = 1;
			break;
		case 'b':
                        pind->groundsensor = 1;
			enc = 1;
			break;
		case 'c':
                        pind->camerafive = 1;
			enc = 1;
			break;				
		case 'e':
                        pind->elman_net = 1;
			enc = 1;
			break;
		case 'g':
                        pind->cameraunits = atoi(c2);
			enc = 2;
			break;
		case 'h':
                        pind->objectsensor = 1;
			enc = 1;
			break;
		case 'l':
                        pind->lightsensors = 1;
			enc = 1;
			break;
		case 'm':
                        pind->motmemory++;
			enc = 1;
			break;
		case 'o':
                        pind->oppsensor = 1;
			enc = 1;
			break;
		case 'q':
                        pind->camerarange = atoi(c2);
			enc = 2;
			break;
		case '4':
                        pind->nifsensors = 4;
			enc = 1;
			break;
		case '6':
                        pind->nifsensors = 6;
			enc = 1;
			break;
		case '8':
                        pind->nifsensors = 8;
			enc = 1;
			break;
		case 'n':
                        pind->battery = 1;
			enc = 1;
			break;
		case 'p':
                        pind->light2sensors = 1;
			enc = 1;
			break;
                /* motors */
		case 'A':
                        pind->autoteach = 1;
			enc = 1;
			break;				
		case 'D':
			pind->modularity = 2;
			enc = 1;
			break;			
		case 'G':
			pind->gripper = 1;
			enc = 1;
			break;
		case 'M':
                        pind->wheelmotors = 1;
			enc = 1;
			break;			
		case 'R':
                        pind->randomweights = 1;
			enc = 1;
			break;
		case 'X':
                        pind->fixed = 1;
			enc = 1;
			break;
		case 'W':
                        pind->hebbian_learning = 1;
			enc = 1;
			break;				
		case '.':
			enc = 1;
			break;
		default:
			sprintf(error,"pop:unknown key <%c> in %s\n",c, fileroot);
			display_error(error);
			break;

		}
	     if (enc == 1 && nc > 1)
	        {
		  sprintf(error,"pop:unexpected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	     if (enc == 2 && nc < 2)
	        {
		  sprintf(error,"pop:expected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	  }

	 }

	compute_parameters();
        create_pop();

	/* 
	 * Evolution parameters
	 */
	fgets(commands,128,cfp);
	c1 = commands;
	while(*c1 == ' ')
               c1++;
	while (*c1 != '\0' && *c1 != '\n')
	   {
             nc = getword(comm,c1,' ');
	     c = *comm;
	     c2 = comm;
	     c2++;
	     while(*c1 == ' ')
               c1++;
 	     switch (c) 
		{
		case 'G':
			generations = atoi(c2);
			enc = 2;
			break;	
		case 'g':
                        startgeneration = atoi(c2);
			enc = 2;
			break;
		case 'o':
                        percmut = atoi(c2);
			enc = 2;
			break;
		case 'R':
                        seed = atoi(c2);
			enc = 2;
			break;
		case 'd':
                        delfile = atoi(c2);
			enc = 2;
			break;
		case 'M':
                        all_of_fame = 1;
			enc = 1;
			break;
		case 'N':
			nreplications = atoi(c2);
			enc = 2;
			break;
		case 'X':
			sim_real = 1;
			enc = 1;
			break;
		case 'P':
			serialport = atoi(c2);;
			enc = 2;
			break;
		case 'S':
			serialspeed = atoi(c2);;
			enc = 2;
			break;
		case '.':
			enc = 1;
			break;
		default:
			sprintf(error,"ev:unknown key <%c> in %s\n",c, fileroot);
			display_error(error);
			break;

		}
	     if (enc == 1 && nc > 1)
	        {
		  sprintf(error,"ev:unexpected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	     if (enc == 2 && nc < 2)
	        {
		  sprintf(error,"ev:expected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	  }

	/* 
	 * Lifetime parameters
	 */
	fgets(commands,128,cfp);
	c1 = commands;
	while(*c1 == ' ')
               c1++;
	while (*c1 != '\0' && *c1 != '\n')
	   {
             nc = getword(comm,c1,' ');
	     c = *comm;
	     c2 = comm;
	     c2++;
	     while(*c1 == ' ')
               c1++;
 	     switch (c) 
		{
		case 'e':
			amoebaepochs = atoi(c2);
			enc = 2;
			break;	
		case 's':
                        sweeps = atoi(c2);
			enc = 2;
			break;
		case 'L':
                        timelaps = atoi(c2);
			enc = 2;
			break;
		case 'E':
                        save_error_each = atoi(c2);
			enc = 2;
			break;
		case 'r':
                        rate = (float) atof(c2);
			enc = 2;
			break;
		case 'n':
                        sensornoise = (float) atof(c2);
			enc = 2;
			break;
		case 'T':
			testsamep = 1;
			enc = 1;
			break;
		case 'b':
                        nobreak = 1;
			enc = 1;
			break;
		case 'p':
			reappears = 1;
			enc = 1;
			break;
		case 'c':
                        stop_when_crash = 1;
			enc = 1;
			break;
		case 'A':
			random_sround = 1;
			enc = 1;
			break;
		case 'C':
			random_round = 1;
			enc = 1;
			break;
		case 'o':
			startonfood = 1;
			enc = 1;
			break;
		case 'f':
                        startfrontal = 1;
			enc = 1;
			break;
		case 'B':
			startfarfrom = 1;
			enc = 1;
			break;
		case 'w':
			nworlds = atoi(c2);
			enc = 2;
			break;	
		case 'F':
			random_foodzone = 1;
			enc = 1;
			break;
		case 'a':
			alternate_wall = 1;
			enc = 1;
			break;
		case '.':
			enc = 1;
			break;
		default:
			sprintf(error,"life:unknown key <%c> in %s\n",c, fileroot);
			display_error(error);
			break;

		}
	     if (enc == 1 && nc > 1)
	        {
		  sprintf(error,"life:unexpected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	     if (enc == 2 && nc < 2)
	        {
		  sprintf(error,"life:expected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	  }

	/* 
	 * Display parameters
	 */
	fgets(commands,128,cfp);
	c1 = commands;
	while(*c1 == ' ')
               c1++;
	while (*c1 != '\0' && *c1 != '\n')
	   {
             nc = getword(comm,c1,' ');
	     c = *comm;
	     c2 = comm;
	     c2++;
	     while(*c1 == ' ')
               c1++;
 	     switch (c) 
		{
		case 'r':
			drawindividual = 1;
			enc = 1;
			break;	
		case 't':
                        drawtrace = 1;
			enc = 1;
			break;
		case 'c':
                        drawcamera = 1;
			enc = 1;
			break;
		case 'i':
                        drawindif = 1;
			enc = 1;
			break;
		case 'l':
                        drawindlight = 1;
			enc = 1;
			break;
		case 'g':
                        graphsensors = 1;
			enc = 1;
			break;
		case 's':
			drawinputs = 1;
			enc = 1;
			break;
		case 'h':
                        drawhiddens = 1;
			enc = 1;
			break;
		case 'o':
			drawoutputs = 1;
			enc = 1;
			break;
		case 'e':
                        drawerror = 1;
			enc = 1;
			break;
		case 'v':
			drawperceived = 1;
			enc = 1;
			break;
		case 'f':
                        drawfitness = 1;
			enc = 1;
			break;
		case 'j':
			timeadjust = atoi(c2);
			enc = 2;
			break;
		case 'u':
                        userscale = atoi(c2);
			enc = 2;
			break;
		case 'Q':
			oldstylewts = 1;
			enc = 1;
			break;
		case 'D':
			drawdata = 1;
			enc = 1;
			break;
		case 'w':
			draweights = 1;
			enc = 1;
			break;
		case 'z':
			drawallepochs = 1;
			enc = 1;
			break;
		case 'y':
			drawallpopulation = 1;
			enc = 1;
			break;	
		case 'b':
			drawteaching = 1;
			enc = 1;
			break;				
		case '.':
			enc = 1;
			break;
		default:
			sprintf(error,"display:unknown key <%c> in %s\n",c, fileroot);
			display_error(error);
			break;

		}
	     if (enc == 1 && nc > 1)
	        {
		  sprintf(error,"display:unexpected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	     if (enc == 2 && nc < 2)
	        {
		  sprintf(error,"display:expected par. in flag <%c> in %s\n",c, fileroot);
		  display_error(error);
	        }
	  }

        fclose(cfp);

	if (sim_real == 1)
	   aprecom();
	return(1);

}


/* 
 * save the parameters (with the exception of environmental data) in a .cf file 
 */
void
save_parameters(char *filename)

{

	extern  double atof();
	extern  int atoi();
	FILE	*fopen();
	FILE	*cffile;
	int     n;
        struct  individual *pind;

	cffile = fopen(filename, "w");
	if (cffile == NULL)
          display_warning("unable to same parameters in the specified filename");
	 /* 
	  * General parameters (included in parameters->population
	  */
         fprintf(cffile,"N%d ",npopulations);
         fprintf(cffile,"B%d ",bestfathers);
         fprintf(cffile,"d%d ",dpopulation);
         fprintf(cffile,"C%d ",nkids);
	 if (opp_from_prev > 0)
           fprintf(cffile,"O%d ",opp_from_prev);

         fprintf(cffile,"\n");
	 /* 
	  * Population parameters
	  */
	 for (n=0,pind=ind; n<npopulations; n++,pind++)
	   {
	    // current individual (pind->dindividual) not saved
	    if (pind->nifsensors == 4)
               fprintf(cffile,"4 ");
	    if (pind->nifsensors == 6)
               fprintf(cffile,"6 ");
	    if (pind->nifsensors == 8)
               fprintf(cffile,"8 ");
	    if (pind->lightsensors == 1)
               fprintf(cffile,"l ");
	    if (pind->groundsensor == 1)
               fprintf(cffile,"b ");
	    if (pind->camerafive == 1)
               fprintf(cffile,"c ");
	    if (pind->newcamerafive == 1)
               fprintf(cffile,"a ");
	    if (pind->oppsensor == 1)
               fprintf(cffile,"o ");
	    if (pind->objectsensor == 1)
               fprintf(cffile,"h ");
	    if (pind->elman_net == 1)
               fprintf(cffile,"e ");
	    if (pind->motmemory == 1)
               fprintf(cffile,"m ");
	    if (pind->battery == 1)
               fprintf(cffile,"n ");
	    if (pind->light2sensors == 1)
               fprintf(cffile,"p ");
	    if (pind->cameraunits > 0)
              fprintf(cffile,"g%d ",pind->cameraunits);
	    if (pind->camerarange > 0)
              fprintf(cffile,"q%d ",pind->camerarange);
	    if (pind->gripper == 1)
               fprintf(cffile,"G ");
	    if (pind->wheelmotors == 1)
               fprintf(cffile,"M ");
	    if (pind->modularity == 2)
               fprintf(cffile,"D ");
            //fprintf(cffile,"L%d ",pind->nlayers);
	    if (pind->nhiddens > 0)
              fprintf(cffile,"H%d ",pind->nhiddens);
	    if (pind->input_outputs > 0)
               fprintf(cffile,"P%d ",pind->input_outputs);
            fprintf(cffile,"I%.2f ",pind->wrange);
	    if (pind->randomweights == 1)
               fprintf(cffile,"R ");
	    if (pind->autoteach == 1)
               fprintf(cffile,"A ");
	    if (pind->hebbian_learning == 1)
               fprintf(cffile,"W ");
            fprintf(cffile,"F%d ",pind->ffitness);
	    if (pind->fixed == 1)
               fprintf(cffile,"X ");
            fprintf(cffile,"\n");
	   }
	/* 
	 * Evolution parameters
	 */
         fprintf(cffile,"G%d ",generations);

         fprintf(cffile,"G%d ",generations);
         fprintf(cffile,"g%d ",startgeneration);
         fprintf(cffile,"o%d ",percmut);
         fprintf(cffile,"R%d ",seed);
         fprintf(cffile,"d%d ",delfile);
         fprintf(cffile,"N%d ",nreplications);
         fprintf(cffile,"P%d ",serialport);
         fprintf(cffile,"S%d ",serialspeed);
	 if (all_of_fame == 1)
               fprintf(cffile,"M ");
	 if (sim_real == 1)
               fprintf(cffile,"X ");
         fprintf(cffile,"\n");
	/* 
	 * Lifetime parameters
	 */
         fprintf(cffile,"e%d ",amoebaepochs);
         fprintf(cffile,"s%d ",sweeps);
         fprintf(cffile,"L%d ",timelaps);
         fprintf(cffile,"E%d ",save_error_each);
         fprintf(cffile,"r%.2f ",rate);
         fprintf(cffile,"n%.2f ",sensornoise);
	 if (testsamep == 1)
               fprintf(cffile,"T ");
	 if (nobreak == 1)
               fprintf(cffile,"b ");
	 if (reappears == 1)
               fprintf(cffile,"p ");
	 if (stop_when_crash == 1)
               fprintf(cffile,"c ");
	 if (random_sround == 1)
               fprintf(cffile,"A ");
	 if (random_foodzone == 1)
               fprintf(cffile,"F ");
	 if (startonfood == 1)
               fprintf(cffile,"o ");
	 if (startfrontal == 1)
               fprintf(cffile,"f ");
	 if (startfarfrom == 1)
               fprintf(cffile,"B ");
	 if (nobreak == 1)
               fprintf(cffile,"b ");
	 if (alternate_wall == 1)
               fprintf(cffile,"a ");
         fprintf(cffile,"w%d ",nworlds);
         fprintf(cffile,"\n");
	/* 
	 * Display parameters
	 */
	 if (drawindividual == 1)
               fprintf(cffile,"r ");
	 if (drawtrace == 1)
               fprintf(cffile,"t ");
	 if (drawcamera == 1)
               fprintf(cffile,"c ");
	 if (drawindif == 1)
               fprintf(cffile,"i ");
	 if (drawindlight == 1)
               fprintf(cffile,"l ");
	 if (graphsensors == 1)
               fprintf(cffile,"g ");
	 if (drawinputs == 1)
               fprintf(cffile,"s ");
	 if (drawhiddens == 1)
               fprintf(cffile,"h ");
	 if (drawoutputs == 1)
               fprintf(cffile,"o ");
	 if (drawerror == 1)
               fprintf(cffile,"e ");
	 if (drawperceived == 1)
               fprintf(cffile,"v ");
	 if (drawfitness == 1)
               fprintf(cffile,"f ");
	 if (drawteaching == 1)
               fprintf(cffile,"b ");
         fprintf(cffile,"j%d ",timeadjust);
         fprintf(cffile,"u%d ",userscale);
	 if (drawdata == 1)
               fprintf(cffile,"D ");
	 if (draweights == 1)
               fprintf(cffile,"w ");
	 if (drawallepochs == 1)
               fprintf(cffile,"z ");
	 if (drawallpopulation == 1)
               fprintf(cffile,"y ");
         fprintf(cffile,"\n");

	 // missing all_of_fame,
        fclose(cffile);
}


/*
 * Initialize population parameters (we always initialize 2 populations)
 */
void
init_parameters()

{
    struct  individual *pind;
    int    n;
	
    for (n=0,pind=ind; n < 2 /* npopulations */; n++,pind++)
	 {
	  pind->population = n;
          pind->autoteach=0;    
          pind->modularity=0;  
          pind->modulskip=0;      
	  pind->gripper=0;
          pind->nifsensors=0;        
          pind->motmemory=0;          
          pind->groundsensor=0;       
          pind->objectsensor=0;          
          pind->camerafive=0;
	  pind->newcamerafive=0;
	  pind->lightsensors=0;
	  pind->light2sensors=0;
	  pind->battery=0;
	  pind->learning=0;
	  pind->elman_net=0;
	  pind->fixed=0;
	  pind->samepop=0;
          pind->wheelmotors=0;
	  pind->randomweights=0;
	  pind->error_sum = 0.0;
	  pind->perror= 0.0;
	  pind->error_n = 0;
	  pind->oppsensor=0;
          pind->dindividual = 0;
	  pind->nhiddens = 0;
	  pind->ffitness = 0;
	  pind->nlayers  = 0;
	  pind->wrange  = 10.0f;
	  pind->cameraunits   = 0;
	  pind->camerarange = 0;
	  pind->hebbian_learning = 0;
	  pind->input_outputs = 0;
	  pind->bottomsensor  = 0.0f;
	  pind->lightbarsensor = 0.0f;
	 }

}



/* 
 * Compute ninputs, noutputs and other variables for all populations
 */

void
compute_parameters()

{

   struct  individual *pind;
   int     i;
   int     ninputs,noutputs;
   int     n;
   int	   *ldp;
	

   nindividuals = nkids * bestfathers;
   for (n=0,pind=ind; n<npopulations; n++,pind++)
	 {
	  /* ninputs */
	  ninputs  = 0;
	  noutputs = 0;
          if (pind->newcamerafive == 1)
	     ninputs += 5;		
          if(pind->groundsensor   == 1)
             ninputs += 1;
	  if (pind->camerafive    == 1)
	     ninputs += 5;		
          if (pind->elman_net     == 1)
	     ninputs += pind->nhiddens;		    		    		    
          if (pind->objectsensor  == 1)
	     ninputs += 1;		    
          if (pind->lightsensors  == 1)
	     ninputs += 8;
          if (pind->motmemory     == 1)
             ninputs += 2;		    		    		    		    				    
          if (pind->nifsensors    == 4)
	     ninputs += 4;
          if (pind->nifsensors    == 6)
	     ninputs += 6;
          if (pind->nifsensors    == 8)
	     ninputs += 8;
          if (pind->cameraunits    > 0)
	      ninputs += pind->cameraunits; 
          if (pind->battery       == 1)
	      ninputs += 1; 
          if (pind->light2sensors == 1)
	      ninputs += 2; 
          /* noutputs */
          if (pind->wheelmotors   == 1)
	      noutputs += 2;
          if (pind->gripper       == 1)
	      noutputs += 2;     
  	  
	  /* special cases */
          if (pind->nhiddens > 0)
	     pind->nlayers = 3;
	    else
	     pind->nlayers = 2;
	  if (pind->input_outputs > 0)
	   {
	     noutputs += pind->input_outputs;
             ninputs  += pind->input_outputs;
	   }
          pind->modulskip = noutputs;
          if (pind->modularity == 2)	       	 	 
	      noutputs = noutputs * 2;
	  /*
	   * set the number of input, hidden, and output units and
	   *  calculate and save the number of weights amd biases
	   */
          for (i=0, ldp=pind->layer_descp; i<pind->nlayers; i++, ldp++) 
             {
	      if (i == 0)
		  *ldp = ninputs;
	      if (i == (pind->nlayers - 1))
		  *ldp = noutputs;
	      if (i > 0 && i < (pind->nlayers - 1))
		  *ldp = pind->nhiddens;
	     }
	  pind->numweights=0;
	  pind->numbiases=0;	
	  for (i=0, ldp=pind->layer_descp; i<pind->nlayers; i++, ldp++) 
            {
	      pind->numbiases += *ldp;
              if (i < (pind->nlayers -1))
	        pind->numweights += *ldp * *(ldp + 1);
	    }
	  pind->numchars = pind->numweights + pind->numbiases + 1;

	  if (pind->autoteach == 1 || pind->modularity == 2)
	    pind->numchars = pind->numchars * 2;	  
	 }

}


/* 
 * allocate space for population pointers
 * (we always allocate space for two populations)
 */
void
create_poppointer()

{
	extern	char *malloc();
        struct individual *pind;
	int n;

	create1 = 1;
        ind = (struct individual *) malloc(/* npopulations */ 2 * sizeof(struct individual));
        if (ind == NULL) 
	       {
		printf("individual malloc err\n");
	       }
	for(n=0,pind=ind; n < 2/* npopulations */;n++,pind++)
	  {
           pind->layer_descp = (int *)malloc(10 * sizeof(int)); 
	   pind->population = n;
	   if (pind->layer_descp == NULL) 
	       {
		printf("pind->layer_descp malloc err\n");
	       }
	  }
}

/*
 * allocate space for the genome of the population
 * for the genome of the reproducing individuals
 * and for one individual phenotype
 */
void
create_pop()

{
	extern	char  *malloc();
        struct  individual *pind;	
	int     i;
	int     j;
	int     p;
	int     num;
	int	**gpi;
	int	*gi;	
	float	**wp;
	float	**bp;
	float   **lp;
	float	**awp;
	float	**abp;
	float   **alp;
	int	*ldp;
		
	create2 = 1;
        /*
	 * compute the number of individuals
	 */
        nindividuals=nkids * bestfathers;

        /* 
	 * allocate space for the genome of individuals
	 * and initialize
	 */
 	genome = (int **) malloc(((nindividuals + opp_from_prev) * npopulations) * sizeof(int *));
	if (genome == NULL) 
	    display_error("System error: genome malloc");
	for (p = 0, pind=ind, gpi=genome; p < npopulations; p++, pind++)
	 {
	  /* we allocate space for individual genome */
	  for (i=0; i < nindividuals; i++,gpi++) 
	        {
	         *gpi = (int *)malloc(pind->numchars * sizeof(int));
	         if (*gpi == NULL)
	            display_error("System error: gpi malloc");
                 for (j=0,gi = *gpi; j < pind->numchars; j++,gi++)
		    {
		     *gi = mrand(256);
		    }
	        }
	  /* we allocate space for best genome of previous generations */
	  for (i=0; i < opp_from_prev; i++,gpi++) 
	        {
	         *gpi = (int *)malloc(pind->numchars * sizeof(int));
	         if (*gpi == NULL)
	            display_error("System error: previous genome malloc");
                 for (j=0,gi = *gpi; j < pind->numchars; j++,gi++)
		    {
		     *gi = mrand(256);
		    }
	        }
	 }

        /* 
	 * allocate space for the genome of reproducing ind.
	 */
 	bestgenome = (int **) malloc((bestfathers * npopulations) * sizeof(int *));
	if (bestgenome == NULL)
	   display_error("System error: bestgenome malloc");
	for (p = 0, pind=ind, gpi=bestgenome; p < npopulations; p++, pind++)
	  {
	    for (i=0; i < bestfathers; i++,gpi++) 
	        {
	         *gpi = (int *)malloc(pind->numchars * sizeof(int));
	         if (*gpi == NULL) 
	             display_error("System error: bestgi malloc");
	        }
	  }

        /*
         * for each population's individual
	 */
	for (p = 0, pind=ind; p < npopulations; p++, pind++)
	  {
	   /* 
	    * allocate space for layer pointers
	    */
	   pind->layerp = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->autoteach == 1 || pind->modularity == 2)
	      pind->alayerp = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->layerp == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && pind->alayerp == NULL))
	      display_error("System error: layerp malloc");	       
	   /*
	    * allocate space for layers 
	    */
	   for (i=0, lp=pind->layerp, alp=pind->alayerp, ldp=pind->layer_descp; i<pind->nlayers; i++, lp++, alp++, ldp++) 
	     {
	      *lp = (float *)malloc(*ldp * sizeof(float));
              if (pind->autoteach == 1 || pind->modularity == 2)
	        *alp = (float *)malloc(*ldp * sizeof(float));
	      if (*lp == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && *alp == NULL))
	         display_error("System error: lp malloc");
	     }
	   /*
	    * allocate space for bias pointers
	    */
	   pind->biasp = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->autoteach == 1 || pind->modularity == 2)
	     pind->abiasp = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->biasp == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && pind->abiasp == NULL)) 
	         display_error("System error: biasp malloc");	       
	   /*
	    * allocate space for biases
	    */
	   for (i=0, bp=pind->biasp, abp=pind->abiasp, ldp=pind->layer_descp; i<pind->nlayers; i++, bp++, abp++, ldp++) 
	    {
	     *bp = (float *)malloc(*ldp * sizeof(float));
	     if (pind->autoteach == 1 || pind->modularity == 2)
	       *abp = (float *)malloc(*ldp * sizeof(float));
	     if (*bp == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && *abp == NULL))
	         display_error("System error: bp malloc");
	    }
	   /*
	    * allocate space for weight pointers 
	    */
	   pind->weightp = (float **) malloc((pind->nlayers-1) * sizeof(float *));
	   if (pind->autoteach == 1 || pind->modularity == 2)
	     pind->aweightp = (float **) malloc((pind->nlayers-1) * sizeof(float *));
	   if (pind->weightp == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && pind->aweightp == NULL))
	        display_error("System error: weightp malloc");
	   /*
	    * ...and for actual weights (each unit in a given layer connecting
	    * to each unit in layer below)
	    */
	   for (i=0, wp=pind->weightp, awp=pind->aweightp, ldp=(pind->layer_descp+1); i<(pind->nlayers-1); i++,wp++,awp++) 
	       {
	         num = *(ldp + i) * *(ldp + i -1);
	         *wp = (float *)malloc(num * sizeof(float));
	         if (pind->autoteach == 1 || pind->modularity == 2)
	           *awp = (float *)malloc(num * sizeof(float));
	         if (*wp == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && *awp == NULL))
		   display_error("System error: wp malloc");		     
	        }
	   /* 
	    * allocate space for delta pointers
	    */
	   pind->deltap = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->autoteach == 1 || pind->modularity == 2)
	     pind->adeltap = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->deltap == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && pind->adeltap == NULL))
	      display_error("System error: deltap malloc");
	   /*
	    * allocate space for deltas
	    */
	   for (i=0, lp=pind->deltap, alp=pind->adeltap, ldp=pind->layer_descp; i<pind->nlayers; i++, lp++, alp++, ldp++) 
	     {
	      *lp = (float *)malloc(*ldp * sizeof(float));
	      if (pind->autoteach == 1 || pind->modularity == 2)
	        *alp = (float *)malloc(*ldp * sizeof(float));
	      if (*lp == NULL || ((pind->autoteach == 1 || pind->modularity == 2) && pind->adeltap == NULL)) 
	         display_error("System error: dp malloc");		  
	     }
	   /* 
	    * allocate space for wrules
	    */
	   pind->wrules = (int **) malloc(pind->nlayers * sizeof(int *));
	   if (pind->wrules == NULL)
	      display_error("System error: wrules malloc");
	   for (i=0, gpi=pind->wrules, ldp=pind->layer_descp; i<(pind->nlayers - 1); i++, gpi++, ldp++) 
	     {
	      num = *ldp * *(ldp + 1);
	      *gpi = (int *)malloc(num * sizeof(int));
	      if (*gpi == NULL) 
	         display_error("System error: wrules-gpi malloc");		  
	     }
	   /* 
	    * allocate space for wrates
	    */
	   pind->wrates = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->wrates == NULL)
	      display_error("System error: wrates malloc");
	   for (i=0, lp=pind->wrates, ldp=pind->layer_descp; i<(pind->nlayers - 1); i++, lp++, ldp++) 
	     {
	      num = *ldp * *(ldp + 1);
	      *lp = (float *)malloc(num * sizeof(float));
	      if (*lp == NULL) 
	         display_error("System error: wrates-lp malloc");		  
	     }
	   /*
	    * allocate space for brules
	    */
	   pind->brules = (int **) malloc(pind->nlayers * sizeof(int *));
	   if (pind->brules == NULL) 
	         display_error("System error: brules malloc");	      
	   for (i=0, gpi=pind->brules, ldp=pind->layer_descp; i<pind->nlayers; i++, gpi++, ldp++) 
	    {
	     *gpi = (int *)malloc(*ldp * sizeof(int));
	     if (*gpi == NULL)
	         display_error("System error: gpi-rules");
	    }
	   /*
	    * allocate space for brates
	    */
	   pind->brates = (float **) malloc(pind->nlayers * sizeof(float *));
	   if (pind->brates == NULL) 
	         display_error("System error: brates malloc");	      
	   for (i=0, bp=pind->brates, ldp=pind->layer_descp; i<pind->nlayers; i++, bp++, ldp++) 
	    {
	     *bp = (float *)malloc(*ldp * sizeof(float));
	     if (*bp == NULL)
	         display_error("System error: bp-rates");
	    }
       }

}


/*
 * randomize the genomes of the entire population
 */
void
randomize_pop()

{

        struct  individual *pind;	
	int     i;
	int     j;
	int     p;
	int	**gpi;
	int	*gi;
	
	for (p = 0, pind=ind, gpi=genome; p < npopulations; p++, pind++)
	 {
	  for (i=0; i < (nindividuals + opp_from_prev); i++,gpi++) 
	        {
                 for (j=0,gi = *gpi; j < pind->numchars; j++,gi++)
		    {
		     *gi = mrand(256);
		    }
	        }
	}


}


/*
 * 
void
identical_pop()

{

        struct  individual *pind;	
	int     i;
	int     j;
	int     p;
	int	**gpi;
	int	*gi;
	int     **gpi2;
	int     *gi2;
	
	for (p = 0, pind=ind, gpi=genome, gpi2=genome; p < npopulations; p++, pind++)
	 {
	  for (i=0; i < (nindividuals + opp_from_prev); i++) 
	        {
                 for (j=0,gi = *gpi, gi2 = *gpi2; j < pind->numchars; j++,gi++,gi2++)
		    {
		     if (p == 1)
		       *gi = *gi2;
		    }
		 gpi++;
		 if (p == 1)
		   gpi2++;
	        }
	}


}
*/



/*
 * allocate space for sampled objects
 */ 
void
allo_space()

{
    extern	char  *malloc();
    int 	**wa;
    int		**ob;
    float	**mo;
    int         *v;
    int         i;
    int         ii;

     /*
     * wall
     * allocate space for 30 distances for each sensor
     */
    wall = (int **) malloc(30 * sizeof(int *));
    wallcf = (int *) malloc(5 * sizeof(int));

    if (wall == NULL || wallcf == NULL) 
	 display_error("system error: wall malloc");

    /*
     * allocate space for directions
     */
    wa = wall;
    for (i=0; i<30; i++,wa++)
	  {
	   *wa = (int *)malloc((8 * 180) * sizeof(int));
	   if (*wa == NULL) 
	      display_error("system error: wa malloc");
	   for (ii=0,v=*wa; ii < (8 *180); ii++,v++)
		*v = 0;
	  }

    /*
     * round obstacles
     * allocate space for 30 distances for each sensor
     */
     obst = (int **) malloc(30 * sizeof(int *));
     obstcf = (int *) malloc(5 * sizeof(int));

     if (obst == NULL || obstcf == NULL)
	  display_error("system error: obst malloc");	       	 
      /*
       * allocate space for directions
       */
      ob = obst;
      for (i=0; i<30; i++,ob++)
	  {
	   *ob = (int *)malloc((8 * 180) * sizeof(int));
	   if (*ob == NULL) 
	      display_error("system error: ob malloc");	 
	   for (ii=0,v=*ob; ii < (8 *180); ii++,v++)
		*v = 0;	   
	  }
    /*
     * small round obstacles
     * allocate space for 30 distances for each sensor
     */
     sobst = (int **) malloc(30 * sizeof(int *));
     sobstcf = (int *) malloc(5 * sizeof(int));

     if (sobst == NULL || sobstcf == NULL) 
	 display_error("system error: sobst malloc");	       	 	      
      /*
       * allocate space for directions
       */
      ob = sobst;
      for (i=0; i<30; i++,ob++)
	  {
	   *ob = (int *)malloc((8 * 180) * sizeof(int));
	   if (*ob == NULL) 
	     display_error("system error: ob malloc");
	   for (ii=0,v=*ob; ii < (8 *180); ii++,v++)
		*v = 0;	   
	  } 
    /*
     * light
     * allocate space for 30 distances for each sensor
     */
    light = (int **) malloc(30 * sizeof(int *));
    lightcf = (int *) malloc(5 * sizeof(int));

    if (light == NULL || lightcf == NULL) 
	 display_error("system error: light malloc");
      /*
       * allocate space for directions
       */
    ob = light;
    for (i=0; i<30; i++,ob++)
	  {
	   *ob = (int *)malloc((8 * 180) * sizeof(int));
	   if (*ob == NULL) 
	    display_error("system error: light malloc");
	   for (ii=0,v=*ob; ii < (8 *180); ii++,v++)
		*v = 0;	   
	  }
    /*
     * allocate space for 25(v1) different speeds
     */
    motor = (float **) malloc(25 * sizeof(float *));
    if (motor == NULL) 
	    display_error("system error: motor malloc");	       
      /*
       * allocate space for 25(v1) different speeds (ang & dist)
       */
    mo = motor;
    for (i=0; i<25; i++,mo++)
	  {
	   *mo = (float *)malloc((25 * 2) * sizeof(float));
	   if (*mo == NULL) 
	    display_error("system error: mo malloc");
	  }	  
	  
  
}
