/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>


/*
 * This source can be compiled on DOS (Borland C) and Unix Machines
 * In the case of Unix machined the UNIX swhich should be on
 * In addition, in the case of sun the SUN swhich should be set on
 */
/* #define UNIX 1 */
/* #define SUN 1 */

#define MAXGENERATIONS   2000     /* max number of generations */
#define MAXINDIVIDUALS   1000     /* max numer of individuals for each generation */


/**********************************************************************
 *                       global variables
 **********************************************************************/

/*
 * parameters
 */

extern int     seed;                  /* random seed                              */
extern float   rate;                  /* rate                                     */
extern int     amoebaepochs;          /*  number of epochs                        */
extern int     generations;           /* number of generations                    */
extern int     nkids;                 /* number of kids for father                */
extern int     bestfathers;           /* number of fathers that make kids         */
extern int     startgeneration;       /* start generation (>=0 means load weig.)  */
extern int     percmut;               /* percentage of mutations                  */
extern int     delfile;               /* save only the best N amoebas each gen.   */
extern int     notadaptive;           /* natural selection (i.e. random fitness)  */
extern float   sensornoise;           /* noise on infrared sensors                */
extern int     testsamep;             /* test, start from the same position       */
extern int     timelaps;              /*  timelaps x 100 milliseconds             */
extern int     timeadjust;            /*  overtime due to computation time        */
extern int     fullforward;           /*  full feed-forward net                   */
extern int     nobreak;               /*  no artificial break on epochs           */
extern int     reappears;             /*  object reappears in fron after grasping */
extern int     startx;                /*  graphics x upleft angle                 */
extern int     starty;                /*  graphics y upleft angle                 */
extern int     wendx;                 /*  end x coordinate of the world           */
extern int     userscale;             /*  graphics scale (user) 999=auto          */
extern float   scale;                 /*  graphics scale                          */
extern float   energydecrease;        /*  sensor lost each cycle                  */  
extern int     cpopulation;           /*  current population                      */
extern int     dpopulation;           /*  population to be displayed              */
extern int     masterturnament;       /*  master turnament                        */
extern int     save_error_each;       /*  save error each N. cycles               */
extern int     nolearn;               /*  do not learn                            */
extern int     stop_when_crash;       /*  we stop lifetime if a crash occurred    */
extern int     all_of_fame;           /*  we test against all previous generation */ 
extern int     nworlds;               /*  number of different environments        */
extern int     cworld;                /*  idn of current world			   */    
extern int     random_round;          /*  randomize round position                */
extern int     random_sround;         /*  randomize sround position               */
extern int     random_foodzone;       /*  randomize foodzones                     */
extern int     random_minwd;          /*  min distance from walls of randomize ob */
extern int     random_minod;          /*  min distance between randomize objects  */
extern int     startonfood;           /*  khepera starts on foodzone              */
extern int     startfrontal;          /*  odd and even indivdiduals start in front*/
extern int     startfarfrom;          /*  khepera start 10cm far from obstacles   */   
extern int     ball;                  /*  the environment contains a ball         */
extern float   ball_color;            /*  the color of the ball                   */
extern float   background_color;      /*  the background color                    */
extern int     graphsensors;          /*  display sensors activation              */
extern int     drawinputs;            /*  swiches of graphsensors                 */ 
extern int     drawhiddens;  
extern int     drawoutputs;  
extern int     drawerror;
extern int     drawperceived;
extern int     drawfitness;
extern int     drawteaching;
extern int     drawindividual;        /* display the individual and the environment */
extern int     drawtrace;             /* swiches of drawindividual                  */ 
extern int     drawindif;
extern int     drawindlight;
extern int     drawcamera;
extern int     drawallepochs;         /* display different epochs in different positions */
extern int     drawallpopulation;     /* display all population                          */
extern int     graphfrequence;        /*  display frequence in self-sel maps             */
extern int     drawselfselmaps;       /*  draw self-sel maps while moving                */
extern int     drawsampleob;          /* sample object to be displayed                   */
extern int     sampleob;              /* object to be sampled                            */
extern int     sampleturn;            /* turn of the object to be sampled                */
extern int     samplenturns;          /* turns of the new sample object                  */ 
extern int     sampleinitdist;        /* start dist of the new sampled object            */
extern int     sampleinterdist;       /* interval dist of the new sample object          */
extern int     samplesave;            /* save the sample object                          */
extern int     black_and_white;       /* no colors                                       */
extern int     drawdata;              /* draw fitness and statistics                     */
extern int     draweights;            /* draw weights                                    */
extern int     sim_real;              /* 0 = simulato 1 = real robot                     */
extern int     alternate_wall;        /* alternate wall1.sam and wall2.sam each gen.     */
extern int     opptype;               /* opponent from 0=current gen, 1=best prev gene, 2= same pop */
extern int     oppid;                 /* opponent id number */
extern int     serialport;            /* serial port                                     */
extern int     serialspeed;           /* communication speed, can be 9600,19200,38400    */
			       
/*
 * file names
 */

extern  char    fileroot[];          /* config file name                         */
extern  char    worldfiles[];        /* the name of the first world file         */



/*
 * the population and the individual phenotype
 * and the environments
 */

extern struct  individual *ind;       /* individual phenotype                   */
extern int     **genome;	      /* genome of the population               */
extern int     **bestgenome;	      /* genome of the best individuals         */
extern struct  environment env[5];    /* environments                           */

/*
 * the individual phenotype, the environments
 */

extern float **motor;              /* 12 rows each with 12*2(ang and dist) columnsi*/
extern int   **wall;               /* wall (180 degrees and 38 distances)          */
extern int   **obst;               /* round obstacle (180 d. and 38 distances      */
extern int   **sobst;              /* round obstacle (180 d. and 20 distances      */
extern int   **ballobst;           /* ball (180 a. 20 distances                    */
extern int   **light;              /* light (180 degrees and 27 distances)         */
extern int   *wallcf;              /* wall  configuration                          */
extern int   *obstcf;              /* obst  configuration                          */
extern int   *sobstcf;             /* sobst configuration                          */
extern int   *lightcf;             /* light configuration                          */
extern int   *ballobstcf;          /* ball  configuration                          */
extern int   **objtosample;        /* just a pointer to the object to sample       */
extern int   *objtosamplecf;       /* just a ponter to the object to sample conf   */


extern float   input[];
extern float   flx,fly;            /* arm coordinate                               */
extern float   frx,fry;            /*      "                                       */
extern float   blx,bly;            /*      "                                       */
extern float   brx,bry;            /*      "                                       */
extern float   obx,oby;            /* object position when taken                   */
extern float   lgripper[4];        /* left  gripper coordinate                     */
extern float   rgripper[4];        /* right gripper coordinate                     */
extern float   p_gripper;          /* gripper position                             */
extern float   ob_sensed;          /* the object is between the gripper            */
extern double  ldist;              /* dist of the object in the hand               */
extern double  rdist;
extern double  fdist;
extern float   ob_fitness;         /* fitness gathered by the object               */
extern double  vdirection;         /* kepera virtual direction                     */
extern double  P90;
extern double  P180;
extern double  P360;
extern float   virtual_camera[360];

/*
 * others
 */

extern int     nindividuals;           /* number of individuals                  */
extern int     npopulations;           /* number of populations                  */
extern long int sweeps;                /* number of life sweeps                  */
extern long int run;                   /* learning cycle counter                 */
extern double  tfitness[2][MAXINDIVIDUALS]; /*  population fitness                    */
extern double  statfit[MAXGENERATIONS][4]; /*  best and mean fit for each gen. & pop */  
extern double  astatfit[MAXGENERATIONS][4];/*  statfit averaged for replications */
extern float   ave_mot;                /* average motors                         */
extern float   dif_m;                  /* differential motors                    */
extern float   maxs;                   /* max activated sensor                   */
extern int     explored[100][100];     /* explored cells                         */
extern int     oselected;              /* selected output                        */
extern int     ncrashes1;              /* n of crashes with ob_taken=0           */
extern int     ncrashes2;              /* n of crashes with ob_taken=1           */
extern int     ntaken;                 /* n of object taken                      */
extern int     nreleased;              /* n of object released                   */
extern int     radarmap[2][4];         /* old and current radar position         */
extern int     o_perceived[];          /* perceived objects wall/round/sround/others    */
extern int     individual_crash;       /* a crash occurred between two individuals */
extern int     opp_from_prev;          /* opponent taken from previous x generations */
extern double  oppangle;               /* the angle of the opponent robot         */
extern double  oppdist;                /* the distance of the opponent robot      */
extern int     create1;                /* if pop structure have been created      */
extern int     create2;                /* if pop space has been allocated         */
extern int     cfloaded;               /* 1 if .cf file has been loaded           */
extern int     parmodified;            /* if vital parameters has been modified   */
extern int     createw2;               /* if secondary window has been created    */
extern int     cwindow;                /* current window                          */
extern int     cwindowdx;              /* current dx dimension of the window      */
extern int     cwindowdy;              /* current dy dimension of the window      */
extern int     userbreak;              /* user want to stop what's going on       */
extern int     oldstylewts;            /* old style weights (without end)         */
extern int     cepoch;                 /* current epoch                           */
extern int     testindividual;         /* test one individual                     */
extern int     run_evolution;          /* run evolution                           */
extern int     run_masterturnament;    /* run masterturnament                     */
extern int     run_testsc;             /* test serial connection                  */
extern int     nreplications;          /* n replication seeds                     */
extern double  savedata[];             /* vector of extern double used by save_data      */
extern int     fitmode;                /* if 1 we divide the total fitness for lifecycles */
extern int     noturret1;              /* if 1, turret 1 not installed            */
extern int     noturret2;              /* if 2, turret 2 not installed            */

/**********************************************************************
 *                       structures
 **********************************************************************/

struct individual

{
  /* network structure */		
  int	nlayers;				/* number of layers in network              */
  int	*layer_descp;			   	/* no. of units in each layer               */
  int	numweights;				/* total number of weights                  */
  int   nhiddens;                               /* number of hidden units                   */
  int	numbiases;				/* total number of learnable biases         */
  int   numchars;				/* number of chars needed for genotypes     */
  float	**layerp;				/* the layers themselves                    */
  float **deltap;				/* delta for each node, each layer          */
  float	**weightp;				/* weights between layers                   */
  float	**biasp;				/* biases for each node, each layer         */
  int   **wrules;                               /* learning rules for each connection       */
  float **wrates;                               /* learning rates for each connection       */ 
  int   **brules;                               /* learing rules for each bias              */
  float **brates;                               /* learning rates for each bias             */
  float	**alayerp;				/* the layers themselves (autoteach)        */
  float **adeltap;				/* delta for each node,  (autoteach)        */
  float	**aweightp;				/* weights between layers (autoteach)       */
  float	**abiasp;				/* biases for each node, each layer (aut.   */
  float **playerp;                              /* previous layerp (bp through time)        */
  float wrange;                                 /* weights range                            */
  float amoebat[50];                            /* teaching inputs                          */
  int   autoteach;				/* if 1 use autoteaching                    */
  int   modularity;				/* 1=two blocks 2=emergent mod.             */
  int   modulskip;				/* outputs to skip with modularity = 1      */
 
  /* type of sensors */
  int     nifsensors;				/* number of infrared sensors               */
  int     motmemory;				/* copy motor activation on sensors         */
  int     groundsensor;				/* ground sensor                            */
  int     objectsensor;				/* gripper infrared sensor                  */
  int     camerafive;				/* camera 5 ranges                          */
  int     newcamerafive;                        /* camera 5 (new camera with 220 degrees)   */
  int     lightsensors;                         /* light sensors of the opponent's light    */
  int     elman_net;				/* elman network sensors                    */
  int     oppsensor;                            /* sesor of the opponent dist               */
  int     cameraunits;				/* the number of visual sectors and corresponding units */
  int     camerarange;				/* the angular range of the camera (default is 36) */
  int     battery;                              /* battery level                            */
  int     light2sensors;                        /* light a frontal and backward sensor      */
  int     input_outputs;                        /* additional output copied back into inputs*/
  float	  virtual_camera[360];                  /* a virtual camera with a 360 degree view  */
  float   real_camera[32];                      /* a real camera with a maximum of 32 receptors*/
  int     pixminint;                            /* the pixel of the camera with the minimal intensity */
  double  pixcvar;                              /* camera countours variation               */
  double  camerastdev;                          /* camera standard deviation                */


  /* type of motors */
  int     wheelmotors;                          /* presence of wheel motors                 */
  int     gripper;				/* presence of gripper                      */

  /* motors */
  float   pos[2];				/* kepera position (center of the body)     */
  double  direction;				/* kepera direction                         */
  float   oldpos[2];    
  double  oldirection;
  float   speed[2];				/* actual speed                             */

  /* current state */

  int     ob_taken;				/*  the object is gripped                   */
  float   p_arm;                                /*  arm position                            */
  float   lightbarsensor;                       /*  light barrier sensor on the gripper     */
  int     ffitness;				/*  fitness formula                         */
  double  fitness;				/*  current fitness                         */
  double  oldfitness;				/*  fitness up to the last epoch            */
  double  oldcfitness;				/*  fitness up to the last cycle            */
  float   energy;				/*  energy level                            */
  float   global_error;                         /*  current error                           */              
  double  error_sum;                            /*  actual sum of errors                    */
  long int error_n;                             /*  n. of cycle from last reset             */
  double  perror;                               /*  error in the preceeding epoch           */
  float   rate;                                 /*  learning rate                           */
  int     population;                           /*  current population                      */
  int     learning;				/*  lifetime learning                       */
  int     lifecycle;				/*  lifecycle (within the current epoch)    */                    
  int     totcycles;                            /*  total lifecycles                        */
  int     randomweights;                        /*  it start from random weights            */
  int     dindividual;                          /*  the individual to be displayed          */
  int     hebbian_learning;                     /*  hebbian learning                        */
  float   cifsensors[8];                        /*  infrared sensors                        */                          
  float   clightsensors[8];                     /*  light sensors                           */ 
  float   bottomsensor;                         /*  sensor placed on the bottom of khepera  */

  /* parameters */
  int     fixed;                                /*  a population which do not evolve        */
  int     samepop;                              /*  the individual came from the same pop   */ 
};


struct environment

{

int   dx,dy;                /* world charthesian extension                  */
float obstacles[20][4];     /* wall obstacles (x1,y1,x2,y2)                 */
float radius;               /* radius of round obstacles                    */
float sradius;              /* radius of small round obstacles              */
float roundobst[20][2];     /* center of round obstacles (x,y)              */
float sroundobst[20][3];    /* center of small obstacles (x,y,taken)        */
float xtarget[5];           /* targets to remove or to display              */
float foodzones[20][3];     /* foodzone (x,y,r)                             */
float bulblights[20][2];    /* bulblights (x,y)                             */
float starts[50][5];        /* initial coo.(kepera's xyd and foodz'xy)      */
float landmark[20][6];      /* ladmarks: [0,1] center, [2] radiuns,         */
                            /* [3] cilind/oriz-surf/vert-surf,[4] color     */                    
int   nstarts;              /* fixed starts                                 */
int   nobstacles;           /* number of wall obstacles                     */
int   nroundobst;           /* number of round obstacles                    */
int   nsroundobst;          /* number of round obstacles                    */
int   nfoodzones;           /* number of food zones                         */
int   nbulblights;          /* number of bulb lights                        */
int   nlandmarks;           /* number of landmarks                          */

};

/**********************************************************************
 *                       functions
 **********************************************************************/

/* activate.c */
float logistic(float f);
void act(float *b_actp, int b_size, float *t_actp, int t_size, float *t_weightp, float *t_biasp);
void activate_net(struct individual *pind);
void auto_activate_net(struct individual *pind);
void find_best_match(struct individual *pind, int nvalues, float *values);
void activate_kohonen(struct  individual *pind);
void activate_cameranet();
/* config.c */
int  getword(char *word, char *line, char stop);
void clear_from_space(char *stringa);
int  load_parameters(char *fileroot);
void save_parameters(char *filename);
void init_parameters();
void compute_parameters();
void create_poppointer();
void create_pop();
void randomize_pop();
void   allo_space();
/* generati.c */
void select_besti();
void individual(int gen, int ind, int pop, int opp);
void reproduce(int g);
void generati();
/* graphics.c */
void draw_world();
void draw_ind_sensors(struct individual *pind,float x, float y);
void draw_ind_camera(struct individual *pind);
void draw_ind_arm(float x,float y,float arm,float grip,double direction,int ob_taken, float scalearm);
void draw_ind(int cycle);
void draw_statistics(int g, int startg);
void draw_data(struct individual *pind);
void draw_sensors(struct individual *pind);
void draw_weights(struct individual *pind);
void draw_io_maps();
void draw_all_arrows();
void draw_selfsel_maps(struct individual *pind, int cycle);
void draw_map(char *filename);
void draw_real_robot(struct individual *pind);
void draw_sample(int **object, int  *objectcf);
void sample_object(int **object, int distances);
/* interface.c */
void display_error(char *emessage);
void display_warning(char *emessage);
void display_double(double val);
void save_data(char *filen, int n, int d);
// please note that other functions are declared in the file itself
/* learning.c */
float hebb(float x, float y, float w);
float presynaptic(float x, float y, float w);
float postsynaptic(float x, float y, float w);
float ccov(float x, float y, float w);
void  hebbian_update(struct individual *pind);
float delta_out(float *outp, int outsize, float *tarp, float *outdeltap, int  ninputs, float *inpdeltap);
void  delta(int t_size, float *t_deltap, float *t_weightp, float *b_actp, int b_size, float *b_deltap);
void  upd_weight(int t_size, float *t_deltap, float *t_weightp, float *t_biasp, float *b_actp, int b_size);
void  update_net(struct individual *pind, float *teachp);
/* main.c */
void  init_khepsim();
void  get_str_word(char *st1, char *st2);
void  learn(struct individual *pind, long int cycle);
void  loop(int gen, int pop, int nind, int epoch);
float rans(double n);
int   mrand(int i);
/* real.c */
void  setcomm();
int   aprecom();
int   comandok(char tbuff[],char rbuff[]);
void  sample_turn(int turn, int **object, int *objectcf);
void  random_real_pos(int ncycles);
void  stop_real();
void  check_real_arm(struct individual *pind);
void  init_arm();
void  setpos_real(struct individual *pind,float o1, float o2);
void  set_input_real(struct individual *pind);
/* simulato.c */
void   SETP(); 
double RADtoDGR(double r);
double DGRtoRAD(double d); 
float  SetX(float x,float d,double angle);
float  SetY(float y,float d,double angle);
double linedist(float px,float py,float ax,float ay,float bx,float by);
double mdist(float x, float y, float x1, float y1);
double normang(double a);
double mang(int x, int y, int x1, int y1, double dist);
double mangrel(double absolute,double kepera);
double mangrel2(double absolute,double kepera);
void   init_world();
int    load_world();
void   save_world();
void   load_obstacle(char *filename, int **object, int *objectcf);
void   save_obstacle(char *filename, int **object, int  *objectcf);
void   load_motor();
float  oselect(struct individual *pind, int out);
void   set_input(struct individual *pind);
int    read_ground(struct individual *pind);
int    check_crash(float p1,float p2);
void   check_hand(int ta,float x,float y,float  grip, double direction);
float  check_arm(struct individual *pind);
void   setpos(struct individual *pind,float o1, float o2);
void   reset_pos_from_crash(struct individual *pind);
void   set_khepera_position(struct  individual *pind);
void   amoebastart(int gen,int pop, int nind,int epoch);
/* w32graph.c */
void kouttextxy(float x,float y, char *c);
void kouttextxy2(float x,float y, char *c);
void koutfloat(float x,float y, char *c, float f);
void kmessage(char *message);
void kputpixel(float x, float y, int c);
void kline(float x, float y,float x2, float y2);
void krectangle(float x, float y,float x2, float y2);
void kellipse(float x, float y,float dxx, float dyy);
void kcircle(float x, float y,float r);
void karrow(float x0,float y0,float x,float y,float rad);
void kbar(float x, float y,float x2, float y2);
void kfillellipse(float x, float y,float dxx, float dyy);
void kfillcircle(float x, float y,float r);
//COLORREF kvgacolor(int c); declared within the file
void ksetfillstyle(int type, int c);
void ksetcolor(int c);
int  kcolor_scale(float val); 
void save_image(int n, float x1,float y1,float x2, float y2);
void restore_image(int n);
void kinit_graphics();  
void kclear_device();
void kclear_device_from(int x, int y);
void select_window(int n);
void kdim_device(); 
void set_scale(int deltax,int deltay);
void out_message(char *c);
/* weights.c */
int   parsegenfile(char *filename);
float getbits(int v, float range);
void  getlrule(int g, float *wt, int *rule, float *rate);
int   mutate(int w);
void  getgenome(int n, int p, int type);
void  saveanet(FILE *fp, int n, int p);
void  loadanet(FILE *fp, int n, int p, int mode);
void  saveallw(int gen);
void  loadallw(int gen, char *filew);
void  load_previous(int gen);
void  getbestweights(int n, int nn, int p);
void  putbestweights(int n, int nn, int p);
void  reset_net(struct individual *pind); 
void  loadstatistics(char *filew);
/* others standard */
//extern double exp();  
//extern  double atof();
//extern  int atoi();
//extern  char *malloc();
//FILE *fopen();
//extern double tanh();
//extern double fabs();
//extern double sqrt();
//extern int rand();
//extern double asin();
//extern double cos();
//extern int abs();
//extern int  strlen();
//extern int strcmp();



