/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * generati.c
 */

#include "defs.h"
#include "w32graph.h"


/*
 * select the best individual (on the basis of statfit)
 * place the results in pind->dindividual
 */
void
select_besti()

{

       struct  individual *pind;		
       int     g;
       int     p;
       double  bestfit;
       int     besti;

       for(p=0,pind=ind;p<npopulations;p++,pind++)
         {
	   bestfit = -9999.9;
	   besti   = -1;
	   for(g = 0; g < generations; g++)
	   {
	     if (statfit[g][0] == -9999.9)
	       break;
             if (statfit[g][0+(p*2)] > bestfit)
	      {
                 bestfit = statfit[g][0+(p*2)];
	         besti = g;
	      }
	   }
	   if (besti < (nindividuals + opp_from_prev))
	     pind->dindividual = besti;
	 }
}



/*
 * Decode genotype of individuals and let them live
 * if pred/prey, antagonist is taken from previous generations
 */
void
individual(int gen, int nind, int pop, int opp)

 {
 
       struct  individual *pind;		
       int i;
       int p;
       int opp_id;
       int opp_type;
       char   sbuffer[64];
       FILE   *fp;
       FILE   *fopen();


       // initialize epoch variables
       ncrashes1 = ncrashes2 = ntaken = nreleased = 0;
       for(p=0,pind=ind;p<npopulations;p++,pind++)
         {
           pind->fitness   = 0.0;
           pind->oldfitness= 0.0;
	   pind->totcycles = 0;
	   pind->perror  = 0.0;
	   getgenome(nind,p,0);
         }       
       pind = (ind + pop);


       // if masterturnament we save the fitness of each epoch in a file
       if (masterturnament == 1)
       {
	 sprintf(sbuffer,"masterP%dS%d.map",pop,seed);
	 if (nind == 0) 
	  {
           fp=fopen(sbuffer , "w");
	   fprintf(fp,"%d %d\n",nindividuals,nindividuals);
	  }
	  else
           fp=fopen(sbuffer , "a");
	 if (fp == NULL)
	       display_error("unable to save master.mas data");
       }

       // we run the epoch cycle
       for (i=0;i<amoebaepochs;i++)
	     {
	       cepoch = i;
	       // we set the scale  
	       if (drawindividual == 1 && drawallepochs == 1)
	       {
                 kdim_device();
		 if (cepoch == 0)
		 {
		   startx = (int) (50.0 * scale);
		   starty = (int) (50.0 * scale);
		 }
		 else
		 {
		  startx += (int) (((float) 50.0 + (float) env[cworld].dx) * scale);
		  if (startx + ((float) env[cworld].dx * scale) > cwindowdx)
		   { 
 		    startx =  (int) ((float) 50.0 * scale);
		    starty += (int) (((float) 50.0 + (float) env[cworld].dy) * scale);
		   }
		  if (starty + ((float) env[cworld].dy * scale) > cwindowdy)
		   {
 		    startx =  (int) ((float) 50.0 * scale);
 		    starty =  (int) ((float) 50.0 * scale);
		   }
		 }
	        }

               /* 
	        * if two populations, we select the opponent
		* (id_type 0 from current, 1 from the best of previous generations, 2 from previous of the population 0)
	        */
	       if (npopulations == 2)
	       {
	        if (opp_from_prev > 0)
	        {
		 // we select the best opponent of [cepoch] generation before (type = 1)
		 // if we run only few generations (< cepoch) we select from a randomly selected best of previous generations (type=1)
		 // if generation 0 we select one randomly from the current generation (type=0)
		 opp_type = 1;
	         if ((cepoch >= gen || all_of_fame == 1) && gen > 0)
		   opp_id = mrand(gen);
		  else
	           opp_id = cepoch;
		 if (gen == 0)
		  {
		   opp_id   = mrand(nindividuals);
		   opp_type = 0;
		  }
	        }
		// if masterturnament we select the [cepoch] opponent of the current generation
	        if (masterturnament == 1)
	        {
		  opp_type = 0;
		  opp_id   = cepoch;
	        }
		// if fixed, we use the individual 0 from current population
                if (pind->fixed == 1)
		  {
	            opp_id   = 0;
		    opp_type = 0;
		  }
		// if the user call this function with test-individual we use the 
		// user's specified individual (pind->dindividual)
		// negative numbers are interpreted as individuals from previous bests
	        if (opp != -999)
	        {
		  if (opp >= 0)
		   {
                    opp_type = 0;
		    opp_id   = opp;
		   }
		   else
		   {
	             opp_type = 1;
		     opp_id = (0 - opp);
		   }
		   if ((opp_type == 0 && opp_id >= nindividuals) || (opp_type == 1 && opp_id >= opp_from_prev))
		    {
		      opp_type = 0;
		      opp_id = 0;
		    }
		}
		// check for possible errors
		if (opp_type == 0 && opp_id >= nindividuals)
		  display_error("no enough opponents in the current generation");
		if (opp_type == 1 && opp_id >= opp_from_prev)
		  display_error("no enough opponents from previous generatios specified");
                for(p=0,pind=ind;p<npopulations;p++,pind++)
                 {
		   if (p != pop)
		     {
	              getgenome(opp_id,p,opp_type);
		      opptype = opp_type;
		      oppid   = opp_id;
		     }
	         }
	       }
		
	       // we test the individual for one epoch
               pind = (ind + pop);

	       loop(gen,pop,nind,i);

	       // we change world if we have more than one
	       if (nworlds > 1)
	       {
		 cworld++;
		 if (cworld >= nworlds)
		   cworld = 0;
	       }

	       if (userbreak == 1)
                  i = amoebaepochs;
              
	       if (masterturnament == 1)
		 fprintf(fp,"%.2f ", pind->fitness - pind->oldfitness);
	     }

         if (masterturnament == 1)
	  {
	   fprintf(fp,"\n");
           fclose(fp);
	  }
	     
         // we store fitness into the tfitness vector
         for(p=0,pind=ind;p<npopulations;p++,pind++)
          {
            if (p == pop) 
	      if (fitmode == 1)
                tfitness[p][nind] = pind->fitness / (double) (sweeps * amoebaepochs);
	       else
	        if (fitmode == 2)
                  tfitness[p][nind] = pind->fitness / amoebaepochs;
                 else
                  tfitness[p][nind] = pind->fitness;
          }      



}


/*
 * produce a new generations
 * if last generations it saves the best but does not generate the new population
 */
void
reproduce(int g)

 {



       struct  individual *pind;     
       char sroot[64];
       char sbuffer[64];
       FILE *fp;
       int p;
       int f;
       int x;
       int n;
       int k;
       float v;

   
       for(p = 0, pind = ind; p < npopulations; p++, pind++)
        {
	 if (pind->fixed == 0 && pind->samepop == 0)
          for(f=0;f<bestfathers;f++)
	     {
	      x = 0;
	      v = (float) -9999.0;
	      for(n = 0;n < nindividuals;n++)
	       if (tfitness[p][n] >= v)
	       {
		x = n;
		v = (float) tfitness[p][n];
	       }
              putbestweights(x,f,p);	       
	      tfitness[p][x] = -9999.0;
	      // if we restarted evolution we do not save the bests of the initial generation
	      if ((f+1) <= delfile && (startgeneration == 0 || g > startgeneration))
		{
		 sprintf(sbuffer,"B%dP%dS%d.gen",f+1,p,seed);
		 if (g == 0)
		  {
		   if ((fp=fopen(sbuffer, "w")) == NULL)
		    {
		     sprintf(sroot,"I cannot open file super%d_%d",p,f+1);
	             display_error(sroot);
		    }
		  }
		  else
		  {
		   if ((fp=fopen(sbuffer, "a")) == NULL)
		    {
		     sprintf(sroot,"I cannot open file B%dP%dS%d.gen",f+1,p,seed);
	             display_error(sroot);
		    }
		  }
		 fprintf(fp,"**NET : s%d_%d.wts\n",g,x);
		 saveanet(fp,x,p);
		 fflush(fp);
		 fclose(fp);
		}
	      }
          }
       if (g+1 < generations)
       {
        for(p=0, pind = ind; p < npopulations; p++, pind++)
        {
	 if (pind->fixed == 0 && pind->samepop == 0)
	   for(f=0;f<bestfathers;f++)
	     {
	      for (k=0;k<nkids;k++)
		{
                 getbestweights(k+(f*nkids),f,p);		    
		}
	     }
	}
       }

}




/*
 * Load starting generation, select and reproduce
 */
void
generati()

 {

       struct  individual *pind;              
       int    n,g,p,nn,pp;
       double bestfit,avefit;
       char   sbuffer[64];
       FILE   *fp;
       FILE   *fopen();

       for(g=startgeneration;g<generations;g++)
	{
	 /*
	  * if we alternate walls we load wall1.sam and wall2.sam
	  * in odd and even generations
	  */
	 if (alternate_wall == 1)
	   if ((g % 2) == 0)
	    {
	     kmessage("we load wall1.sam");
	     load_obstacle("wall1.sam",   wall,     wallcf);
	    }
	    else
	    {
	     kmessage("we load wall2.sam");
	     load_obstacle("wall2.sam",   wall,     wallcf);
	    }
	 for (p=0,pind=ind;p<npopulations;p++,pind++)
	 {
	  /* 
	   * if a population is fixed 
	   * we do not test individuals and we assigne
	   * them the same fitness of the corresponding 
	   * individual of the first population
	   */
	  if (pind->fixed == 0 && pind->samepop == 0)
	   {
	    for(n=0;n<nindividuals;n++)
	     {
	       if (drawallpopulation == 1 && drawindividual == 1)
	       {
		 if (n == 0)
		 {
		   startx = (int) (50.0f * scale);
		   starty = (int) (50.0f * scale);
		 }
		 else
		 {
                  kdim_device();
		  startx += (int) (((float) 50.0 + (float) env[cworld].dx) * scale);
		  if (startx + ((float) env[cworld].dx * scale) > cwindowdx)
		   { 
 		    startx =  (int) ((float) 50.0 * scale);
		    starty += (int) (((float) 50.0 + (float) env[cworld].dy) * scale);
		   }
		  if (starty + ((float) env[cworld].dy * scale) > cwindowdy)
		   {
 		    startx = (int) ((float) 50.0 * scale);
 		    starty = (int) ((float) 50.0 * scale);
		   }
		  }
	       }
	      individual(g,n,p,-999);
	      if (userbreak == 1)
		    break;
	      if (notadaptive ==1)
	        tfitness[p][n] = mrand(100);

	      if (masterturnament == 1 && drawindividual == 0)
	       {
                  statfit[n][0+p*2] = tfitness[p][n];
                  statfit[n][1+p*2] = 0.0;
                  astatfit[n][0+p*2] += tfitness[p][n];
                  astatfit[n][1+p*2] += 0.0;
                  if (p == 0)
		   {
                    statfit[n][2] = 0.0;
                    statfit[n][3] = 0.0;
		   }
	          draw_statistics(n, 0);
	       }

	      sprintf(sbuffer, "Seed %d Pop %d Gen.%d  Ind.%d  Fit.%.3f",seed,p,g,n,tfitness[p][n]);
	      kmessage(sbuffer);
	     }
	   }
	   else // if fixed
	   {
	    for(n=0;n<nindividuals;n++)
	         tfitness[p][n] = tfitness[p][0];
	   }

	 } // end for p<npopulations

	 /* we save fitness of each individual in a .txt file*/
         if (masterturnament == 0)
	  {
	   for(n = 0;n < npopulations; n++)
	    {
	     sprintf(sbuffer,"FitP%dS%d.txt",n,seed);
	     if (g == 0)
	       fp=fopen(sbuffer , "w");
              else
	       fp=fopen(sbuffer , "a");
	     if (fp == NULL)
	       display_error("unable to save individual fitnesses on a file");
	     for(nn = 0; nn < nindividuals; nn++)
	       fprintf(fp,"%.2f ",tfitness[n][nn]);
	     fprintf(fp,"\n");
	     fflush(fp);
	     fclose(fp);
	    }
	   /* we save statistics in a .fit file */ 
	   sprintf(sbuffer,"statS%d.fit",seed);
	   if (g == 0)
	       fp=fopen(sbuffer , "w");
	      else
	       fp=fopen(sbuffer , "a");
	   if (fp == NULL)
	       display_error("unable to save statistics on a file");
	   for(pp = 0;pp < npopulations; pp++)
	      {
	        bestfit = -99990.0;
	        avefit  = 0.0;
	        for(n = 0;n < nindividuals;n++)
	        {
	         avefit += tfitness[pp][n];
	         if (tfitness[pp][n] >= bestfit)
		    bestfit = tfitness[pp][n];
	        }
               avefit = avefit / (double) nindividuals;
	       if (userbreak == 0)
	         fprintf(fp,"%.3f %.3f ",bestfit,avefit);	  
	       if (g < MAXGENERATIONS)
	        {
                 statfit[g][0+pp*2] = bestfit;
                 statfit[g][1+pp*2] = avefit;
                 astatfit[g][0+pp*2] += bestfit;
                 astatfit[g][1+pp*2] += avefit;
                 statfit[g+1][0] = -9999.0;
                 if (userbreak == 0 && drawindividual == 0)
	           draw_statistics(g, 0); // startgeneration always 0
	        }
	      }
	   fprintf(fp,"\n");
	   fflush(fp);
	   fclose(fp);
	  }
          else
	  // masterturnament (summary)
	  // we save the total fitness of each individual in a fit file
	  {
	     sprintf(sbuffer,"MasterS%d.fit",seed);
	     fp=fopen(sbuffer , "w");
	     if (fp == NULL)
	        display_error("unable to save master turnament data on a file");
	     for(nn = 0;nn < nindividuals; nn++)
	      {
	       for(n = 0;n < npopulations; n++)
	          fprintf(fp,"%.2f %.2f ",tfitness[n][nn], 0.0f);
	       fprintf(fp,"\n");
	       fflush(fp);
	      }
	     fclose(fp);
	   }
          
	  // eventually we save the current generation
	  if (g == 0 || userbreak == 1 || (g>0 && (delfile==0 || g+1 == generations)))
	  {
	   saveallw(g);
	  }

	 if (userbreak == 1)
		 break;

	 if (g < generations && masterturnament == 0)
           reproduce(g);
	 	 
	 // we load opp_from_prev
	 if (npopulations == 2 && (opp_from_prev > 0) && ((g+1) < generations))
            load_previous(g);
      } // end generations

}
	 
