/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * globals.c
 */

#include "defs.h"

/*
 * parameters
 */

int	seed=0;                /* random seed                              */
float   rate = (float) 0.2;    /* rate                                     */
int     amoebaepochs=0;        /*  number of epochs                        */
int     generations=0;         /* number of generations                    */
int     nkids=0;               /* number of kids for father                */
int     bestfathers=0;         /* number of fathers that make kids         */
int     startgeneration = 0;   /* start generation (>=0 means load weig.)  */
int     percmut=2;             /* percentage of mutations                  */
int     delfile=1;             /* save only the best N individual each gen.   */
int     notadaptive=0;         /* natural selection (i.e. random fitness)  */
float   sensornoise=(float)0.0;/* noise on infrared sensors                */
int     testsamep=0;           /* test, start from the same position       */
int     timelaps=1;            /*  timelaps x 100 milliseconds             */
int     timeadjust=0;          /*  overtime due to computation time        */
int     fullforward=0;         /*  full feed-forward net                   */
int     nobreak=0;             /*  no artificial break on epochs           */
int     reappears=0;           /*  object reappears in fron after grasping */
int     startx= 50;            /*  graphics x upleft angle                 */
int     starty= 50;            /*  graphics y upleft angle                 */
int     wendx=0;               /*  end x coordinate of the world           */
int     userscale=999;         /*  graphics scale (user) 999=auto          */
float   scale = (float) 1.0;   /*  graphics scale                          */
float   energydecrease=0.01f;  /*  sensor lost each cycle                  */  
int     cpopulation;           /*  current population                      */
int     dpopulation=0;         /*  population to be displayed              */
int     masterturnament=0;     /*  master turnament                        */
int     save_error_each=0;     /*  save error each N. cycles               */
int     nolearn=0;             /*  do not learn                            */
int     stop_when_crash=0;     /*  we stop lifetime if a crash occurred    */
int     all_of_fame=0;         /*  we test against all previous generation */ 
int     nworlds=1;             /*  number of different environments        */
int     cworld=0;              /*  idn of current world			   */    
int     random_round=0;        /*  randomize round position                */
int     random_sround=0;       /*  randomize sround position               */
int     random_foodzone=0;     /*  randomize foodzones                     */
int     random_minwd=125;      /*  min distance from walls of randomize ob */
int     random_minod=125;      /*  min distance between randomize objects  */
int     startonfood=0;         /*  khepera starts on foodzone              */
int     startfrontal=0;        /*  odd and even indivdiduals start in front*/
int     startfarfrom=0;        /*  khepera start 10cm far from obstacles   */   
int     ball=0;                /*  the environment contains a ball         */
float   ball_color=(float)1.0; /*  the color of the ball                   */
float   background_color=(float)0.0;  /*  the background color                    */
int     graphsensors=0;        /*  display sensors activation              */
int     drawinputs=0;          /*  swiches of graphsensors                 */ 
int     drawhiddens=0;
int     drawoutputs=0;
int     drawerror=0;
int     drawperceived=0;
int     drawfitness=0;
int     drawteaching=0;
int     drawindividual = 0;    /* display the individual and the environment */
int     drawtrace=0;           /* swiches of drawindividual                  */ 
int     drawindif=0;
int     drawindlight=0;
int     drawcamera=0;
int     drawallepochs=0;       /* display different epochs in different positions */
int     drawallpopulation=0;   /* display all population                          */
int     graphfrequence=0;      /*  display frequence in self-sel maps             */
int     drawselfselmaps=0;     /*  draw self-sel maps while moving                */
int     drawsampleob=1;        /* sample object to be displayed                   */
int     sampleob=0;            /* object to be sampled                            */
int     sampleturn=0;          /* turn of the object to be sampled                */
int     samplesave=0;          /* save the sample object                          */
int     samplenturns=20;       /* turns of the new sample object                  */ 
int     sampleinitdist=5;      /* start dist of the new sampled object            */
int     sampleinterdist=2;     /* interval dist of the new sample object          */
int     black_and_white=0;     /* no colors                                       */
int     drawdata=0;            /* draw fitness and statistics                     */
int     draweights=0;          /* draw weights                                    */
int     sim_real=0;            /* 0 = simulato 1 = real robot                     */
int     alternate_wall=0;      /* alternate wall1.sam and wall2.sam each gen.     */
int     serialport=2;          /* serial port                                     */
int     serialspeed=38400;     /* communication speed, can be 9600,19200,38400    */

			       
/*
 * file names
 */

char    fileroot[1024];         /* config file name                         */
char    worldfiles[1024];       /* the name of the first world file         */



/*
 * the population and the individual phenotype
 * and the environments
 */

struct  individual *ind=NULL;   /* individual phenotype                   */
int	**genome;		/* genome of the population               */
int	**bestgenome;		/* genome of the best individuals         */
struct  environment env[5];     /* environments                           */

/*
 * others
 */

int     nindividuals;           /* number of individuals                  */
int     npopulations;           /* number of populations                  */
long int sweeps = 0;            /* number of life sweeps                  */
long int run = 0;               /* learning cycle counter                 */
double  tfitness[2][MAXINDIVIDUALS];/*  populations fitness                   */
double  statfit[MAXGENERATIONS][4]; /*  best and mean fit for each gen. & pop */  
double  astatfit[MAXGENERATIONS][4];/*  statfit averaged for replications */
float   ave_mot;                /* average motors                         */
float   dif_m;                  /* differential motors                    */
float   maxs;                   /* max activated sensor                   */
int     explored[100][100];     /* explored cells                         */
int     oselected;              /* selected output                        */
int     ncrashes1;              /* n of crashes with ob_taken=0           */
int     ncrashes2;              /* n of crashes with ob_taken=1           */
int     ntaken;                 /* n of object taken                      */
int     nreleased;              /* n of object released                   */
int     radarmap[2][4];         /* old and current radar position         */
int     o_perceived[4];         /* perceived objects wall/round/sround/others    */
int     individual_crash;       /* a crash occurred between two individuals */
int     opp_from_prev=0;        /* opponent taken from previous x generations */
double  oppangle;               /* the angle of the opponent robot         */
double  oppdist;                /* the distance of the opponent robot      */
int     create1=0;              /* if pop structure have been created      */
int     create2=0;              /* if pop space has been allocated         */
int     cfloaded=0;             /* 1 if .cf file has been loaded           */
int     parmodified=0;          /* if vital parameters has been modified   */
int     createw2=0;             /* if secondary window has been created    */
int     cwindow=0;              /* current window                          */
int     cwindowdx=0;            /* current dx dimension of the window      */
int     cwindowdy=0;            /* current dy dimension of the window      */
int     userbreak=0;            /* user want to stop what's going on       */
int     oldstylewts=0;          /* old style weights (without end)         */
int     cepoch;                 /* current epoch                           */
int     testindividual=0;       /* test one individual                     */
int     run_evolution=0;        /* run evolution                           */
int     run_masterturnament=0;  /* run masterturnament                     */
int     run_testsc=0;           /* test serial connection                  */
int     nreplications=0;        /* n replication seeds                     */
double  savedata[50];           /* vector of double used by save_data      */
int     fitmode=0;              /* 1=fitness/lifecycles 2=fitness/epochs   */
int     opptype=0;              /* opponent from 0=current gen, 1=best prev gene, 2= same pop */
int     oppid=0;                /* opponent id number */
int     noturret1=0;            /* if 1, turret 1 not installed            */
int     noturret2=0;            /* if 2, turret 2 not installed            */