/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/*
 * graphic.c
 */

#include "defs.h"
#include <windows.h>
#include <math.h>
#include "w32graph.h"



/*
 * display the world
 */
void
draw_world()


{

    int i;
    	
    ksetcolor(WHITE);
    ksetfillstyle(SOLID_FILL,WHITE);
    kbar((float) -45.0, (float) -45.0, (float) (env[cworld].dx + 45.0), (float) (env[cworld].dy + 45.0));     
    //kclear_device();	    
    ksetcolor(BLACK);
    for (i=0; i < env[cworld].nobstacles; i++)
      {
          kline(env[cworld].obstacles[i][0],env[cworld].obstacles[i][1],env[cworld].obstacles[i][2], env[cworld].obstacles[i][3]);
      }
    ksetcolor(BLACK);
    for (i=0; i < env[cworld].nroundobst; i++)
         {
           ksetfillstyle(SOLID_FILL,MAGENTA);
           kfillellipse(env[cworld].roundobst[i][0],env[cworld].roundobst[i][1],env[cworld].radius,env[cworld].radius);
         }
    ksetcolor(BLACK);
    for (i=0; i < env[cworld].nsroundobst; i++)
         {
           ksetfillstyle(SOLID_FILL,RED);
           kfillellipse(env[cworld].sroundobst[i][0],env[cworld].sroundobst[i][1],env[cworld].sradius,env[cworld].sradius);
         }	 
    ksetcolor(GREEN);
    for (i=0; i < env[cworld].nfoodzones; i++)
         {
          kcircle(env[cworld].foodzones[i][0],env[cworld].foodzones[i][1],env[cworld].foodzones[i][2]);
         }
    ksetcolor(BLACK);
    for (i=0; i < env[cworld].nbulblights; i++)
         {
           ksetfillstyle(SOLID_FILL,BLUE);
           kfillellipse(env[cworld].bulblights[i][0], env[cworld].bulblights[i][1],(float) 10.0, (float) 10.0);
         }
    ksetcolor(BLUE);
    for (i=0; i < env[cworld].nlandmarks; i++)
         {
           ksetfillstyle(SOLID_FILL,MAGENTA);
		   if (env[cworld].landmark[i][3] == 0)
             kfillellipse(env[cworld].landmark[i][0], env[cworld].landmark[i][1],env[cworld].landmark[i][2], env[cworld].landmark[i][2]);
		   if (env[cworld].landmark[i][3] == 1)
			 kline(env[cworld].landmark[i][0]-env[cworld].landmark[i][2], env[cworld].landmark[i][1],env[cworld].landmark[i][0]+env[cworld].landmark[i][2], env[cworld].landmark[i][1]);
		   if (env[cworld].landmark[i][3] == 2)
			 kline(env[cworld].landmark[i][0],env[cworld].landmark[i][1]-env[cworld].landmark[i][2],env[cworld].landmark[i][0],env[cworld].landmark[i][1]+env[cworld].landmark[i][2]);
         }

}


/*
 * display the state of the sensors within the robot body
 */
void
draw_ind_sensors(struct individual *pind,float x, float y)

{


     double  ang;
     int     i;
     float   headx,heady;
     float   headx2,heady2;     

     for (i=0; i < pind->nifsensors; i++)
      {
       if (i < 6)
	  ang = pind->direction - 90 + (i * 36);
       if (i == 6)
	  ang = pind->direction - 160;
       if (i == 7)
	  ang = pind->direction + 160;
       if (ang < 0)
	   ang += 360;
       if (ang > 359)
	   ang -= 360;
        headx = SetX(x,(float) 27,ang);
        heady = SetY(y,(float) 27,ang);
	ang = ang - 180;
	if (ang < 0)
	    ang += 360;
        headx2 = SetX(headx,(float) 15,ang);
        heady2 = SetY(heady,(float) 15,ang);
	ksetcolor(LIGHTGRAY);
	kline(headx2, heady2, headx, heady);
	if (drawindif == 1 && pind->nifsensors > 0)
	  {
	    ksetcolor(MAGENTA);
            headx2 = SetX(headx,(float) (pind->cifsensors[i] * (float) 15.0),ang);
            heady2 = SetY(heady,(float) (pind->cifsensors[i] * (float) 15.0),ang);
          }
	if (drawindlight == 1 && (pind->lightsensors > 0 || pind->light2sensors > 0))
	  {
	  ksetcolor(BLUE);
          headx2 = SetX(headx,(float) (pind->clightsensors[i] * (float) 15.0),ang);
          heady2 = SetY(heady,(float) (pind->clightsensors[i] * (float) 15.0),ang);
          }
	kline((float) headx2,(float) heady2,(float) headx, (float) heady);
      } 	
}

/*
 * draw linear camera
 */
void
draw_ind_camera(struct individual *pind)

{

			     
     double  ang;
     int     i;
     float   headx,heady;

     if (pind->cameraunits > 0)
     {
      // virtual camera
      ksetcolor(LIGHTGRAY);
      for (i=0; i < 320; i++)
       { 
        headx = SetX(pind->pos[0],(float) (pind->virtual_camera[i] * 27.0),(double) i * 1.125);
        heady = SetY(pind->pos[1],(float) (pind->virtual_camera[i] * 27.0),(double) i * 1.125);
	kline(pind->pos[0], pind->pos[1],headx, heady);
       }
      // real camera
      ang = pind->direction - (pind->camerarange / 2);
      ksetcolor(BLUE);
      for (i=0; i < pind->cameraunits; i++)
       {
        if (ang < 0)
	    ang += 360;
        if (ang > 359)
	    ang -= 360;     
         headx = SetX(pind->pos[0],(float) (pind->real_camera[i] * 27.0),ang);
         heady = SetY(pind->pos[1],(float) (pind->real_camera[i] * 27.0),ang);
	 kline(pind->pos[0], pind->pos[1],headx, heady);
         ang += (pind->camerarange / pind->cameraunits);
       }
     }

     if (pind->camerafive > 0 || pind->newcamerafive > 0)
     {
      if (pind->camerafive > 0)
        ang = pind->direction + 18.0;
       else
        ang = pind->direction + 120.0;
      for (i=0; i < 5; i++)
       {
        if (ang < 0)
	    ang += 360;
        if (ang > 359)
	    ang -= 360;
	if (pind->real_camera[i] > 0.5f)
           ksetcolor(BLUE);
	  else
           ksetcolor(LIGHTGRAY);
         headx = SetX(pind->pos[0], 27.0f,ang);
         heady = SetY(pind->pos[1], 27.0f,ang);
	 kline(pind->pos[0], pind->pos[1],headx, heady);
         if (pind->camerafive > 0)
           ang -= 9.0;
          else
           ang -= 60;
       }
     }

}



/*
 * arm  0=30up 0.5=10up  1=10down 
 * gripper = 0 (about 4cm) gripper = 1 (about 1.5cm)
 */
void
draw_ind_arm(float x,float y,float arm,float grip,double direction,int ob_taken, float scalearm)

{
      
     double  idirection;
     double  ldirection;
     double  rdirection;     
     float   headx,heady;
     float   backx,backy;     
     float   leftx,lefty;          
     float   rightx,righty;               
     float   fdeltax,fdeltay;
     float   bdeltax,bdeltay;     
      
     /*
      * we compute the central points of the square
      */
     idirection = direction + 180.0;
     if (idirection > 359.0)
	 idirection -= 360.0;
     ldirection = direction + 90.0;
     if (ldirection > 359.0)
	ldirection -= 360.0;
     rdirection = direction - 90.0;
     if (rdirection < 0.0)
	rdirection += 360.0; 
    
     headx  = SetX(x, 42.5f*scalearm,direction);
     heady  = SetY(y, 42.5f*scalearm,direction);
     backx  = SetX(x, 27.5f*scalearm,idirection);
     backy  = SetY(y, 27.5f*scalearm,idirection);     
     rightx = SetX(x, 35.0f*scalearm,rdirection);
     righty = SetY(y, 35.0f*scalearm,rdirection);
     leftx  = SetX(x, 35.0f*scalearm,ldirection);
     lefty  = SetY(y, 35.0f*scalearm,ldirection);          
     
     /*
      * we compute the vertices
      */     
     fdeltax = (headx - x);
     fdeltay = (heady - y);          
     bdeltax = (x - backx);
     bdeltay = (y - backy);               
     flx  =  leftx + fdeltax;   
     fly  =  lefty + fdeltay;        
     frx  =  rightx + fdeltax;   
     fry  =  righty + fdeltay;                  
     blx  =  leftx - bdeltax;   
     bly  =  lefty - bdeltay;        
     brx  =  rightx - bdeltax;   
     bry  =  righty - bdeltay;                  

     ksetcolor(BLACK);

     kline(flx,fly,frx,fry);
     kline(blx,bly,brx,bry);
     kline(flx,fly,blx,bly);
     kline(frx,fry,brx,bry);


     /*
      * we compute the gripper vertices
      */          	  
     grip = (grip / (float) 2.0) + (float) 0.25;
     if (ob_taken == 1)
       grip = (float) 0.75;
     fdeltax = ((flx - frx) / (float) 2.5) * grip;
     fdeltay = ((fly - fry) / (float) 2.5) * grip;

     backx  = flx - fdeltax;
     backy  = fly - fdeltay;
     headx  = SetX(backx,35.0f,direction);
     heady  = SetY(backy,35.0f,direction);

     lgripper[0]  = flx - fdeltax;
     lgripper[1]  = fly - fdeltay;
     lgripper[2]  = SetX((float) lgripper[0],(float) 35,direction);
     lgripper[3]  = SetY((float) lgripper[1],(float) 35,direction);

     rgripper[0]  = frx + fdeltax;
     rgripper[1]  = fry + fdeltay;
     rgripper[2]  = SetX((float) rgripper[0],(float) 35,direction);
     rgripper[3]  = SetY((float) rgripper[1],(float) 35,direction);

     kline(lgripper[0],lgripper[1],lgripper[2],lgripper[3]);
     kline(rgripper[0],rgripper[1],rgripper[2],rgripper[3]);

     ksetcolor(RED);
     ksetfillstyle(SOLID_FILL,RED);
     if (ob_taken == 1)
        {
          obx  = SetX(x,62.5f*scalearm,direction);
          oby  = SetY(y,62.5f*scalearm,direction);
          kfillellipse(obx,oby,env[cworld].sradius*scalearm,env[cworld].sradius*scalearm);
        }

}


/*
 * display an inividual (eventually with the sensors and the arm)
 */
void
draw_ind(int cycle)

{


    int headx,heady;    
    struct individual *pind;
    int i;
    float addspace;
 
    if (cycle > 0)
      {
	for (pind=ind,i=0; i < npopulations; pind++, i++)
          restore_image(i);
	 //if (ball > 0)
	 //  restore_image(2);
      }
     // we display background information
     for (pind=ind,i=0; i < npopulations; pind++, i++)
       {
        if (drawtrace == 1)
         {
	  if (pind->population == 0)
            ksetcolor(BLUE);
	   else
            ksetcolor(CYAN);
          kline(pind->pos[0],pind->pos[1],pind->oldpos[0],pind->oldpos[1]);
         }  
        if (pind->gripper > 0)
         {
           if (env[cworld].xtarget[0] == 1.0 || env[cworld].xtarget[0] == 2.0)
                      {
                       ksetcolor(RED);
                       ksetfillstyle(SOLID_FILL,BLACK);
                       kfillellipse(env[cworld].xtarget[1],env[cworld].xtarget[2],env[cworld].sradius,env[cworld].sradius);
                      }
           if (env[cworld].xtarget[0] == 2.0 || env[cworld].xtarget[0] == 3.0)
                      {
                       ksetcolor(RED);
                       ksetfillstyle(SOLID_FILL,RED);
                       kfillellipse(env[cworld].xtarget[3],env[cworld].xtarget[4],env[cworld].sradius,env[cworld].sradius);
                      }
                     env[cworld].xtarget[0] = (float) 0.0;
         }
      }

      //we save the background
      for (pind=ind,i=0; i < npopulations; pind++, i++)
       {
	 if (pind->gripper == 0)
	   addspace = 30.0f;
	  else
	   addspace = 84.0f;
         save_image(i, (float) (pind->pos[0]-84.0),
                       (float) (pind->pos[1]-84.0),
                       (float) (pind->pos[0]+84.0),
                       (float) (pind->pos[1]+84.0));
       }
      //if (ball > 0)
      //  save_image(2, (float) (Bxpos-(Brad+2.0)),
      //                (float) (Bypos-(Brad+2.0)),
      //                (float) (Bxpos+(Brad+2.0)),
      //                (float) (Bypos+(Brad+2.0))); 
      
      //we draw khepera and if requested the sensors
      for (pind=ind,i=0; i < npopulations; pind++, i++)
       {
        if (pind->population == 0)
          ksetcolor(BLACK);
         else
          ksetcolor(MAGENTA);

        kcircle(pind->pos[0],pind->pos[1],(float) 27.0);
        headx = (int) SetX(pind->pos[0],(float) 27.0, pind->direction);
        heady = (int) SetY(pind->pos[1],(float) 27.0, pind->direction);
        if (drawindif == 0 && drawindlight == 0 && (drawcamera == 0 || (pind->cameraunits == 0 && pind->camerafive == 0 && pind->newcamerafive == 0)))
	  kline(pind->pos[0],pind->pos[1], (float) headx, (float) heady);

        if (drawindif == 1 || drawindlight == 1)
          draw_ind_sensors(pind, pind->pos[0],pind->pos[1]);
	if (drawcamera > 0)
          draw_ind_camera(pind);
        if (pind->gripper > 0)
          draw_ind_arm(pind->pos[0],pind->pos[1],pind->p_arm,p_gripper,pind->direction,pind->ob_taken,1.0f);
       }
      //we display the ball
      //if (ball > 0)
      //   {
      //     ksetcolor(BROWN);
      //     ksetfillstyle(SOLID_FILL,GREEN);
      //     kfillellipse((float) Bxpos, (float) Bypos, (float) Brad, (float) Brad);
      //   }
       


}



/*
 * display fitness curves and statistics  
 * we consider the value -9999.0 as a stopping signal
 */
void
draw_statistics(int g, int startg)

{

     double fit_max, fit_min; 
     double g_scale, fit_scale;
     double max_y, max_x, bord_xy;
     char   sbuffer[64];
     int i,n;

     max_y = max_x = 300.0;
     bord_xy = 50;

     select_window(0);
     startx = (int) 50;
     starty = (int) 50;
     if (userscale != 999)
            {
	      scale = (float) userscale / (float) 100.0;
              startx = (int) ((float) startx * scale);
              starty = (int) ((float) starty * scale);
            }
	   else
	   {
	       set_scale(300,300+20);	     
	   }

     ksetcolor(WHITE);
     ksetfillstyle(SOLID_FILL,WHITE);
     kbar((float) 0.0, (float) 0.0, (float) (max_x + bord_xy), (float) (max_y + bord_xy));  

     // compute range to display
     fit_max = 0.0;
     fit_min = 0.0;
     for(i=startg; i <= g; i++)
      {
	if (statfit[i][0] == -9999.0)
	 {
	  g = i - 1;
	 }
         else
	 {
          if (statfit[i][0] > fit_max)
            fit_max = statfit[i][0];
          if (statfit[i][0] < fit_min)
            fit_min = statfit[i][0];
	  if (npopulations == 2)
	   {
            if (statfit[i][2] > fit_max)
             fit_max = statfit[i][2];
            if (statfit[i][2] < fit_min)
             fit_min = statfit[i][2];
	   }
	 }
      }
   

     if ((fit_max - fit_min) != 0)
       fit_scale = max_y / (fit_max - fit_min);
      else
       fit_scale = 0.0;
     if (g == 0)
       g_scale   = max_x;
      else
       g_scale   = max_x / (double) g;


     ksetcolor(BLACK);
     kline((float) (0.0 + bord_xy), (float) max_y,(float) (0.0 + bord_xy), (float) 0.0);
     kline((float) (0.0 + bord_xy), (float) max_y,(float) (max_x + bord_xy), (float) max_y);
     kline((float) (max_x + bord_xy), (float) 0.0,(float) (max_x + bord_xy), (float) max_y);
     kline((float) (0.0 + bord_xy), (float) 0.0,(float) (max_x + bord_xy), (float) 0);
     if (fit_min > 100 || fit_min < - 100)
       sprintf(sbuffer,"%.0f",fit_min);
      else
       sprintf(sbuffer,"%.2f",fit_min);
     kouttextxy((float) 0,(float) max_y, sbuffer);

     if (fit_max > 100 || fit_max < - 100)
       sprintf(sbuffer,"%.0f",fit_max);
      else
       sprintf(sbuffer,"%.2f",fit_max);
     kouttextxy((float) 0,(float) 0, sbuffer);

     sprintf(sbuffer,"0");
     kouttextxy((float) (0.0 + bord_xy),(float) (max_y + (bord_xy / 2)), sbuffer);
     sprintf(sbuffer,"%d", g);
     kouttextxy((float) (max_x + bord_xy),(float) (max_y + (bord_xy / 2)), sbuffer);

     if (g > 0)
       {
	 for(n=0;n < (npopulations * 2); n++)
	 {
          if (n == 0)
            ksetcolor(RED);
          if (n == 1)
            ksetcolor(BLUE);
          if (n == 2)
            ksetcolor(BROWN);
          if (n == 3)
            ksetcolor(GREEN);
          for(i=startg;i <= (g - 1); i++)
	    {
             kline((float) ((i * g_scale) + bord_xy), (float) (max_y - (statfit[i][n] * fit_scale)), (float) (((i + 1) * g_scale) + bord_xy), (float) (max_y - (statfit[i+1][n] * fit_scale)));
	    }
	 }
       }
     if (g == 0)
       {
          ksetcolor(BLUE);
          kellipse((float) bord_xy, (float) (max_y - (statfit[0][1] * fit_scale)), (float) 5.0, (float) 5.0);
	  if (npopulations == 2)
	   {
	    ksetcolor(BROWN);
            kellipse((float) bord_xy, (float) (max_y - (statfit[0][3] * fit_scale)), (float) 5.0, (float) 5.0);
	   }
       }

}


/*
 * display fitness and statistics  
 */
void
draw_data(struct individual *pind)

{

     char    sbuffer[1024];

     select_window(0);
     if (npopulations == 1)
      {
       sprintf(sbuffer,"epoch %d cycle %d fitness %.3f tfitness %.3f                        ",
	      cepoch,pind->totcycles,pind->fitness - pind->oldcfitness, pind->fitness);
      }
      else
      {
       if (opptype == 0)
         sprintf(sbuffer,"epoch %d cycle %d fitness %.3f tfitness %.3f opponent %d from current gen.",
	      cepoch,pind->totcycles,pind->fitness - pind->oldcfitness, pind->fitness, oppid);
       if (opptype == 1)
         sprintf(sbuffer,"epoch %d cycle %d fitness %.3f tfitness %.3f opponent %d from previous gen.",
	      cepoch,pind->totcycles,pind->fitness - pind->oldcfitness, pind->fitness, oppid);
       if (opptype == 2)
         sprintf(sbuffer,"epoch %d cycle %d fitness %.3f tfitness %.3f opponent %d from the same pop.",
	      cepoch,pind->totcycles,pind->fitness - pind->oldcfitness, pind->fitness, oppid);
      }

     kmessage(sbuffer);
}

/*
 * display sensor and motor states in the secondary window
 */
void
draw_sensors(struct individual *pind)

{
			     
     int     i;
     int     cycle;
     float   o;
     char    sbuffer[64];
     int     nouts;
     int     nunits;     
     int     h1,h2;
     int     cu;
     int     pstarty;
     float   oldscale;
     int     oldstartx,oldstarty;
     float   fit;
     float   error;

     h1 = 13;
     h2 = 16;     
     cu = 0;

     oldscale = scale;
     oldstartx = startx;
     oldstarty = starty;
     scale = (float) 1.0;
     startx = wendx;
     starty = 0;
     // select_window(1);
	
     nunits = 0;
     // compute number of drawing units 
     if (drawinputs == 1)
	 nunits += *pind->layer_descp;
     if (drawhiddens == 1)
	 nunits += *(pind->layer_descp + 1);
     if (drawoutputs == 1)
        {
	 nouts = pind->modulskip;
	 nunits += nouts;
        }
     if (drawteaching == 1)
	 nunits += nouts;
     if (drawerror == 1)
	nunits += 1;
     if (drawfitness == 1)
	nunits += 1;
     if (drawperceived == 1)
	nunits += 1;

     // we maximize h1 and h2 by taking into account the current window dimension
     // but there should be at least 8 pixels
     kdim_device();
     h2 = (cwindowdy / (nunits + 1));
     h1 = h2 - (h2 / 4);
     if (h2 < 8)
       h2 = 8;
     if (h1 < 6)
       h1 = 6;
     pstarty = h2;

     // clear window when we reach the right border 
     if (cwindowdx > 30)
       cycle = (pind->lifecycle % (wendx - (cwindowdx - 30)));
      else
       cycle = 0;
     if (cycle == 0 || pind->lifecycle == 0)
	 kclear_device_from(startx,starty);

     // display perceived object (wall,round,sround,opponent robot)
     if (drawperceived == 1)
     {
     if (cycle == 0)
       {
        ksetcolor(MAGENTA);     	 	      
	kouttextxy ((float) 0, (float) (pstarty)-h1+0,"T");
       }       
     if (o_perceived[0] > 0)
      {
       ksetcolor(BLACK); 
       kline((float) cycle+20,(float) pstarty+0-h1-2,(float)cycle+20,(float) pstarty+0-h1);     
      }
     if (o_perceived[1] > 0)
      {
       ksetcolor(MAGENTA); 
       kline((float) cycle+20,(float) pstarty+3-h1-2,(float)cycle+20,(float) pstarty+3-h1);     
      }
     if (o_perceived[2] > 0)
      {
       ksetcolor(RED);        
       kline((float) cycle+20,(float) pstarty-h1+6-2,(float) cycle+20, (float) pstarty-h1+6);
      }
     if (o_perceived[3] > 0)
      {
       ksetcolor(LIGHTGRAY);     
       kline((float) cycle+20,(float) pstarty-h1+9-2,(float)cycle+20,(float) pstarty-h1+9);     
      }
     pstarty += h2;
     }
     // outputs    
     if (drawoutputs > 0)
      {
        for (i=0; i < nouts; i++, cu++, pstarty += h2)
        {
	 if (cycle == 0)
	  {
           ksetcolor(MAGENTA);     	 	      
	   sprintf(sbuffer,"o%d",i);
	   kouttextxy((float) 0, (float) (pstarty - h1)+4,sbuffer);
          }
	 o = oselect(pind, i);
         if (oselected == 0)
	    ksetcolor(BLUE);
           else
            ksetcolor(GREEN);
 	 kline((float) cycle+20,(float) (pstarty - (o * h1)),(float) cycle+20,(float) pstarty);	   
	}
       }
       // teaching
       if (drawteaching > 0)
        {
         for (i=0; i < nouts; i++, cu++, pstarty += h2)
         {
	   if (cycle == 0)
	    {
             ksetcolor(MAGENTA);     	 	      
	     sprintf(sbuffer,"t%d",i);
	     kouttextxy((float) 0, (float) (pstarty - h1)+4,sbuffer);
            }
	   if (pind->amoebat[i] != -999.0)
	    {
             ksetcolor(GREEN);
 	     kline((float) cycle+20,(float) (pstarty - (pind->amoebat[i] * h1)),(float) cycle+20,(float) pstarty);	   
	    }
	}
       }
       // hidden 	
       if (drawhiddens == 1 && pind->nlayers > 2) 
        {
         ksetcolor(LIGHTGRAY);
         for (i=0; i < *(pind->layer_descp + 1); i++, cu++, pstarty += h2)
	 {
	 kline((float) cycle+20,(float) (pstarty - (pind->layerp[1][i]*h1)), (float) cycle+20, (float) pstarty);
	 if (cycle == 0)
	  {
           ksetcolor(MAGENTA);	      
	   sprintf(sbuffer,"h%d",i);
	   kouttextxy((float) 0, (float) pstarty-h1+4,sbuffer);
          }	 
         }
        }
       // inputs 	
       if (drawinputs == 1)       
        {
         ksetcolor(MAGENTA);
         for (i=0; i < *pind->layer_descp; i++, cu++, pstarty += h2)
	 {
	  kline((float) cycle+20,(float) (pstarty - (pind->layerp[0][i]*h1)), (float) cycle+20, (float) pstarty);
	  if (cycle == 0)
	   {
            ksetcolor(MAGENTA);	
	    sprintf(sbuffer,"I%d",i);
	    kouttextxy((float) 0, (float) pstarty-h1+4,sbuffer);
           }	 
          }
        }
       // learning error 	
       if (drawerror == 1)       
        {
	 error = pind->global_error;
         ksetcolor(LIGHTGRAY);
	 if (error > 1.0)
	 {
           error = 1.0f;
           ksetcolor(RED);
	 }
	 if (cycle == 0)
	   {
            ksetcolor(MAGENTA);
 	    sprintf(sbuffer,"er");
	    kouttextxy((float) 0, (float) pstarty-h1+4,sbuffer);
           }
	 kline((float) cycle+20,(float) (pstarty - (error  * h1)), (float) cycle+20, (float) pstarty);
	 pstarty += h2;
	 cu++;
        }
      // fitness 	      
      if (drawfitness == 1)
       {
        fit = (float) (pind->fitness - pind->oldcfitness);
        ksetcolor(GREEN);     	 
        if (fit > 1.0)
	 {
          ksetcolor(RED);
	  fit = (float) 1.0;
	 }
	if (cycle == 0)
	   {
            ksetcolor(MAGENTA);	      
	    sprintf(sbuffer,"f",i);
	    kouttextxy((float) 0,(float) pstarty-h1+4,sbuffer);
           }
         kline((float) cycle+20,(float) (pstarty-(fit * h1 * 1.0)),(float) cycle+20,(float) pstarty);
	 pstarty += h2;
	 cu++;
      }

//    }
   scale = oldscale;
   startx = oldstartx;
   starty = oldstarty;
   select_window(0);

}


/*
 * draw_weights    
 */
void
draw_weights(struct individual *pind)

{

     char    sbuffer[64];
     float   oldscale;
     int     oldstartx,oldstarty;     
     int     *ldp_b;
     int     *ldp_t;
     int     nunits;
     int     pstarty;     
     int     h1,h2;
     int     cu;
     int     i;
     int     b;
     int     t;
     float   o;
     float   **weip;
     float   **biap;
     float   *wp;
     float   *bp;
     int     **wrul;
     int     *wr;
     float   **wrat;
     float   *wa;
     int     **brul;
     int     *br;
     float   **brat;
     float   *ba;
     int     cycle;
     float   wrange;

     h1 = 23;
     h2 = 26;     
     cu = 0;
 
     oldscale = scale;
     oldstartx = startx;
     oldstarty = starty;
     scale = (float) 1.0;
     startx = wendx;
     starty = 0;
     // select_window(1);
	
     // compute number of drawing units
     nunits = 0;
     for(i=0, ldp_b = pind->layer_descp, ldp_t = (pind->layer_descp + 1); i < (pind->nlayers - 1); i++, ldp_b++, ldp_t++)
       nunits += (*ldp_b * *ldp_t) + *ldp_t;  
     if (nunits > 50)
       {
	display_warning("too many weights to display");
	userbreak = 1;
	return;
       }
   
     //we maximize h1 and h2 by taking into account the current window dimension
     //but there should be at least 8 pixels for each weight
     kdim_device();
     h2 = (cwindowdy / nunits) - 1;
     h1 = h2 - (h2 / 4);
     if (h2 < 8)
       h2 = 8;
     if (h1 < 6)
       h1 = 6;
     pstarty = h2;

     // clear window when we reach the right border 
     if (cwindowdx > 30)
       cycle = (pind->lifecycle % (wendx - (cwindowdx - 30)));
      else
       cycle = 0;
     if (cycle == 0 || pind->lifecycle == 0)
	 kclear_device_from(startx,starty);

     // we set the weight range
     wrange = pind->wrange;

     // we draw the weights
     cu = 0;
     for(i=0, ldp_b = pind->layer_descp, ldp_t = (pind->layer_descp + 1), weip = pind->weightp, biap = (pind->biasp + 1), wrul = pind->wrules, brul = pind->brules, wrat = pind->wrates, brat = pind->brates; i < (pind->nlayers - 1); i++, ldp_b++, ldp_t++, weip++,biap++, wrul++, brul++, wrat++, brat++)
      {
	for (t=0, wp = *weip, bp = *biap, wr = *wrul, br = *brul, wa = *wrat, ba = *brat; t < *ldp_t; t++, bp++, br++, ba++) 
	   {
	     for (b=0; b < *ldp_b; b++) 
		{
	          if (cycle == 0)
	           {
                    ksetcolor(MAGENTA);
		    if (pind->hebbian_learning > 0)
	               sprintf(sbuffer,"%d%d %d%d", b,t,*(wr + b + (t * *ldp_b)), (int) ( *(wa + b + (t * *ldp_b)) * 10.0)); 
		      else
	               sprintf(sbuffer,"%d-%d",b,t);
	            kouttextxy((float) 0, (float) (pstarty - h1)+4,sbuffer);
                   }
		  /*
		   * if autoteaching range might increase
		   * therefore we use 10.0 as a range and we truncate weights after that
		   */
		  if (pind->autoteach == 0)
                     o = *(wp + b + (t * *ldp_b)) / wrange;
		    else
		    {
                     o = *(wp + b + (t * *ldp_b)) / 10.0f;
		     if (o > 1.0)
			o = 1.0f;
		     if (o < -1.0)
			o = -1.0f;
		    }
		  if (o > 0)
		    {
	              ksetcolor(BLUE);
		    } 
		    else
		    {
	              ksetcolor(RED);
                      o = (float) 0.0 - o;
		    }
 	          kline((float) cycle+50,(float) (pstarty - (o * h1)),(float) cycle+50,(float) pstarty);
		  pstarty += h2;
		}
	      if (cycle == 0)
	        {
                 ksetcolor(MAGENTA);
		 if (pind->hebbian_learning > 0)
	            sprintf(sbuffer,"b%d %d%d", t,*br, (int) ( *ba * 10.0) );
		   else
	            sprintf(sbuffer,"b%d",t);			   
	         kouttextxy((float) 0, (float) (pstarty - h1)+4,sbuffer);
                }
	      if (pind->autoteach == 0)
	      {
	       o = *bp / wrange;
	      }
	      {
	       o = *bp / 10.0f;
	       if (o > 1.0)
		 o = 1.0f;
	       if (o < -1.0)
		 o = -1.0f;
	      }
	      if (o > 0)
		 {
	           ksetcolor(BLUE);
		 } 
		 else
		 {
	           ksetcolor(RED);
                   o = (float) 0.0 - o;
		 }
 	      kline((float) cycle+50,(float) (pstarty - (o * h1)),(float) cycle+50,(float) pstarty);
	      pstarty += h2;
	   }
      }

   scale = oldscale;
   startx = oldstartx;
   starty = oldstarty;
   select_window(0);

}



/*
 * print activation of motor neuron for each different direction and
 * distances (180*40)
 */     
void
draw_io_maps()

{
    
    struct individual *pind;
    int  **type;
    int  *w;    
    int  stx,sty;
    int nind;
    int o,u;
    int v,volte;
    int a,d,a1;          
    int ii;
    int color;
    int noutputs;
    int mselected;
    char sbuffer[64];
    float   oldscale;
    int     oldstartx,oldstarty;
    int  npickup[4];
    int  nrelease[4];


    oldscale = scale; 
    oldstartx = startx;
    oldstarty = starty;
    scale = (float) 1.0;
    startx = 20;
    starty = 20;    

    for (v=0; v<4;v++)
     {
       npickup[v]  = 0;
       nrelease[v] = 0;
     }


    kclear_device();
    amoebastart(0,0,0,0);
    
    // load individual
    pind = (ind + dpopulation);
    nind = pind->dindividual;
    getgenome(nind,dpopulation,0); 
   
    noutputs = 2;
    volte    = 1;
    if (env[cworld].nsroundobst > 0)
      volte++;
    if (pind->gripper > 0)
      { 
       noutputs = noutputs * 2;
       volte    = volte * 2;
      }
    ksetcolor(WHITE);
    kouttextxy((float) 75, (float) -10,"no-object");    
    kouttextxy((float) 305,(float) -10,"object");          
    kouttextxy((float) 427,(float)  80,"w");            
    kouttextxy((float) 427,(float) 100,"a");                
    kouttextxy((float) 427,(float) 120,"l");                
    kouttextxy((float) 427,(float) 140,"l");                    
    kouttextxy((float) 427,(float) 280,"t");            
    kouttextxy((float) 427,(float) 300,"a");                
    kouttextxy((float) 427,(float) 320,"r");                
    kouttextxy((float) 427,(float) 340,"g");                        
    kouttextxy((float) 427,(float) 360,"e");                        
    kouttextxy((float) 427,(float) 380,"t");                            
    kouttextxy((float)  20-10,    (float) 470,"180");                
    kouttextxy((float)  20-10+45, (float) 470,"-90");                    
    kouttextxy((float)  20-3+90,  (float) 470,"0");                        
    kouttextxy((float)  20-5+130, (float) 470,"90");                          
    kouttextxy((float)  20-15+180,(float) 470,"180");
    kouttextxy((float) 240-10,    (float) 470,"180");                
    kouttextxy((float) 240-10+45, (float) 470,"-90");                    
    kouttextxy((float) 240-3+90,  (float) 470,"0");                        
    kouttextxy((float) 240-5+130, (float) 470,"90");                          
    kouttextxy((float) 240-15+180,(float) 470,"180");    
    for (v=0;v<volte;v++)
     {
      if (v == 0)
       {
        stx = 20;
        sty = 10;
	type = wall;
	pind->ob_taken = 0;
       }
      if (v == 1)
       {
        stx = 20;
        sty = 245;
	type = sobst;	
	pind->ob_taken = 0;	
       }
      if (v == 2)
       {
        stx = 240;
        sty = 10;
	type = wall;	
	pind->ob_taken = 1;	
       }
      if (v == 3)
       {
        stx = 240;
        sty = 245;
	type = sobst;		
	pind->ob_taken = 1;	
       }       
     ksetcolor(BLACK);
     for (ii=0;ii<5;ii++)
      {
       kouttextxy((float) stx-20,(float) sty+2+(ii*45),"40");        
       kouttextxy((float) stx-20,(float) sty-5+(ii*45)+35," 0");           
       if (ii == 0)     
         sprintf(sbuffer,"left motor");	   
       if (ii == 1)     
         sprintf(sbuffer,"right motor");	   	   
       if (ii == 2)     
         sprintf(sbuffer,"pick up");	   	   
       if (ii == 3)     	   
         sprintf(sbuffer,"release");	   	   
       if (ii == 4)     
         sprintf(sbuffer,"mod. combination");	    
       kouttextxy((float) 445,(float) sty-5+(ii*45)+20,sbuffer);
       krectangle((float) stx-1,(float) sty+(45*ii)+1,(float) stx+180,(float) (20*2)+sty+(45*ii)+2);       
      }
     for(a=0;a<180;a++)
     {
      a1 = a;
      if (a1 >= 90)
	 a1 -= 90;
        else
         a1 += 90;	 
      for(d=0;d<20;d++)
       {
        w = *(type + d);
        w = (w + (a * 8));
        for(ii=0;ii<8;ii++,w++)
          input[ii] = (*w / (float) 1024.0);
        if (pind->nifsensors == 6)      
	  input[6] = (float) pind->ob_taken;      
        if (pind->nifsensors == 8)      
	  input[8] = (float) pind->ob_taken;
        for(u=0;u<*pind->layer_descp;u++)
                 pind->layerp[0][u] = input[u];	
//	*pind->layerp = input;	
	activate_net(pind);	  
        if (pind->autoteach == 1 || pind->modularity == 2)
            {
             for(u=0;u<*pind->layer_descp;u++)
               pind->alayerp[0][u] = pind->layerp[0][u];
             auto_activate_net(pind);
            }      
	mselected = 0;
        for (o=0; o < noutputs; o++)	    
	  {
           if (black_and_white == 0)	      
	     {
	      color = kcolor_scale(oselect(pind,o));	
	     }
	     else
	     {
	       if (oselect(pind,o) > 0.5)
		 color = BLACK;
	        else
		 color = WHITE;
	     }
	   if (o == 0)
                  mselected += oselected;			
	   if (o == 1)
                  mselected += oselected * 2;			
	   if (o == 2)
                  mselected += oselected * 4;			
	   if (o == 3)
                  mselected += oselected * 8;
	   if (mselected == GREEN && black_and_white == 1)
	       mselected = WHITE;
           if (black_and_white == 0)	      
	   {
 	   if (o == 2)	       
	    if (oselect(pind,o) > 0.75 && pind->ob_taken == 0)
	     {
	      color = RED;
	      npickup[v] += 1;
	     }
             else
	     {
	      color = WHITE;
	     }
	   if (o == 3)
	    if (oselect(pind,o) > 0.5 && pind->ob_taken == 1)
	     {
	      color = RED;
	      nrelease[v] += 1;
	     }
             else
	     {
	      color = WHITE;
	     }
           }
	   else
	   {
 	   if (o == 2)	       
	    if (oselect(pind,o) > 0.75 && pind->ob_taken == 0)
	     {
	      color = BLACK;
	      npickup[v] += 1;
	     }	
             else
	     {
	      color = WHITE;
	     }
	   if (o == 3)
	    if (oselect(pind,o) > 0.5 && pind->ob_taken == 1)
	     {
	      color = BLACK;
	      nrelease[v] += 1;
	     }	        
             else
	     {
	      color = WHITE;	      	    
	     }
	   }
	   ksetcolor(color);
	   kline((float) a1+stx,(float) (20*2)-(d*2)+(45*o)+sty,(float) a1+stx+1,(float) (20*2)-(d*2)+1+(45*o)+sty);
           // it was +1 but polyline do not reach ending point
         }
	if (pind->modularity == 2)
	 {
	  o = noutputs;
	  ksetcolor(mselected);
	  kline((float) a1+stx,(float) (20*2)-(d*2)+(45*o)+sty,(float) a1+stx+1,(float) (20*2)-(d*2)+1+(45*o)+sty);
           // it was +1 but polyline do not reach ending point
	 }

       }
      }
     }
     
   scale = oldscale;
   startx = oldstartx;
   starty = oldstarty;

   /*
   for (v=0; v<4; v++)
    {
      sprintf(sbuffer, "%d %d", npickup[v], nrelease[v]);
      kouttextxy((float) 600.0, (float) (0.0 + (v * 50)), sbuffer);
    }
    */

}






/*
 * display the current position of the robot with respect to the perceived objects
 * it uses the secondary window
 */
void
draw_selfsel_maps(struct individual *pind, int cycle)



{


     FILE *fopen();
     FILE *fp;          
     int     v1,v2;
     int     a;
     int     d,v;
     int     stx,sty;
     int     t;
     int     h1,h2;
     int     cu;
     int     nunits;
     char    sbuffer[16];
     int     surface;
     float   oldscale;
     int     oldstartx;
     int     oldstarty;

     //oldcwindow = cwindow;
     // select_window(1);

     // we set the scale by trying to maximize the image and
     //  by taking into account the xspace necessary for the environment
     // we restore the oldscale before exit
     oldscale  = scale;
     oldstartx = startx;
     oldstarty = starty;
     startx = (int) 10;
     starty = (int) 0;
     if (pind->gripper == 0)
       set_scale((int) (360+40+(env[cworld].dx*scale)), (120+40)*2);
      else
       set_scale((int) ((360+40+(env[cworld].dx*scale))*2), (120+40)*2);

     startx = (int) ((float) startx * scale) + wendx;
     starty = (int) ((float) starty * scale);

     h1 = 45; /* graph at the bottom net-height */
     h2 = 50; /* graph at the bottom height */
     cu = 0;
     nunits = 3;
     stx = 10; /* offset x */
     sty = 10; /* offset y */
     surface = 0; /* if 1 we display the separability surface */
     graphfrequence = 0; /* if 1 we display frequency */

     /* we display the graph's limits with x and y legends */
     if (cycle== 0)
      {
      kclear_device_from(startx,starty);
      for (t=0;t < (3 + (pind->gripper * 3));t++)
       {
        stx = 10;
        sty = 10;
        if (t < 3)
         {
          stx = 10;
          sty += t * (120 + 50);
	 }
         else
         {
          stx = (360 + 50);
          sty += (t - 3) * (120 + 50);
	 } 
        /* if not big round we skip them */
	if (env[cworld].nroundobst == 0 && (t == 2 || t == 5))
          sty -= (120 + 50);
	ksetcolor(BLACK);
	krectangle((float)stx-1,(float)sty-1+6,(float)(180*2)+stx+1,(float)(20*6)+5+1+sty);	           
        /* if surface we open the surface.sur file */
	if (surface == 1)
	  fp = fopen("surface.sur","r");

        for (a=0;a<180;a++)
         {
	 if (a == 0 || (a % 45) == 0 || a == 179)
	   {
	    v1 = a;
	    if (v1 == 179)
	      v1 = 180;
            if (v1 >= 90)
	      v1 -= 90;
	     else
	      v1 += 90;     	       
 	    ksetcolor(BLACK);	       
            sprintf(sbuffer,"%d",v1*2);
	    kouttextxy((float)a*2+stx-10,(float)(20*6)+sty+15,sbuffer);
	   }
	   /* we display the separability surface */
	  if (surface == 1)
           for (d=0;d<20;d++)
           {
             v1 = a;
             v2 = d;
             if (v1 >= 90)
	       v1 -= 90;
              else
	       v1 += 90;
	     fscanf(fp,"%d ", &v);
	     if (v == 0)
	      {
	        ksetfillstyle(SOLID_FILL,LIGHTGRAY);
	        ksetcolor(LIGHTGRAY);
                krectangle((float)(v1*2)+stx,(float)(20*6)-(v2*6)+sty,(float)(v1*2)+2+stx,(float)(20*6)-(v2*6)+6+sty);
	      }
	   }
         } 
 	ksetcolor(BLACK);	       	 
	kouttextxy((float)stx-30,(float)sty+3+( 0*6),"40");	 
	kouttextxy((float)stx-30,(float)sty+3+(10*6),"20");	 	
	kouttextxy((float)stx-30,(float)sty+3+(20*6)," 0");
       }
        /* if surface we close the surface.sur file */
	if (surface == 1)
	  fclose(fp);
      }

     /*
      * we compute the new coordinate
      */
     v1 = radarmap[1][1];
     v2 = radarmap[1][2];
     if (v1 >= 90)
	 v1 -= 90;
       else
	 v1 += 90;

     /*
      * we have 3 raw of maps (wall,round,sround) on y axis 
      * and 2 columns (ob_taken or not) on x axis
      */
     stx = 10; /* offset x */
     sty = 10; /* offset y */
     if (pind->gripper == 1 && pind->ob_taken == 1)
       stx += (360 + 50); 
     sty += (radarmap[1][0] - 1) * (120 + 50); 
     /* if not big round we skip them */
     if (env[cworld].nroundobst == 0 && radarmap[1][0] == 3)
          sty -= (120 + 50);

     /*
      * we display relative positions in the corresponding map
      */
     /*
     ksetfillstyle(SOLID_FILL,LIGHTGRAY);
     kbar((float)v1*2+stx,(float)(20*6)-(v2*6)+sty,(float)v1*2+1+stx,(float)(20*6)-(v2*6)+5+sty);
     */
     /*
      * we display movement arrows 
      */
     if (graphfrequence == 0)
      {
      ksetcolor(BLACK);
      if (cycle > 0 && radarmap[0][0] == radarmap[1][0] && radarmap[0][3] == radarmap[1][3])
       karrow((float)(radarmap[0][1]*2)+stx,
              (float)(20*6)-(radarmap[0][2]*6)+sty,
              (float)(v1*2)+stx,
              (float)(20*6)-(v2*6)+sty,(float) 2.0);
      }
     /*
      * if is the first cycle or we changed map we backup
      * if we are in the same map we backup only after x cycles
      */
     if (1 > 0)
      {
       radarmap[0][0] = radarmap[1][0];
       radarmap[0][1] = v1;
       radarmap[0][2] = v2;
       radarmap[0][3] = radarmap[1][3];
      }

     //select_window(oldcwindow);
     scale  = oldscale;
     startx = oldstartx;
     starty = oldstarty;


}






/*
 * read a file of values between 0.0 and 1.0 and plot them in grayscale
 */     
void
draw_map(char *filename)

{

   int   x,y;
   int   mx,my;
   float value;
   FILE  *fopen();
   FILE  *fp;
   int   icolor;
   int   scalepix;
   int   s,ss;
   float oldscale;


   oldscale = scale;
   scale = 1.0f;
   scalepix = 2;

   if ((fp=fopen(filename, "r")) == NULL) 
      {
        display_warning("ERROR : I can't open the map file\n");
	return;
      }	 
   fscanf(fp,"%d %d\n",&mx,&my);

   ksetcolor(GREEN);
   kline(0.0f - 1.0f, 0.0f - 1.0f, 0.0f -1.0f, (float) (my * scalepix) + 1.0f);
   kline(0.0f - 1.0f, 0.0f - 1.0f, (float) (mx * scalepix) + 1.0f, 0.0f - 1.0f);
   kline((float) mx * scalepix + 1.0f, 0.0f - 1.0f, (float) (mx * scalepix) + 1.0f, (float) (my * scalepix) + 1.0f);
   kline(0.0f -1.0f, (float) (my * scalepix) + 1.0f, (float) (mx * scalepix) + 1.0f, (float) (my * scalepix) + 1.0f);

   for(y = 0; y < my; y++)
    {
    for(x = 0; x < mx; x++)
     {
      fscanf(fp,"%f ",&value);

      if (value < 0.0 || value > 1.0)
	icolor = 4;
       else
	icolor = 16 + (int) (value * 10.0f);

       for(s = 0; s < scalepix; s++)
         for(ss = 0; ss < scalepix; ss++)
           kputpixel((float) (x*scalepix)+s,  (float) (y*scalepix)+ss, icolor);
      }
    fscanf(fp,"\n");
    }
 
  fclose(fp); 

  scale = oldscale;

}



/* 
 * display a sampled object
 */
void
draw_sample(int **object, int  *objectcf)

{

   int    **ob;
   int    *o;
   int    a,d,s;
   int    a1;
   float stx, sty;
   int   icolor;
   int   height;
   char  sbuffer[256];
 
   
   stx = (float) 0.0;
   sty = (float) 0.0;
   height = 3;
   kdim_device();
   kclear_device();

   sprintf(sbuffer, "nturns %d init dist. %d inter dist. %d ", objectcf[2], objectcf[3], objectcf[4]);
   kmessage(sbuffer);
   
   for (s=0; s < objectcf[0]; s++)
    {
      ob=object;    
      ksetcolor(BLACK);
      if ((s % 2) == 0)
        krectangle(stx - (float) 1.0, sty - (float) 1.0 + (float) 3.0,(float) 180.0 + stx + (float) 0.0, (float) (objectcf[2] * height) + sty + (float) (height + 1.0) - (float) 1.0);
      for (a = 0; a < objectcf[1]; a++)      
       {
         a1 = a;
         if (a1 >= 90)
	   a1 -= 90;
          else
           a1 += 90;	
         for (d=0; d < objectcf[2]; d++)	 
          {
	    o = *(ob + d); 	      
            o = (o + (a * objectcf[0]) + s);
	    icolor = (int) (((double) *o / 1024.0) * 10.0);
	    ksetcolor(icolor + 16);
	    kline((float) a1+stx,(float) (objectcf[2]*height)-(d*height)+sty,(float) a1+stx,(float) (objectcf[2]*height)-(d*height)+height+sty);	    
          }
       }
     sty += (objectcf[2] * height) + 10;
     if (sty + ((objectcf[2] * height) + 10) > (cwindowdy - 50))
      {
       sty = 0.0f;
       stx = stx += (objectcf[1] * 1) + 20;
      }
    }
}



/*
 * visual display while the real robot is running 
 */
void
draw_real_robot(struct individual *pind)

{


    int     i;
    int     icolor;
    float   robotx,roboty;
    float   headx,heady;
    float   headx2,heady2;     
    double  ang;
    char    sbuffer[1024];
    
    robotx = 150.0f;
    roboty = 150.0f;

    // clear
    ksetcolor(WHITE);
    ksetfillstyle(0, WHITE);
    if (pind->lifecycle == 0)
      kclear_device();
     else
      kbar(robotx - 100.0f, roboty - 100.0f, robotx + 100.0f, roboty + 100.0f);

    // draw the robot body
    ksetcolor(BLACK);
    kcircle(robotx,roboty,(float) 81.0);
    // draw the infrared sensors 
    for (i=0; i < 8; i++)
      {
       if (i < 6)
	  ang = 270.0 - 90 + (i * 36);
       if (i == 6)
	  ang = 270.0 - 160;
       if (i == 7)
	  ang = 270.0 + 160;
       if (ang < 0)
	   ang += 360;
       if (ang > 359)
	   ang -= 360;
       ang -= 2;
       headx = SetX(robotx,(float) 81.0,ang);
       heady = SetY(roboty,(float) 81.0,ang);
       ang = ang - 180;
       if (ang < 0)
	    ang += 360;
       headx2 = SetX(headx,(float) 45,ang);
       heady2 = SetY(heady,(float) 45,ang);
       ksetcolor(LIGHTGRAY);
       kline(headx2, heady2, headx, heady);
       ksetcolor(MAGENTA);
       headx2 = SetX(headx,(float) (pind->cifsensors[i] * (float) 40.0),ang);
       heady2 = SetY(heady,(float) (pind->cifsensors[i] * (float) 40.0),ang);
       kline((float) headx2,(float) heady2,(float) headx, (float) heady);
     }
    kouttextxy(0.0f,(roboty*2.0f) + 10.0f, "if:");
    sprintf(sbuffer,"%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f",
	       pind->cifsensors[0],pind->cifsensors[1],pind->cifsensors[2],pind->cifsensors[3],
	       pind->cifsensors[4],pind->cifsensors[5],pind->cifsensors[6],pind->cifsensors[7]);
    kouttextxy(30.0f,(roboty*2.0f) + 10.0f, sbuffer);

    // draw the light sensors
    for (i=0; i < 8; i++)
      {
       if (i < 6)
	  ang = 270.0 - 90 + (i * 36);
       if (i == 6)
	  ang = 270.0 - 160;
       if (i == 7)
	  ang = 270.0 + 160;
       if (ang < 0)
	   ang += 360;
       if (ang > 359)
	   ang -= 360;
       ang += 2;
       headx = SetX(robotx,(float) 81.0,ang);
       heady = SetY(roboty,(float) 81.0,ang);
       ang = ang - 180;
       if (ang < 0)
	    ang += 360;
       headx2 = SetX(headx,(float) 45,ang);
       heady2 = SetY(heady,(float) 45,ang);
       ksetcolor(LIGHTGRAY);
       kline(headx2, heady2, headx, heady);
       ksetcolor(BLUE);
       headx2 = SetX(headx,(float) (pind->clightsensors[i] * (float) 40.0),ang);
       heady2 = SetY(heady,(float) (pind->clightsensors[i] * (float) 40.0),ang);
       kline((float) headx2,(float) heady2,(float) headx, (float) heady);
     }

    kouttextxy(0.0f,(roboty*2.0f) + 30.0f, "al:");
    sprintf(sbuffer,"%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f",
	       pind->clightsensors[0],pind->clightsensors[1],pind->clightsensors[2],pind->clightsensors[3],
	       pind->clightsensors[4],pind->clightsensors[5],pind->clightsensors[6],pind->clightsensors[7]);
    kouttextxy(30.0f,(roboty*2.0f) + 30.0f, sbuffer);
 
    // draw bottom sensor
    icolor = 16 + (int) ((1.0 - pind->bottomsensor) * 10.0f);
    ksetcolor(BLACK);
    ksetfillstyle(SOLID_FILL,icolor);
    kfillellipse(robotx,roboty+15.0f,7.0f,7.0f);

    kouttextxy(90.0f,(roboty*2.0f) + 50.0f, "bs:");
    sprintf(sbuffer,"%.2f", pind->bottomsensor);
    kouttextxy(120.0f,(roboty*2.0f) + 50.0f, sbuffer);

    // draw wheels
    for (i=0; i<2;i++)
    {
     if (i == 0)
     {
      headx = SetX(robotx, 84.0f,0.0);
      heady = SetY(roboty, 84.0f,0.0);
     }
     else
     {
      headx = SetX(robotx, 84.0f,180.0);
      heady = SetY(roboty, 84.0f,180.0);
     }
     ksetcolor(LIGHTGRAY);
     headx2 = SetX(headx, 40.0f, 90.0);
     heady2 = SetY(heady, 40.0f, 90.0);
     kline(headx,heady,headx2,heady2);
     headx2 = SetX(headx, 40.0f, 270.0);
     heady2 = SetY(heady, 40.0f, 270.0);
     kline(headx,heady,headx2,heady2);

     ksetcolor(MAGENTA);
     if (pind->speed[0] > 0.0)
     {
      headx2 = SetX(headx, (float) ((0 - pind->speed[i])*4),90.0);
      heady2 = SetY(heady, (float) ((0 - pind->speed[i])*4),90.0);
     }
     else
     {
      headx2 = SetX(headx, (float) (pind->speed[i]*4),270.0);
      heady2 = SetY(heady, (float) (pind->speed[i]*4),270.0);
     }
     kline(headx,heady,headx2,heady2);
    }
    
    kouttextxy(0.0f,(roboty*2.0f) + 50.0f, "we:");
    sprintf(sbuffer,"%.0f %.0f    ", pind->speed[0],pind->speed[1]);
    kouttextxy(30.0f,(roboty*2.0f) + 50.0f, sbuffer);
    
    // draw gripper
    if (noturret1 == 0)  
     {
       draw_ind_arm(robotx,robotx,0.0f,0.0f,270.0,0,3.0f);

       icolor = 16 + (int) ((1.0 - pind->lightbarsensor) * 10.0f);
       ksetcolor(BLACK);
       ksetfillstyle(SOLID_FILL,icolor);
       kfillellipse(robotx+85.0f,roboty-145.0f,7.0f,7.0f);

       kouttextxy(180.0f,(roboty*2.0f) + 50.0f, "lb:");
       sprintf(sbuffer,"%.2f", pind->lightbarsensor);
       kouttextxy(210.0f,(roboty*2.0f) + 50.0f, sbuffer);
     }

    // draw camera
    if (noturret2 == 0)
    {
     ang = 270.0 - (36.0 / 2.0);
     for (i=0; i < 32; i++)
     {
      if (ang < 0)
        ang += 360;
      if (ang > 359)
        ang -= 360;
      ksetcolor(LIGHTGRAY);
      headx = SetX(robotx, 40.0f,ang);
      heady = SetY(roboty, 40.0f,ang);
      kline(robotx, roboty,headx, heady);
      ksetcolor(BLUE);
      headx = SetX(robotx, (pind->real_camera[i] * 40.0f),ang);
      heady = SetY(roboty, (pind->real_camera[i] * 40.0f),ang);
      kline(robotx, roboty,headx, heady);
      ang += (36.0 / 32.0);
     }
     for (i=0; i < 32; i++)
     {
      icolor = 16 + (int) (pind->real_camera[i] * 10.0f);
      ksetcolor(BLACK);
      ksetfillstyle(SOLID_FILL,icolor);
      kbar((float)10+(i*9),roboty+100.0f,(float)(10+(i+1)*9-1),roboty+125.0f);      
     }
     kouttextxy(250.0f,(roboty*2.0f) + 50.0f, "mp:");
     sprintf(sbuffer,"%2d %.2f   ", pind->pixminint, pind->pixcvar);
     kouttextxy(280.0f,(roboty*2.0f) + 50.0f, sbuffer);
     /*
     kouttextxy(0.0f,(roboty*2.0f) + 70.0f, "ca:");
     sprintf(sbuffer,"%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f",
	       pind->real_camera[0],pind->real_camera[1],pind->real_camera[2],pind->real_camera[3],
	       pind->real_camera[4],pind->real_camera[5],pind->real_camera[6],pind->real_camera[7]);
     kouttextxy(30.0f,(roboty*2.0f) + 70.0f, sbuffer);
     sprintf(sbuffer,"%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f",
	       pind->real_camera[8],pind->real_camera[9],pind->real_camera[10],pind->real_camera[11],
	       pind->real_camera[12],pind->real_camera[13],pind->real_camera[14],pind->real_camera[15]);
     kouttextxy(30.0f,(roboty*2.0f) + 90.0f, sbuffer);    
     sprintf(sbuffer,"%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f",
	       pind->real_camera[16],pind->real_camera[17],pind->real_camera[18],pind->real_camera[19],
	       pind->real_camera[20],pind->real_camera[21],pind->real_camera[22],pind->real_camera[23]);
     kouttextxy(30.0f,(roboty*2.0f) + 110.0f, sbuffer);    
     sprintf(sbuffer,"%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f",
	       pind->real_camera[24],pind->real_camera[25],pind->real_camera[26],pind->real_camera[27],
	       pind->real_camera[28],pind->real_camera[29],pind->real_camera[30],pind->real_camera[31]);
     kouttextxy(30.0f,(roboty*2.0f) + 130.0f, sbuffer);
     */
    }


}


/* 
 * sample an object
 */
void
sample_object(int **object, int distances)

{

   int    **ob;
   int    *o;
   int    a,d,s;
   int    old_cameraunits;
   int    ciclo;
   int    icolor;
   FILE *fp;
 
   

   old_cameraunits = ind->cameraunits;
   ind->cameraunits = 32;
   ciclo = 0;
   draw_world();
   for (d=0,ob=object;d < 10; d++,ob++)
   {
     if (ciclo > 0)
     {
       ind->oldpos[0] = ind->pos[0];
       ind->oldpos[1] = ind->pos[1];
       ind->oldirection = ind->direction;
     }
     ind->pos[0] = (float) (50 + (d * 50));
     ind->pos[1] = (float) 200.0;
     ind->direction = 180.0;
     for (a = 0, o = *ob; a < 360; a += 2)      
       {
         ind->direction += 2.0;
	 if (ind->direction > 360.0)
	   ind->direction -= 360.0;
	 if (ind->direction < 0.0)
	   ind->direction += 360.0;
         set_input(ind);
	 
	 for (s=0; s < ind->cameraunits; s++,o++)
	 {
	   *o = (int) (input[ind->nifsensors + s] * (float) 1024.0);
           if (*o < 0)
	     *o = 0;
	   if (*o > 1024)
	     *o = 1024;
	   ind->layerp[0][ind->nifsensors + s] = input[ind->nifsensors + s];
           icolor = (int) (((float) (*o / (float) 1024.0)) * (float) 10.0);

	 }
	 // we save the data on a file
         fp=fopen("sample.txt","a");
	 for (s=0; s < ind->cameraunits; s++)
          fprintf(fp,"%.3f ",input[ind->nifsensors + s]);
	 for (s=0; s < ind->cameraunits; s++)
          fprintf(fp,"%.3f ",input[ind->nifsensors + s + 32]);
	 fprintf(fp,"\n");
	 fclose(fp);
	 draw_ind(ciclo);
	 ciclo++;
       }
  
   }
   ind->cameraunits = old_cameraunits;
}





