/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/* 
 * file - main.c
 */


#include "defs.h"
#include <stdlib.h>
#include "w32graph.h"    // for using graphic print here
#include <windows.h>     // for Sleep function


/* 
 * try to load parameters from the configuration file (default robot.cf)
 * if the file does not exists the user should set the parameters by hand
 */
void
init_khepsim()

{


	// default window
	select_window(0);

	// default filenames
        sprintf(fileroot,"robot.cf");
        sprintf(worldfiles,"world1.env");

	// initialize world variables 
	init_world();
        
	// create populations pointers (ind) - we always allocate space for two populations
        create_poppointer();

	// initiliaze populations parameters
	init_parameters();


	/*
	 * we load the parameters from the .cf file
	 *  (3) initializing the parameters
	 *  (4) loading the parameters 
	 *  (5) computing the special parameters
	 *  (6) create the populations
	 * otherwise the create e botton of parameneters->general do 2 and 3
	 * and the create bottom of parameters->population do 4, 5, and 6
	 */

       if (load_parameters(fileroot) == 1)
        {
          cfloaded = 1;
        }
        else
	{
          disable_menu();
	}

       load_world();
       allo_space();
       load_motor();     
       load_obstacle("wall.sam",   wall,     wallcf);
       load_obstacle("round.sam",  obst,     obstcf);
       load_obstacle("small.sam",  sobst,    sobstcf);
       load_obstacle("light.sam",  light,    lightcf);

}


/*
 *
 */
void
get_str_word(char *st1, char *st2)

{

    while (*st2 != ' ' && *st2 != '\n' && *st2 != '\t' && *st2 != NULL)
	{
	  *st1 = *st2;
	  st1++;
	  st2++;
    }

}



/*
 * set the teaching input and update the weights 
 */	      
void
learn(struct individual *pind, long int cycle)

{

   FILE *fp;
   int   save_each;
   int   u;
   char  buffer[16];


   if (save_error_each > 0)
      save_each = save_error_each;
     else
      save_each = 100;
   /*
    * if we have autoteaching learning
    * we set the teaching and update the network
    * we also compute the average error
    */
   if (pind->autoteach == 1)
	{

         for(u=0; u < pind->layer_descp[pind->nlayers - 1]; u++)
                pind->amoebat[u] = pind->alayerp[pind->nlayers - 1][u];

         update_net(pind, pind->amoebat);

         pind->error_sum += pind->global_error;
	 pind->error_n++;
	 if (pind->error_n == save_each)
	    {
	      pind->perror = pind->error_sum / (double) pind->error_n;
	      pind->error_n = 0;
	      pind->error_sum = 0.0;
	      if (save_error_each > 0)
	       {
		sprintf(buffer,"%d.err",pind->population);
	        fp = fopen(buffer,"a");
	        fprintf(fp, "%d\t1\t%.3f\n",cycle,pind->perror);
	        fclose(fp);
	       }
	    }
	}

}



/*
 * the regular loop
 */
void
loop(int gen,int pop, int nind,int epoch)


{
        extern double sqrt();	
        extern double fabs();

	struct  individual *pind;
	int     i;
	int     e,ee;
	float   armfitness;
	float   explofitness;	
	float   vel1,vel2;
	int     s;
        int     u;
	int     stop;
	int     lastcycle;
	int     prey_crashed;
	int     predator_crashed;
        int     crash_occurred;


	/* initialize variables */	
	explofitness = (float)0.0;
	run = 0;
	stop = 0;	
	lastcycle = 0;
	amoebastart(gen,pop,nind,epoch);
	prey_crashed     = 0;
	predator_crashed = 0;
	individual_crash = 0;
        env[cworld].xtarget[0] = (float) 0.0;
	for(cpopulation=0,pind=ind;cpopulation<npopulations;cpopulation++,pind++)
	  {
            reset_net(pind);
            pind->lifecycle     = 0;
	    pind->global_error  = 0.0f;
            pind->energy        = 1.0f;
            pind->ob_taken      = 0;
	    pind->p_arm         = 0.0;
	    if (epoch == 0)
	      {
	       pind->oldcfitness = 0.0;
	       pind->oldfitness = 0.0;
	      }
             else
              pind->oldfitness = pind->fitness;
            if (pind->ffitness == 99)
             {
              for (e=0;e<100;e++)
               for (ee=0;ee<100;ee++)
                explored[e][ee] = 0;
             } 
	  }
        if (drawindividual == 1 && sim_real == 0)
              draw_world();
	// if we evolve in the real environment we randomize the robot position
        if (sim_real == 1 && run_evolution == 1)
	    {
              random_real_pos(200);
	    }
	// if we evolve in the real environment and we have a gripper we set its initial position
        if (sim_real == 1 && pind->gripper > 0 && noturret1 == 0)
          init_arm();
	/*
         * BEGIN OF EPOCH LOOP
	 */	
	while ((run < sweeps) && stop == 0)
	 {
	  /* 
	   * then ..for each individual robot 
	   */
	  for(cpopulation=0,pind=ind;cpopulation<npopulations;cpopulation++,pind++)
	   { 
             pind->oldcfitness = pind->fitness;

	     /* we set the input */
	     if (sim_real == 0)
               set_input(pind); 
	      else
	       set_input_real(pind);

             for(u=0;u<*pind->layer_descp;u++)
                 pind->layerp[0][u] = input[u];
	     /* 
	      * we activate the network 
	      */
	     activate_net(pind);
             if (pind->autoteach == 1 || pind->modularity == 2)
                {
                 for(u=0;u<*pind->layer_descp;u++)
                   pind->alayerp[0][u] = pind->layerp[0][u];
                 auto_activate_net(pind);
                }

	    /*
	     * we move khepera and the arm
	     */
    
	    if (sim_real == 0)
	      {
	        if (pind->gripper > 0)	     
	      	   armfitness = check_arm(pind);
	        for(i=0; i < timelaps; i++)
                   setpos(pind,oselect(pind, 0),oselect(pind, 1));
	      }
	    else
	      {	
	        if (pind->gripper > 0 && noturret1 == 0)	     
	      	   check_real_arm(pind);		    
		 setpos_real(pind,oselect(pind, 0),oselect(pind, 1));
	      }
			      
	     /*
	      * learning 
	      */
	     if (pind->hebbian_learning == 1)
		    hebbian_update(pind);
	     if (pind->autoteach == 1)
                    learn(pind, run + (sweeps * epoch));

	    /*
	     * we check for crash 
	     * if no crashes occurred we compute fitness
	     */
	     crash_occurred = 0;
	     if (check_crash(pind->pos[0],pind->pos[1]) || (pind->gripper > 0 && armfitness < 0.0))
              {
		crash_occurred = 1;
		if (stop_when_crash == 1)
		   stop = 1;
		  else
                   reset_pos_from_crash(pind);                   		
                if (pind->ob_taken == 0)
		  ncrashes1++;
                 else 
		  ncrashes2++;
		if (individual_crash == 0)
		 {
                  if (pind->ffitness == 10 || pind->ffitness == 12)
		    predator_crashed = 1;
		  if (pind->ffitness == 11)
		    prey_crashed = 1;
		 }
              }
	    /*
	     * we compute fitness only if no-crash occurred
	     * but we do it anyway for ffitness=22
             */
             if (crash_occurred == 0 || pind->ffitness == 22)
              {
	       /*
	        * obstacle avoidance fitness
	        */
               if (pind->ffitness == 1)
                {
		 fitmode = 1;
		 vel1 = pind->speed[0];
		 vel2 = pind->speed[1];
		 ave_mot = (float) fabs((vel1 / (float) 10.0) + (vel2 / (float) 10.0)) / (float) 2.0;
                 dif_m   = (float) sqrt(fabs( ((vel1 + (float) 10.0) / (float) 20.0) - ((vel2 + (float) 10.0) / (float) 20.0) ) );
		 for(s=0, maxs = (float) 0.0; s<8; s++)
                   if (pind->cifsensors[s] > maxs)
		      maxs = pind->cifsensors[s];
        
		 pind->fitness += ave_mot * (1.0 - dif_m) * (1.0 - maxs);
		}
	       /*
	        * discrimination fitness
		* we get one if we are on the goal target zone
		*/
               if (pind->ffitness == 2)
                {
		  fitmode = 1;
                  if (read_ground(pind) > 0)
                       pind->fitness += 1;
		}
	       /*
	        * rat
                * we score 1 if we spend the last cycle on the foodzone
		*/
               if (pind->ffitness == 3 || pind->ffitness == 4 || pind->ffitness == 5)
                {
		   fitmode = 2;
		   if (read_ground(pind) > 0 && run == (sweeps - 1))
                     pind->fitness += 1.0;
		}	

	       /*
	        * arm fitness
		* armfitness is 1 if the robot has the target and 10000 if releases it correctly
		*/
               if (pind->ffitness == 6)
                {
		   fitmode = 0;
		   pind->fitness += armfitness;
                   if (armfitness >= 100.0 && nobreak == 0)
                      stop = 1;
		}
	       /*
	        * homing fitness
		*/
               if (pind->ffitness == 7)
                {
		   fitmode = 1;
		   vel1 = pind->speed[0];
		   vel2 = pind->speed[1];
		   ave_mot = (float) fabs((vel1 / (float) 10.0) + (vel2 / (float) 10.0)) / (float) 2.0;
		   for(s=0, maxs = (float) 0.0; s<8; s++)
                     if (pind->cifsensors[s] > maxs)
		        maxs = pind->cifsensors[s];
                   if (read_ground(pind) > 0)
		      {
                       pind->energy = 1.0f;
		       //pind->fitness += ave_mot * (1.0 - maxs);
		      }
		      else
		      {
		       pind->fitness += ave_mot * (1.0 - maxs);
		       pind->energy -= 0.01f;
		      }
                   if (pind->energy <= 0.0f)
		    {
                      stop = 1;
		      pind->energy = 0.0f;
		    }
		}
	       /*
	        * changing
                * we score (sweeps - run) if we find the foodzone 
		*/
               if (pind->ffitness == 8)
                {
		   fitmode = 2;
		   if (read_ground(pind) > 0)
		    {
                     pind->fitness += sweeps - run;
		     stop = 1;
		    }
		}


               if (pind->ffitness == 9)
                {
		   if (read_ground(pind) > 0)
		   {
                     pind->fitness += 1.0;
 		     stop = 1;
		   }
		}
		       
              }
          }
        /* 
	 * we display graphics, pause, assign fitness 10 and 11 and 
	 * increment lifecycle, at the end of the cycle
	 */
	  if (drawindividual == 1 && sim_real == 0)
            draw_ind(run);
	    // real only support one population
	  if (sim_real == 1)
            draw_real_robot(ind);

	for(cpopulation=0,pind=ind;cpopulation<npopulations;cpopulation++,pind++)
	 {
	   if (drawindividual == 1 || sim_real == 1)
	   {
	   if (drawselfselmaps > 0 && cpopulation == dpopulation)
	      draw_selfsel_maps(pind, run);
	   if (graphsensors > 0 && cpopulation == dpopulation) 
	      draw_sensors(pind);
	   if (draweights > 0 && cpopulation == dpopulation)
	      draw_weights(pind);
	   if (drawdata > 0 && cpopulation == dpopulation  && run_evolution == 0)
	      draw_data(pind);
	   }

	   // we only pause if we are drowing the individual
	   if (timeadjust > 1 && (drawindividual == 1 || sim_real == 1))
              Sleep(timeadjust);

	   /*
	    * we assign fitness to predator and prey when we have an individual_crash
	    * or when we reach the last cycle
	    * or when predator or prey crashed if we have stop_when_crash on
	    */
	    if (pind->ffitness == 10 || pind->ffitness == 11)
	    {
	      fitmode = 2;
	      // predator catch the prey 
             if (individual_crash == 1)
	      {
                stop = 1;
                if (pind->ffitness == 10)
                 pind->fitness += 1.0f;
              }
	      else
	      {
	       // prey is still alive at last cycle
               if (run == (sweeps - 1))
	        {
	         if (pind->ffitness == 11)
                   pind->fitness +=  1.0f;
	        }
              }
	      // prey or predator crashed
	      if (stop_when_crash == 1 && individual_crash == 0 && (run < (sweeps - 1)))
	      {
		if (pind->ffitness == 10 && prey_crashed)
                  pind->fitness += 1.0f;
		if (pind->ffitness == 11 && predator_crashed)
                  pind->fitness += 1.0f;
	      }
	    }
	    /*
	     * we update cycle number
	     */	
	    pind->lifecycle++;
	    pind->totcycles++;
	   }
	  run++;
	  if (userbreak == 1)
	     run = sweeps;
          }   

	stop_real();

}







 
float
rans(double n)
	
{

// sun version
// return((((float)rand()*0.4656612875e-9) - 0.5) * 2.0 * n); 

 static	double max = 32767;
 return ((float) (n - (2.0 * n *((double) rand() / max))));

}



int
mrand(int i)


{
	extern int rand();
	int r;

	r = rand();
	r = r % (int) i;
	return r;
}




