/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 
 * low level routine for the serial ports have been developed by Daniele Denaro

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * real.c
 */

# include <stdio.h>
# include <stdlib.h>
# include <math.h>
# include <time.h>
# include <conio.h>
# include <ctype.h>

# include <windows.h>
# include <malloc.h>
# include <memory.h>

# include "defs.h"


static DCB dcb;
static HANDLE hCom,hStdout;
static DWORD dwError;
static BOOL fSuccess;
static COMMTIMEOUTS comt;

static char comm[10]="COM2";
static unsigned int bauds=38400;

/*
 * set the serial connection parameters
 */ 
void
setcomm()
{
  if (serialport == 1)
    strncpy(comm,"COM1",4);
   else
    strncpy(comm,"COM2",4);
  
  bauds=serialspeed;
}

/*
 * open and configure the serial port
 */
int
aprecom()

{

 static char tbuff[256];
 static char rbuff[512];

 kmessage("I am trying to open the serial port connection");

 setcomm();

 if (hCom==NULL)
  {
    hCom = CreateFile(comm,
    GENERIC_READ | GENERIC_WRITE,
    0,             // comm devices must be opened w/exclusive-access 
    NULL,          // no security attrs 
    OPEN_EXISTING, // comm devices must use OPEN_EXISTING 
    0,             // not overlapped I/O 
    NULL           // hTemplate must be NULL for comm devices 
    );

  // handle error */
  if (hCom == INVALID_HANDLE_VALUE) 
       dwError = GetLastError();

  // Omit the call to SetupComm to use the default queue sizes.
  //Get the current configuration.

  fSuccess = GetCommState(hCom, &dcb);

  if (fSuccess)
     ;
   else
    display_error("unable to open the serial port");

  // Fill in the DCB: baud=9600, 8 data bits, no parity, 1 stop bit. */

  dcb.BaudRate = bauds;
  dcb.ByteSize = 8;
  dcb.Parity = NOPARITY;
  dcb.StopBits = 2; //ONESTOPBIT;

  dcb.fOutxCtsFlow=0;
  dcb.fOutxDsrFlow=0;
  dcb.fDtrControl=DTR_CONTROL_DISABLE;
  dcb.fDsrSensitivity=0;
  dcb.fRtsControl=RTS_CONTROL_DISABLE;


  GetCommTimeouts(hCom,&comt);
  comt.ReadIntervalTimeout=100;
  SetCommTimeouts(hCom,&comt);

  fSuccess = SetCommState(hCom, &dcb);

  if (fSuccess)
     ;
   else
    display_error("unable to open the serial port");

  noturret1 = 0;
  noturret2 = 0;

 }

 PurgeComm(hCom,PURGE_RXCLEAR|PURGE_TXCLEAR);

 kmessage("serial connection successfully opened");


 sprintf(tbuff,"A,3800,800,100\r");
 comandok(tbuff,rbuff);
 if (rbuff[0] != 'a')
   display_error("serial link not active");

 return 0;

}


/*
 * send a string to the robot and receive a string back
 */
int 
comandok(char tbuff[],char rbuff[])
{

 int lc,lcb;

  WriteFile(hCom,tbuff,strlen(tbuff),&lc,0);
  rbuff[0]=0;
  for (lc=0;lc<500;lc++)
       {
	 ReadFile(hCom,&rbuff[lc],1,&lcb,0);
         if (rbuff[lc]==10) 
		{
		  rbuff[lc+1]=0; 
		  return 0;
		}
         if (lcb==0) 
		{
		  rbuff[lc]=0; 
		  return -1;
		}
      }
 
 //return 0;
}



/*
 * let the robot turn 360 degrees and store while storing sensory information
 */
void
sample_turn(int turn, int **object, int *objectcf)

{

  int    a;
  int   *o;
  int    s;
  int    vel1,vel2;
  static char rbuff[1024];
  static char tbuff[250];
  char   sbuffer[256];
  int    inp[8];

  for (a=0, o = *(object + turn); a < 180; a++)
  {
    if (drawsampleob == 1 || drawsampleob == 2 || drawsampleob == 3)
     {
      sprintf(tbuff,"N\r");
      comandok(tbuff,rbuff);
      sscanf(rbuff,"\nn,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d",
	   &inp[0],&inp[1],&inp[2],&inp[3],&inp[4],&inp[5],&inp[6],&inp[7]);
      for(s=0;s<8;s++,o++)
       *o = inp[s];
     }
    if (drawsampleob == 4)
     {
      sprintf(tbuff,"O\r");
      comandok(tbuff,rbuff);
      sscanf(rbuff,"\no,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d",
	   &inp[0],&inp[1],&inp[2],&inp[3],&inp[4],&inp[5],&inp[6],&inp[7]);
      for(s=0;s<8;s++,o++)
       *o = inp[s];
     }

    if (serialspeed < 38400)
     {
      sprintf(tbuff,"D,-1,1\r");
      comandok(tbuff,rbuff);
     }
     else
     {
      sprintf(tbuff,"D,-2,2\r");
      comandok(tbuff,rbuff);
     }	     

    sprintf(sbuffer,"angle %d",a*2);
    kmessage(sbuffer);

    Sleep(timeadjust);

  }

   // we do the last cycle (to reach 360 degrees) and then stop
   comandok(tbuff,rbuff);
   Sleep(timeadjust);
   sprintf(sbuffer,"angle %d",360);
   kmessage(sbuffer);
   sprintf(tbuff,"D,0,0\r");
   comandok(tbuff,rbuff);
   // we draw the new sampled state
   draw_sample(object, objectcf);

}



/*
 * randomize starting position of the robot by doing obstacle avoidance
 * if it is stack on a wall it first try to move away
 */
void
random_real_pos(int ncycles)

{

  float  net1,net2;
  float  act1,act2;
  int    v1,v2;
  static char rbuff[1024];
  static char tbuff[250];
  int    inp[8];
  float  inpf[4];
  int    c;


  // reset khepera
  sprintf(tbuff,"A,3800,800,100\r");
  comandok(tbuff,rbuff);

  for (c = 0; c < ncycles; c++)
  {

   if (userbreak == 1)
	break;

   sprintf(tbuff,"N\r");
   comandok(tbuff,rbuff);
   sscanf(rbuff,"\nn,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d",
	   &inp[0],&inp[1],&inp[2],&inp[3],&inp[4],&inp[5],&inp[6],&inp[7]);
  
   net1 = 0.0f;
   net2 = 0.0f;
   inpf[0] = (((float) inp[0] / 1024.0f) + ((float) inp[1] / 1024.0f)) / 2.0f;
   inpf[1] = (((float) inp[2] / 1024.0f) + ((float) inp[3] / 1024.0f)) / 2.0f;
   inpf[2] = (((float) inp[4] / 1024.0f) + ((float) inp[5] / 1024.0f)) / 2.0f;
   inpf[3] = (((float) inp[6] / 1024.0f) + ((float) inp[7] / 1024.0f)) / 2.0f;

   if (c < 4)
   {
     if (inpf[1] > 0.5)
     {
      sprintf(tbuff,"D,-2,-2\r");
      comandok(tbuff,rbuff);
      Sleep(1000);
     }
     if (inpf[3] > 0.5)
     {
      sprintf(tbuff,"D,2,2\r");
      comandok(tbuff,rbuff);
      Sleep(1000);
     }
     if (inpf[0] > 0.5)
     {
      sprintf(tbuff,"D,-2,2\r");
      comandok(tbuff,rbuff);
      Sleep(1000);
     }
     if (inpf[2] > 0.5)
     {
      sprintf(tbuff,"D,2,-2\r");
      comandok(tbuff,rbuff);
      Sleep(1000);
     }
   }

   net1 = 0.9688 + (inpf[0] * 0.8984f) + (inpf[1] * -0.0078f) + (inpf[2] * 0.7188f) + (inpf[3] * -0.8906f);
   net2 = 0.9766 + (inpf[0] * -1.0f) + (inpf[1] * -0.9688f) + (inpf[2] * -0.9609f) + (inpf[3] * 0.5f);

   act1 = logistic(net1);
   act2 = logistic(net2);

   v1 = (int) (act1 * 20.0);
   v2 = (int) (act2 * 20.0);  
   if ( ((act1 * 200) - (v1 * 20)) > 5)
     v1++;
   if ( ((act2 * 200) - (v2 * 20)) > 5)
     v2++;
   v1 -= 10;
   v2 -= 10;
  
   if (c >= 4)
    {
    sprintf(tbuff,"D,%d,%d\r",v1,v2);
    comandok(tbuff,rbuff);
    }
  }

  sprintf(tbuff,"D,0,0\r");
  comandok(tbuff,rbuff);

}


/*
 * stop the robot
 */
void
stop_real()

{

  static char rbuff[512];
  static char tbuff[250];

  sprintf(tbuff,"D,0,0\r");
  comandok(tbuff,rbuff);

}

/*
 * set the desired speed of the wheels
 */
void
setpos_real(struct individual *pind,float o1, float o2)

{

  int v1,v2;
  static char rbuff[512];
  static char tbuff[250];

 
  // predator (ffitness 10) has half speed 
  if (pind->ffitness == 10)
  {      
   v1 = (int) (o1 * 10.0);
   v2 = (int) (o2 * 10.0);  
   if ( ((o1 * 100) - (v1 * 10)) > 5)
     v1++;
   if ( ((o2 * 100) - (v2 * 10)) > 5)
     v2++;
   v1 -= 5;
   v2 -= 5;
  }      
  else
  {
   // standard speed 
   v1 = (int) (o1 * 20.0);
   v2 = (int) (o2 * 20.0);  
   if ( ((o1 * 200) - (v1 * 20)) > 5)
     v1++;
   if ( ((o2 * 200) - (v2 * 20)) > 5)
     v2++;
   v1 -= 10;
   v2 -= 10;
  }
  
   sprintf(tbuff,"D,%d,%d\r",v1,v2);
   comandok(tbuff,rbuff);
}


/*
 * set the initial position of the gripper
 * with the arm up and the hand open
 */
void
init_arm()

{

     static char rbuff[1024];
     static char tbuff[250];

     sprintf(tbuff,"T,1,D,0\r");
     comandok(tbuff,rbuff);	   
     Sleep(2000);	
     sprintf(tbuff,"T,1,E,160\r");
     comandok(tbuff,rbuff);
     Sleep(2000);

}



/*
 * pickup and release procedures
 * we do not score for object released because we cannot know if
 *  the robot release the object outside the arena
 */
void
check_real_arm(struct individual *pind)

  
{
       

    float arm;
    float released;
    int   justaken;
    static char rbuff[1024];
    static char tbuff[250];
    int    inp[64];  


    arm = oselect(pind, 2);
    released = oselect(pind, 3);
    justaken = 0;

    if (pind->ob_taken == 0)
    {
      if (arm > 0.75 && pind->p_arm <= 0.75)	
       {
         sprintf(tbuff,"D,0,0\r");
         comandok(tbuff,rbuff);  
         sprintf(tbuff,"D,-10,-10\r");
         comandok(tbuff,rbuff);  	 
         Sleep(200);
         sprintf(tbuff,"D,0,0\r");
         comandok(tbuff,rbuff);
         sprintf(tbuff,"T,1,E,250\r");
         comandok(tbuff,rbuff); 
         Sleep(2000);	 		 		 
         sprintf(tbuff,"T,1,G\r");
         comandok(tbuff,rbuff);
         sscanf(rbuff,"\nt\,1\,g\,%d", &inp[0]);
	 if (inp[0] > 125)
	  {
	   justaken = 1;
           sprintf(tbuff,"T,1,D,1\r");
           comandok(tbuff,rbuff);
           Sleep(2000);	 		 		 
           sprintf(tbuff,"T,1,E,160\r");
           comandok(tbuff,rbuff);
           Sleep(2000);	
           pind->ob_taken = 1;		   
	  }
	  else
	  {
           justaken = 0;
           sprintf(tbuff,"T,1,E,160\r");
           comandok(tbuff,rbuff);
	  }
      }
    }
    else
    { 
      if (released > 0.7 && justaken == 0)
       {
         sprintf(tbuff,"D,0,0\r");
	 comandok(tbuff,rbuff);
         sprintf(tbuff,"T,1,E,220\r");
	 comandok(tbuff,rbuff);
         Sleep(2000);	
         sprintf(tbuff,"T,1,D,0\r");
	 comandok(tbuff,rbuff);	   
         Sleep(2000);	
         sprintf(tbuff,"T,1,E,160\r");
	 comandok(tbuff,rbuff);
         Sleep(2000);	     	 
	 pind->ob_taken = 0;
       }
       
     }
     
     pind->p_arm     = arm;

     
}

/*
 * set the input of the neural controller on the basis of the phisical
 * sensors of the real robot
 */
void
set_input_real(struct individual *pind)

{

  int    vel1,vel2;
  int    i;
  static char rbuff[1024];
  static char tbuff[250];
  int    inp[64];
  int    val;
  int    ii;
  int    nsensor;
  int    *ldp;
  double sum;
  extern double sqrt();


  // infrared sensors
  sprintf(tbuff,"N\r");
  comandok(tbuff,rbuff);
  sscanf(rbuff,"\nn,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d",
	   &inp[0],&inp[1],&inp[2],&inp[3],&inp[4],&inp[5],&inp[6],&inp[7]);
  for (i=0;i<8;i++)
    {
     pind->cifsensors[i] = (float) inp[i] / 1024.0f;
     if (pind->cifsensors[i] > 1.0)
        pind->cifsensors[i] = 1.0;
     if (pind->cifsensors[i] < 0.0)
	pind->cifsensors[i] = 0.0;
    }

  // light sensors
  sprintf(tbuff,"O\r");
  comandok(tbuff,rbuff);
  sscanf(rbuff,"\no,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d",
	   &inp[0],&inp[1],&inp[2],&inp[3],&inp[4],&inp[5],&inp[6],&inp[7]);
  for (i=0;i<8;i++)
    {
     pind->clightsensors[i] = (float) inp[i] / 1024.0f;
     if (pind->clightsensors[i] > 1.0)
        pind->clightsensors[i] = 1.0;
     if (pind->clightsensors[i] < 0.0)
	pind->clightsensors[i] = 0.0;
    }

   // camera - 32 pixels mode 
   if (noturret2 == 0)
   {
    sprintf(tbuff,"T,2,S\r");
    comandok(tbuff,rbuff);
    if (rbuff[0] == 't')
     {
      sscanf(rbuff,"\nt\,2\,s\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d,%d\,%d\,%d\,%d\,%d\,%d\,%d\,%d",
 	   &inp[0],&inp[1],&inp[2],&inp[3],&inp[4],&inp[5],&inp[6],&inp[7],&inp[8],&inp[9],&inp[10],&inp[11],&inp[12],&inp[13],&inp[14],&inp[15],&inp[16],&inp[17],&inp[18],&inp[19],&inp[20],&inp[21],&inp[22],&inp[23],&inp[24],&inp[25],&inp[26],&inp[27],&inp[28],&inp[29],&inp[30],&inp[31]);
      for (i=0;i<32;i++)
        pind->real_camera[i] = (float) inp[i] / 256.0f;
     }
     else
     {
       noturret2 = 1;
     }
   }

   // camera - pixel with minimal intensity

   if (noturret2 == 0)
   {
    sprintf(tbuff,"T,2,P\r");
    comandok(tbuff,rbuff);
      sscanf(rbuff,"\nt\,2\,p\,%d", &inp[0]);
    pind->pixminint = inp[0];
    for (i=0,sum=0.0;i<31;i++)
     {
       sum += ((pind->real_camera[i] - pind->real_camera[i+1]) * (pind->real_camera[i] - pind->real_camera[i+1]));
     }
    pind->pixcvar = (sum / 31.0) * 1000.0;
   }
   else
   {
    pind->pixminint = 0;
    pind->pixcvar  =  0.0;
   }

   // read additional if sensor placed on the bottom side
   sprintf(tbuff,"I,5\r");
   comandok(tbuff,rbuff);
   sscanf(rbuff,"\ni,%d", &val);
   pind->bottomsensor = (float) val / 1024.0f;

   // read the lightbarrier sensor of the gripper
   if (noturret1 == 0)
    {
     sprintf(tbuff,"T,1,G\r");
     comandok(tbuff,rbuff);
     if (rbuff[0] == 't')
      {
        sscanf(rbuff,"\nt\,1\,g\,%d", &inp[0]);
        pind->lightbarsensor = (float) inp[0] / 256.0f;;
      }
      else
      {
        noturret1 = 1;
	pind->lightbarsensor = 0.0f;
      }
    }

   // read current speed
   sprintf(tbuff,"E\r");
   comandok(tbuff,rbuff);
   sscanf(rbuff,"e,%d,%d",&vel1,&vel2);	
   pind->speed[0] = (float) vel1;
   pind->speed[1] = (float) vel2;


   /*
    * we now update the input layer
    */

   nsensor = 0;

   /*
    * we check range of infrared
    */
   for(ii = 0; ii < 8; ii++)
    {
      if (pind->cifsensors[ii] > 1.0)
         pind->cifsensors[ii] = 1.0f; 
    }

   /*
    * add noise to infra-red sensors if requested
    * if ob_taken == 1 extra noise is present
    */
   if (sensornoise > 0.0)
     for(i=0; i < 8; i++)
	{
 	 pind->cifsensors[i] += rans(sensornoise);	
         if (pind->cifsensors[i] < 0.0) pind->cifsensors[i] = (float) 0.0;
         if (pind->cifsensors[i] > 1.0) pind->cifsensors[i] = (float) 1.0;
        }

   /*
    * we copy 4, 6, or 8 if sensors on the input vector
    */
   if (pind->nifsensors == 4)
    {
      input[0] = (pind->cifsensors[0] + pind->cifsensors[1]) / (float) 2.0;	
      input[1] = pind->cifsensors[2];	
      input[2] = pind->cifsensors[3];	
      input[3] = (pind->cifsensors[4] + pind->cifsensors[5]) / (float) 2.0;
      nsensor += 4;
    } 
   if (pind->nifsensors == 6)
    {
      for(ii=0;ii<6;ii++,nsensor++)
         input[nsensor] = pind->cifsensors[ii];   
    } 
   if (pind->nifsensors == 8)
    {
      for(ii=0;ii<8;ii++,nsensor++)
         input[nsensor] = pind->cifsensors[ii];   
    } 

   /*
    * we check range of light sensors
    */
   for(ii = 0; ii < 8; ii++)
     {
      if (pind->clightsensors[ii] > 1.0)
         pind->clightsensors[ii] = 1.0f; 
     }
   /*
    * add noise to infra-red sensors if requested
    * if ob_taken == 1 extra noise is present
    */
   if (sensornoise > 0.0)
     for(i=0; i < 8; i++)
	{
 	 pind->clightsensors[i] += rans(sensornoise);	
         if (pind->clightsensors[i] < 0.0) pind->clightsensors[i] = (float) 0.0;
         if (pind->clightsensors[i] > 1.0) pind->clightsensors[i] = (float) 1.0;
        }
   /*
    * we copy 2 or 8 sesors on the input vector
    */
   if (pind->light2sensors == 1)
    {
       input[nsensor] = pind->clightsensors[2];
       nsensor++;
       input[nsensor] = pind->clightsensors[6];   
       nsensor++;
    } 
   if (pind->lightsensors == 1)
    {
      for(ii=0;ii<8;ii++,nsensor++)
         input[nsensor] = pind->clightsensors[ii];   
    } 

   /*
    * camera
    */
   for (i=0; i < pind->cameraunits; nsensor++)
      {
       input[nsensor] = pind->real_camera[i];
       if (input[nsensor] > 1.0)
		  input[nsensor] = 1.0f;
       if (input[nsensor] < 0.0)
		  input[nsensor] = 0.0f;
       if (pind->cameraunits == 32)
          i++;
         else
	  i += pind->cameraunits / 8;
      }
   /*
    * camera5
    */
    if (pind->camerafive > 0 || pind->newcamerafive > 0)
    {
     input[nsensor+0] = 0.0; 
     input[nsensor+1] = 0.0; 
     input[nsensor+2] = 0.0; 
     input[nsensor+3] = 0.0; 
     input[nsensor+4] = 0.0; 
     if (pind->pixcvar > 1.0)
     {
       if (pind->pixminint >=  0 && pind->pixminint < 13)
         input[nsensor+0] = 1.0; 
       if (pind->pixminint >= 13 && pind->pixminint < 25)
         input[nsensor+1] = 1.0; 
       if (pind->pixminint >= 25 && pind->pixminint < 38)
         input[nsensor+2] = 1.0; 
       if (pind->pixminint >= 38 && pind->pixminint < 51)
         input[nsensor+3] = 1.0; 
       if (pind->pixminint >= 51 && pind->pixminint < 64)
         input[nsensor+4] = 1.0; 
      }
     nsensor += 5;
    }

   /*
    * light-barrier sensor on the gripper
    */
   if (pind->objectsensor > 0)
     {
      input[nsensor] = pind->lightbarsensor;
      nsensor += 1;
     }     

   /*
    * if we have a ground sensor we activate it
    */ 
   if (pind->groundsensor == 1)
     {
       if (pind->bottomsensor > 0.5)
         input[nsensor] = (float) 1.0;
	else
         input[nsensor] = (float) 0.0;	
       nsensor += 1;
     }    

   /*
    * set motor sensors (copy of the speed motor outputs)
    */
   if (pind->motmemory > 0)
     {
       input[nsensor]   = oselect(pind, 0);
       input[nsensor+1] = oselect(pind, 1);
       nsensor += 2;
     }  

   /*
    * set elman network sensors
    */
   if (pind->elman_net == 1 && pind->nlayers > 2)
     {
      ldp = pind->layer_descp;
      ldp++;
      for (i=0;i < *ldp; i++,nsensor++)
         if (pind->lifecycle == 0)
           input[nsensor] = (float) 0.0;
          else
           input[nsensor] = pind->layerp[1][i]; 
     }  
   /*
    * copy recurrent pseudo-output into input
    */     
   if (pind->input_outputs > 0)
    {
      for (i=0;i < pind->input_outputs; i++,nsensor++)
	   input[nsensor] = oselect(pind, ((pind->wheelmotors * 2) + (pind->gripper * 2) + i));
    }



}




