/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * simulato.c
 */

#include "defs.h"
#include <math.h>
#include "w32graph.h"


float **motor;              /* 12 rows each with 12*2(ang and dist) columnsi*/
int   **wall;               /* wall (180 degrees and 38 distances)          */
int   **obst;               /* round obstacle (180 d. and 38 distances      */
int   **sobst;              /* round obstacle (180 d. and 20 distances      */
int   **ballobst;           /* ball (180 a. 20 distances                    */
int   **camera;             /* camera sample (180x20                        */
int   **camera2;            /* camera sample                                */
int   **light;              /* light (180 degrees and 27 distances)         */
int   *wallcf;              /* wall  configuration                          */
int   *obstcf;              /* obst  configuration                          */
int   *sobstcf;             /* sobst configuration                          */
int   *lightcf;             /* light configuration                          */
int   *ballobstcf;          /* ball  configuration                          */
int   *cameracf;            /* camera configuration                         */
int   **objtosample;        /* just a pointer to the object to sample       */
int   *objtosamplecf;       /* just a ponter to the object to sample conf   */



float input[200];
float   flx,fly;            /* arm coordinate                               */
float   frx,fry;            /*      "                                       */
float   blx,bly;            /*      "                                       */
float   brx,bry;            /*      "                                       */
float   obx,oby;            /* object position when taken                   */
float   lgripper[4];        /* left  gripper coordinate                     */
float   rgripper[4];        /* right gripper coordinate                     */
float   p_gripper;          /* gripper position                             */
float   ob_sensed;          /* the object is between the gripper            */
double  ldist,rdist,fdist;  /* dist of the object in the hand               */
float   ob_fitness;         /* fitness gathered by the object               */
double  vdirection;         /* kepera virtual direction                     */
double  P90,P180,P360;


/*
 * just set few costants
 */
void
SETP ()

{
  extern double asin();

  P90 =    asin(1.0);
  P180=2.0*asin(1.0);
  P360=4.0*asin(1.0);
}

/*
 * trasform radiants to degrees
 */
double 
RADtoDGR(double r)

{
   double x;

   SETP();
   if (r < 0.0) 
    x=360.0-(r*(-1.0)/P90)*90;
     else      
    x=(r/P90)*90;
   return(x);
 }


/*
 * transform degrees to radiants
 */
double 
DGRtoRAD(double d)

{ 
    double   x;
    extern double asin();    

    x=(d/90)*asin(1.0);
    return(x);
}

/*
 * compute the deltax given an angle and a distance
 */
float   
SetX(float x, float d, double angle)
  

{
    float   res;
    extern double cos();    

    res =  (float) (x + d * cos(DGRtoRAD(angle)) + 0.000001);
    return(res);
}

/*
 * compute the deltay given an angle and a distance
 */
float   
SetY(float y, float d, double angle)

{
    float   res;
    extern double sin();        
  
    res = (float) (y + d * sin(DGRtoRAD(angle)) + 0.000001);
    return(res);

}


/*
 * return the distance between a point and a line
 */
double
linedist(float px,float py,float ax,float ay,float bx,float by)

{
     extern double sqrt();     
     double d1,d2,d3;

     d1 = ((px-ax)*(px-ax)) + ((py-ay)*(py-ay));     
     d2 = ((px-bx)*(px-bx)) + ((py-by)*(py-by));     
     d3 = ((ax-bx)*(ax-bx)) + ((ay-by)*(ay-by));          

     return(sqrt(d1 - (((d2 - d3 - d1) * (d2 - d3 - d1)) / (4 * d3))));
     
}


/*
 * return the distance between two points
 */

double
mdist(float x, float y, float x1, float y1)

{
   extern double sqrt();
   double qdist;

   qdist = ((x-x1)*(x-x1)) + ((y-y1)*(y-y1));
   return(sqrt(qdist));
}



/*
double
LineAngle(int x,int y,int x1,int y1)

  {

    double dx,dy,res;
    //extern double RADtoDGR();
    extern double atan2();

    dx =(double)x1-x;
    dy =(double)y1-y;
    res=RADtoDGR(atan2((dy*dy),(dx*dx)));
    return(res);
 }
*/

/*
 * normalize and angle between 0 and 360
 */
double
normang(double a)

{

   int v;

   v = 0;
   while((a < 0.0 || a > 360.0) && v < 10)
    {
      if (a < 0.0)
	a += 360.0;
      if (a > 360.0)
	a -= 360.0;
      v++;
    }
   if (v == 10)
	display_warning("normang: angle out of range");
   return(a);
}


/*
 * it takes in input the amoeba and the food coordination and calculate
 * the absolute angle
 */

double
mang(int x, int y, int x1, int y1, double dist)

 {

   extern int abs();
   extern double asin();
   double angolo;
   int       dxang,dyang;
   double    buffer;
   double    alfa,pi2;
   double temp;

   if  (x==x1 && y==y1)
      angolo=0.0;
     else
      {
       dxang=(x1-x);
       dyang=(y1-y);
       buffer=abs(dyang);
       pi2 =asin(1.0);
       temp = buffer/dist;
       if (temp>1.0) temp=1.0;
       alfa=asin(temp);
       alfa=(alfa/pi2)*90;
       if (dxang>0)
	     {
              if (dyang>0);
	      if (dyang<0)
                 alfa=360-alfa; /*alfa+270;*/
	     }
        if (dxang<0)
	     {
              if (dyang>0) alfa=180-alfa; /*alfa+90;*/
	      if (dyang<0) alfa=alfa+180;
	      if (dyang==0)alfa=180;
	      }
	 if (dxang==0)
	     {
		 if (dyang>0) ;
		 if (dyang<0) alfa=270;
		 if (dyang==0)alfa=0;
	     }
	  angolo=alfa;
       }
     return(angolo);
}
 



/* 
 * it calculate the relative angle 
 */
double
mangrel(double absolute,double kepera)



{

    double  relative;      


    relative = absolute - kepera;    

    if (relative > 359.0)
        relative -= 360.0;
    if (relative < 0.0)
        relative += 360.0; 	      

    return(relative); 
      
}



/* 
 * it calculate the relative angle 
 */
double
mangrel2(double absolute,double kepera)

{

    double  relative;      


    relative = 360 - (kepera - absolute);    

    if (relative > 359.0)
        relative -= 360.0;
    if (relative < 0.0)
        relative += 360.0; 	
    
    return(relative); 
      
}



     

/* 
 * initialize world variables
 * 
 */
void
init_world()


{

   int    i,ii;
   int    w;
   
   for(w=0;w<5;w++)
   {
     env[w].dx = 0;
     env[w].dy = 0;
     env[w].nobstacles = 0;
     for (i=0; i < 20; i++)
       for (ii=0; ii < 4; ii++)
          env[w].obstacles[i][ii]  = 0.0f;
     env[w].nroundobst = 0;
     env[w].radius = 0.0f;
     for (i=0; i < 20; i++)
       for (ii=0; ii < 2; ii++)
          env[w].roundobst[i][ii]  = 0.0f;
     env[w].nsroundobst = 0;
     env[w].sradius = 0.0f;
     for (i=0; i < 20; i++)
       for (ii=0; ii < 3; ii++)
          env[w].sroundobst[i][ii] = 0.0f;
     env[w].nfoodzones = 0;
     for (i=0; i < 20; i++)
       for (ii=0; ii < 3; ii++)
          env[w].foodzones[i][ii]  = 0.0f;
     env[w].nbulblights = 0;
     for (i=0; i < 20; i++)
       for (ii=0; ii < 2; ii++)
          env[w].bulblights[i][ii]  = 0.0f;
     env[w].nstarts = 0;
     for (i=0; i < 20; i++)
       for (ii=0; ii < 5; ii++)
          env[w].starts[i][ii]  = 0.0f;  
     env[w].nlandmarks = 0;
     for (i=0; i < 20; i++)
       for (ii=0; ii < 6; ii++)
          env[w].landmark[i][ii]  = 0.0f;  
   }
}



/* 
 * it load files rootN.env (0 < N < 5)
 * default filename is world1.env
 */
int
load_world()


{

   extern int  strlen();
   FILE *fopen();
   FILE *fp;
   int    i,ii;
   int    w;
   int    l;
   char   sbuffer[256];
   char   worldname[256];
   char   rootname[256];
   int    realnworlds;
   
   l = strlen(worldfiles);
   for(i=0;i < (l - 5); i++)
     rootname[i] = worldfiles[i];
   rootname[i] = '\0';
   realnworlds = 0;
   for(w=0;w<nworlds;w++)
   {
     env[w].dx = 0;
     env[w].dy = 0;
     sprintf(worldname,"%s%d.env",rootname,w+1);
     if ((fp = fopen(worldname,"r")) != NULL)
      {
	// walls
        fscanf(fp,"walls %d\n", &env[w].nobstacles);
        for (i=0; i < env[w].nobstacles; i++)
         { 
          fscanf(fp,"%f %f %f %f\n",
            &env[w].obstacles[i][0],&env[w].obstacles[i][1],&env[w].obstacles[i][2],&env[w].obstacles[i][3]);
          if (env[w].obstacles[i][0] > env[w].dx)
            env[w].dx = (int) env[w].obstacles[i][0];
          if (env[w].obstacles[i][2] > env[w].dx)
            env[w].dx = (int) env[w].obstacles[i][2];
          if (env[w].obstacles[i][1] > env[w].dy)
            env[w].dy = (int) env[w].obstacles[i][1];
          if (env[w].obstacles[i][3] > env[w].dy)
            env[w].dy = (int) env[w].obstacles[i][3];
          }
	// round obstacles
        fscanf(fp,"rounds %d radius %f\n", &env[w].nroundobst, &env[w].radius);
        for (i=0; i < env[w].nroundobst; i++)
          fscanf(fp,"%f %f\n", &env[w].roundobst[i][0],&env[w].roundobst[i][1]); 
  	// small round obstacles
        fscanf(fp,"srounds %d radius %f\n", &env[w].nsroundobst, &env[w].sradius);      
        for (i=0; i < env[w].nsroundobst; i++)
          {
           fscanf(fp,"%f %f\n", &env[w].sroundobst[i][0],&env[w].sroundobst[i][1]);
           env[w].sroundobst[i][2] = (float) 0.0;       
          }
        // foodzones
        fscanf(fp,"foodzones %d\n", &env[w].nfoodzones);
	for (i=0; i < env[w].nfoodzones; i++)
          fscanf(fp,"%f %f %f\n",
             &env[w].foodzones[i][0],&env[w].foodzones[i][1],&env[w].foodzones[i][2]);
        // lights
        fscanf(fp,"lights %d\n", &env[w].nbulblights);
        for (i=0; i < env[w].nbulblights; i++)
          fscanf(fp,"%f %f\n", &env[w].bulblights[i][0],&env[w].bulblights[i][1]);              
        // starting positions
        fscanf(fp,"starts %d\n", &env[w].nstarts);
        for (i=0; i < env[w].nstarts; i++) 
         {
          for (ii=0; ii < 3; ii++)        
            fscanf(fp,"%f", &env[w].starts[i][ii]);	 
            fscanf(fp,"\n");	     
         }
        // landmarks
        fscanf(fp,"landmarks %d\n", &env[w].nlandmarks);
        for (i=0; i < env[w].nlandmarks; i++) 
         {    
            fscanf(fp,"%f %f %f %f %f", &env[w].landmark[i][0],&env[w].landmark[i][1],&env[w].landmark[i][2],&env[w].landmark[i][3],&env[w].landmark[i][4]);	 
            fscanf(fp,"\n");	     
         }
        fscanf(fp, "%s",sbuffer);
        if (strcmp(sbuffer,"END") != 0)
	   display_warning("unable to find END in the world's file");

        fclose(fp);
	realnworlds++;
       }
      else
       {
	return(-1);
       }
  }
   nworlds = realnworlds;
   return(1);
}


/* 
 * it load files world.N (0 < N < 5)
 * 
 */
void
save_world()


{

   FILE *fopen();
   FILE *fp;
   int    i,ii;
   int    w;
   int    l;
   char   sbuffer[256];
   char   worldname[256];
   char   rootname[256];
   
   l = strlen(worldfiles);
   for(i=0;i < (l - 5); i++)
   rootname[i] = worldfiles[i];
   rootname[i] = '\0';
   for(w=0;w<nworlds;w++)
   {
     env[w].dx = 0;
     env[w].dy = 0;
     sprintf(worldname,"%s%d.env",rootname,w+1);
     if ((fp = fopen(worldname,"w")) != NULL)
      {
	// walls
        fprintf(fp,"walls %d\n", env[w].nobstacles);
        for (i=0; i < env[w].nobstacles; i++)
         { 
          fprintf(fp,"%.1f %.1f %.1f %.1f\n",
            env[w].obstacles[i][0],env[w].obstacles[i][1],env[w].obstacles[i][2],env[w].obstacles[i][3]);
          }
	// round obstacles
        fprintf(fp,"rounds %d radius %.1f\n", env[w].nroundobst, env[w].radius);
        for (i=0; i < env[w].nroundobst; i++)
          fprintf(fp,"%.1f %.1f\n", env[w].roundobst[i][0],env[w].roundobst[i][1]); 
  	// small round obstacles
        fprintf(fp,"srounds %d radius %.1f\n", env[w].nsroundobst, env[w].sradius);      
        for (i=0; i < env[w].nsroundobst; i++)
          {
           fprintf(fp,"%.1f %.1f\n", env[w].sroundobst[i][0],env[w].sroundobst[i][1]);
          }
        // foodzones
        fprintf(fp,"foodzones %d\n", env[w].nfoodzones);
	for (i=0; i < env[w].nfoodzones; i++)
          fprintf(fp,"%.1f %.1f %.1f\n",
             env[w].foodzones[i][0],env[w].foodzones[i][1],env[w].foodzones[i][2]);
        // lights
        fprintf(fp,"lights %d\n", env[w].nbulblights);
        for (i=0; i < env[w].nbulblights; i++)
          fprintf(fp,"%.1f %.1f\n", env[w].bulblights[i][0],env[w].bulblights[i][1]);              
        // starting positions
        fprintf(fp,"starts %d\n", env[w].nstarts);
        for (i=0; i < env[w].nstarts; i++) 
         {
          for (ii=0; ii < 3; ii++)        
            fprintf(fp,"%.1f ", env[w].starts[i][ii]);	 
            fprintf(fp,"\n");	     
         }
        // landmarks
        fprintf(fp,"landmarks %d\n", env[w].nlandmarks);
        for (i=0; i < env[w].nlandmarks; i++) 
         {    
            fprintf(fp,"%.1f %.1f %.1f %.1f %.1f", env[w].landmark[i][0],env[w].landmark[i][1],env[w].landmark[i][2],env[w].landmark[i][3],env[w].landmark[i][4]);	 
            fprintf(fp,"\n");	     
         }
        fprintf(fp, "END",sbuffer);

        fclose(fp);
       }
      else
       {
	  sprintf(sbuffer,"unable to save the environment %s",worldname);
	  display_warning(sbuffer);
       }
  }
}



/* 
 * load an obstacle from a .sam file 
 */
void
load_obstacle(char *filename, int **object, int  *objectcf)

{

   extern int strcmp();
   int  **ob;
   int  *o;
   FILE *fopen();
   FILE *fp;
   int  i,v,s;
   char sbuffer[64];

   if ((fp = fopen(filename,"r")) != NULL)
	  {
	  }
	  else
	  {
	    sprintf(sbuffer,"I cannot open file %s", filename);
	    display_error(sbuffer);	      	      
	  }   
   /*
    * we read the object configuration data
    * 0-nsensors, 1-nangles, 2-ndistances,3-initdist,4-distinterval
    */
   for (i=0, o=objectcf; i < 5; i++, o++)
     fscanf(fp,"%d ",o);
   fscanf(fp,"\n",o);

   for (v=0, ob=object; v < objectcf[2]; v++, ob++)
    {
     fscanf(fp,"TURN %d\n",&v);
     for (i = 0, o = *ob; i < objectcf[1]; i++)
      {
       for (s=0; s < objectcf[0]; s++,o++)
        {
	   fscanf(fp,"%d ",o);   
        }
       fscanf(fp,"\n");
      }
    }
   fscanf(fp, "%s\n",sbuffer);
   if (strcmp(sbuffer,"END") != 0)
     {
      sprintf(sbuffer,"loading file %s: file might be inconsistent", filename);
      display_error(sbuffer);
     }
   fflush(fp);    
   fclose(fp);
}


/* 
 * save an obstacle in a .sam file 
 */
void
save_obstacle(char *filename, int **object, int  *objectcf)

{

   extern int strcmp();
   int  **ob;
   int  *o;
   FILE *fopen();
   FILE *fp;
   int  i,v,s;
   char sbuffer[64];

   if ((fp = fopen(filename,"w")) != NULL)
	  {
	  }
	  else
	  {
	    sprintf(sbuffer,"I cannot open file %s", filename);
	    display_error(sbuffer);	      	      
	  }   
   /*
    * we write the object configuration data
    * 0-nsensors, 1-nangles, 2-ndistances,3-initdist,4-distinterval
    */
   for (i=0, o=objectcf; i < 5; i++, o++)
     fprintf(fp,"%d ",*o);
   fprintf(fp,"\n");

   for (v=0, ob=object; v < objectcf[2]; v++, ob++)
    {
     fprintf(fp,"TURN %d\n",v);
     for (i = 0, o = *ob; i < objectcf[1]; i++)
      {
       for (s=0; s < objectcf[0]; s++,o++)
        {
	   fprintf(fp,"%d ",*o);   
        }
       fprintf(fp,"\n");
      }
    }
   fprintf(fp, "END\n");
   fflush(fp);    
   fclose(fp);
}


/* 
 * load motor samples
 * the first line should be the number of samples     
 * for each sample we compute 3 simmitrical
 */
void
load_motor()

{

   extern  float **motor;
   float  **mo;
   float  *m;
   FILE   *fopen();
   FILE   *fp;
   int    v,vv;

   if ((fp = fopen("motor.sam","r")) != NULL)
	  {
	  }
	  else
	  {
	    display_error("cannot open file motor.cam");
	  }
   for (v=0,mo = motor; v < 25; v++,mo++)
    {  
     for (vv=0,m = *mo; vv < 25; vv++,m++)
      {
        fscanf(fp,"%f\t",m);
        m++;
      }
      fscanf(fp,"\n");
    }
   for (v=0,mo = motor; v < 25; v++,mo++)
    {  
     for (vv=0,m = *mo; vv < 25; vv++,m++)
      {
        m++;
        fscanf(fp,"%f\t",m);
      }
      fscanf(fp,"\n");
    }
   fclose(fp);
}



/*
 * return the right output taking into account modularity
 * set oselected to the corresponding number
 * works only for 1 or 2 modules
 */

float
oselect(struct individual *pind, int out)
 
{
	               

     if (pind->modularity == 0)
      {
        oselected = 0;
        return(pind->layerp[pind->nlayers - 1][out]);    
      }     
     if (pind->modularity == 1)
      {
       if (pind->ob_taken == 0)
        {
         oselected = 0;
         return(pind->layerp[pind->nlayers - 1][out]);    
        }
        else
        {
         oselected = 1;
         return(pind->layerp[pind->nlayers - 1][out + pind->modulskip]);    
        }
      }
     if (pind->modularity == 2)
      {
	if (pind->alayerp[pind->nlayers - 1][out] >= pind->alayerp[pind->nlayers-1][out + pind->modulskip])
	 {
          oselected = 0;	    
          return(pind->layerp[pind->nlayers - 1][out]);
         }
        else
	 {
          oselected = 1;	    
          return(pind->layerp[pind->nlayers - 1][out + pind->modulskip]);         
	 }
      }
   return((float) 0.0);
}

/*
 * set sensor activations
 */
void
set_input(struct individual *pind)

{

  extern double sqrt();
  extern double fabs();
  extern double sin();        
  extern double cos();          
  extern double exp();
  float  p1; 
  float  p2; 
  double direction;
  int    i,ii;
  int    g;
  float  x,y;
  double dist;
  double  angle; 
  int     relang;
  int     reldist;
  int     nsensor;
  int     *w;
  int     *o;
  int     *ldp;
  double  ang;
  struct individual *cind;
  double max_dist_l;
  int    most_dist_l;
  float  land_first[2];
  float  land_second[2];
  double dist_first;
  double dist_second;
  double ang_first;
  double ang_second;
  double delta_ang1;
  double delta_ang2;
  double ang_d;

  p1        = pind->pos[0]; 
  p2        = pind->pos[1];
  direction = pind->direction; 

  nsensor = 0;

  // 0=wall,1=round,2=smallr,3=other khepera
  for(i=0;i<4;i++)
    o_perceived[i] = 0; 

  for(i=0;i<8;i++)
    {
      pind->cifsensors[i]    = 0.0f;
      pind->clightsensors[i] = 0.0f;
    }

  radarmap[1][0] = 0;
  radarmap[1][3] = pind->ob_taken;  

  /*
   * infrared sensors from walls
   */
  for(i=0;i < env[cworld].nobstacles;i++)
   {
    if (p1 >= env[cworld].obstacles[i][0] && p1 <= env[cworld].obstacles[i][2])
       x = p1;
     else
       x = env[cworld].obstacles[i][0];
    if (p2 >= env[cworld].obstacles[i][1] && p2 <= env[cworld].obstacles[i][3])
       y = p2;
     else
       y = env[cworld].obstacles[i][1];
    dist = mdist((float)p1,(float)p2,(float)x,(float)y); 
    if (dist < (wallcf[3] + 27.5 + (wallcf[2] * wallcf[4])))     // min.dist + khepera rad. + max. dist 
     {                     // 5 + 27.5 + 40 = 70.5 */
       if (o_perceived[0] == 0)
         o_perceived[0] = 1;
        else
	 o_perceived[0] = 2;
       if (x == p1)
         {
	 if (y < p2)
           angle = 270.0;
          else
           angle = 90.0;
         }
         else
         {
         if (x < p1)
           angle = 180.0;
          else
           angle = 0.0;
         }
       relang  = (int) (mangrel(angle,direction) / 2.0);
       dist -= 27.5;          /* khepera rad. = 27.5 */
       if (dist < wallcf[3])        /* min. dist. = 5.0 */
         dist = wallcf[3]; 
       reldist = (int) (dist - wallcf[3]);  /* 5 for addressing */
       reldist = (int) reldist / wallcf[4]; /* for addressing each 2 millimeters */

       radarmap[1][0] = 1;
       radarmap[1][1] = relang;
       radarmap[1][2] = reldist;
        
       w = *(wall + reldist);
       w = (w + (relang * 8));
       for(ii=0;ii<8;ii++,w++)
          pind->cifsensors[ii] += (*w / (float) 1024.0);
     } 
   } 

  /*
   * infrared sensors from round obstacles
   */
  o_perceived[1] = 0;       
  for(i=0;i< env[cworld].nroundobst;i++)
   {
    dist = mdist((float)p1,(float)p2,(float)env[cworld].roundobst[i][0],(float)env[cworld].roundobst[i][1]) - env[cworld].radius; 
    ang = mang((int)p1,(int)p2,(int)env[cworld].roundobst[i][0],(int)env[cworld].roundobst[i][1],(double)dist+env[cworld].radius);
    relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
    if (dist < (obstcf[3] + 27.5 + (obstcf[2] * obstcf[4])))       // min.dist + khepera rad. + (dists*inter) 
     {                     // 72.5 = 5 + 27.5 + 40 = 72.5 
       
       if (o_perceived[1] == 0)
         o_perceived[1] = 1;
        else
	 o_perceived[1] = 2;    
        
       ang = mang((int)p1,(int)p2,(int)env[cworld].roundobst[i][0],(int)env[cworld].roundobst[i][1],(double)dist+env[cworld].radius);
       relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
       dist -= 27.5;          // khepera rad. 
       if (dist < obstcf[3])  // min. dist. = 5.0 
         dist = obstcf[3]; 
       reldist = (int) (dist - obstcf[3]);  // 5 for addressing 
       reldist = (int) reldist / obstcf[4]; // for addressing each 2 millimeters

       radarmap[1][0] = 2;
       radarmap[1][1] = relang;
       radarmap[1][2] = reldist;
      
       o = *(obst + reldist);
       o = (o + (relang * 8));
       for(ii=0;ii<8;ii++,o++)
	  pind->cifsensors[ii] += (*o / (float) 1024.0);
     } 
   }


  /*
   * activate sensors from small round obstacles if not taken
   */   
  o_perceived[2] = 0;       
  for(i=0;i<env[cworld].nsroundobst;i++)
   {
    if (env[cworld].sroundobst[i][2] == 0.0)
     {
     dist = mdist((float)p1,(float)p2,(float)env[cworld].sroundobst[i][0],(float)env[cworld].sroundobst[i][1]) - env[cworld].sradius; 
     if (dist < (sobstcf[3] + 27.5 + (sobstcf[2] * sobstcf[4])))       // min.dist + khepera rad. + max. dist 
      {                     // 5 + 27.5 + 40 = 72.5 
       if (o_perceived[2] == 0)
         o_perceived[2] = 1;
        else
	 o_perceived[2] = 2;    	     

       ang = mang((int)p1,(int)p2,(int)env[cworld].sroundobst[i][0],(int)env[cworld].sroundobst[i][1],(double)dist+env[cworld].sradius);
       relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
       dist -= 27.5;          /* khepera rad. = 27.5 */
       if (dist < sobstcf[3])        /* min. dist. = 5.0 */
         dist = sobstcf[3]; 
       reldist = (int) (dist - sobstcf[3]);  /* for addressing */
       reldist = (int) reldist / (int) sobstcf[4]; /* for addressing each 2 millimeters */       
 
       radarmap[1][0] = 3;
       radarmap[1][1] = relang;
       radarmap[1][2] = reldist;

       o = *(sobst + reldist);
       o = (o + (relang * 8));
       for(ii=0;ii<8;ii++,o++)
          pind->cifsensors[ii] += (*o / (float) 1024.0);
      } 
     }
   }


  /*
   * activate sensors from other individuals
   * for the other khepera we used the sample sample of round obstacles
   */
  o_perceived[3] = 0;       
  for(i=0,cind=ind;i< npopulations;i++,cind++)
   {
    if ((p1 != cind->pos[0] || p2 != cind->pos[1]))
    {     
     dist = mdist((float)p1,(float)p2,(float) cind->pos[0],(float)cind->pos[1]) - 27.5; 
     ang = mang((int)p1,(int)p2,(int)cind->pos[0],(int)cind->pos[1],(double)dist+27.5);
     relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
     oppangle = relang * 2.0;
     oppdist  =  dist;
     if (dist < (obstcf[3] + 27.5 + (obstcf[2] * obstcf[4])))       
     {                     
       if (o_perceived[3] == 0)
         o_perceived[3] = 1;
        else
	 o_perceived[3] = 2;      
        
       ang = mang((int)p1,(int)p2,(int)cind->pos[0],(int)cind->pos[1],(double)dist+27.5);
       relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
       oppangle = relang;     
       dist -= 27.5;          
       if (dist < obstcf[3])        
         dist = obstcf[3]; 
       reldist = (int) (dist - obstcf[3]); 
       reldist = (int) reldist / (int) obstcf[4]; 

       radarmap[1][0] = 2;
       radarmap[1][1] = relang;
       radarmap[1][2] = reldist;
    
       o = *(obst + reldist);
       o = (o + (relang * 8));
       for(ii=0;ii<8;ii++,o++)
	  pind->cifsensors[ii] += (*o / (float) 1024.0);
     }
    }
   }

   /*
    * we check range
    */
   for(ii = 0; ii < 8; ii++)
    {
      if (pind->cifsensors[ii] > 1.0)
         pind->cifsensors[ii] = 1.0f; 
    }

   /*
    * add noise to infra-red sensors if requested
    * if ob_taken == 1 extra noise is present
    */
   if (sensornoise > 0.0)
     for(i=0; i < 8; i++)
	{
 	 pind->cifsensors[i] += rans(sensornoise);	
         if (pind->cifsensors[i] < 0.0) pind->cifsensors[i] = (float) 0.0;
         if (pind->cifsensors[i] > 1.0) pind->cifsensors[i] = (float) 1.0;
        }

   /*
    * we copy 4, 6, or 8 if sensors on the input vector
    */
   if (pind->nifsensors == 4)
    {
      input[0] = (pind->cifsensors[0] + pind->cifsensors[1]) / (float) 2.0;	
      input[1] = pind->cifsensors[2];	
      input[2] = pind->cifsensors[3];	
      input[3] = (pind->cifsensors[4] + pind->cifsensors[5]) / (float) 2.0;
      nsensor += 4;
    } 
   if (pind->nifsensors == 6)
    {
      for(ii=0;ii<6;ii++,nsensor++)
         input[nsensor] = pind->cifsensors[ii];   
    } 
   if (pind->nifsensors == 8)
    {
      for(ii=0;ii<8;ii++,nsensor++)
         input[nsensor] = pind->cifsensors[ii];   
    } 

  /*
   * activate light sensors 
   */
  if (pind->lightsensors == 1 || pind->light2sensors == 1)
  {
   // from the other individual
   if (npopulations > 1 && pind->oppsensor == 1)
   for(i=0,cind=ind;i< npopulations;i++,cind++)
   {
    if (p1 != cind->pos[0] || p2 != cind->pos[1])
    {     
     dist = mdist((float)p1,(float)p2,(float) cind->pos[0],(float)cind->pos[1]); 
     ang = mang((int)p1,(int)p2,(int)cind->pos[0],(int)cind->pos[1],(double)dist);
     relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
     if (dist <= (lightcf[3] + 27 + (lightcf[2] * lightcf[4])))    // 560 = 13 + 27 + 520 (26*20)  
     { 
       dist  -= 27;                         // khepera rad
       if (dist < lightcf[3])
	 dist = lightcf[3];       
       reldist = (int) (dist - lightcf[3]); // 13 for adressing
       reldist = (int) (dist / lightcf[4]); // for addressing each 20mm
       o = *(light + reldist); 
       o = (o + (relang * 8));
       for(ii=0;ii<8;ii++,o++)
             pind->clightsensors[ii] += (*o / (float) 1024.0);
     }
    }
   }
   // from light bulbs
   for(i=0;i< env[cworld].nbulblights;i++)
   {   
     dist = mdist((float)p1,(float)p2,(float) env[cworld].bulblights[i][0],(float)env[cworld].bulblights[i][1]); 
     ang = mang((int)p1,(int)p2,(int)env[cworld].bulblights[i][0],(int)env[cworld].bulblights[i][1],(double)dist);
     relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
     if (dist <= (lightcf[3] + 27 + (lightcf[2] * lightcf[4])))    // 560 = 13 + 27 + 520 (26*20)  
     {
       dist  -= 27;                         // khepera rad
       if (dist < lightcf[3])
	 dist = lightcf[3];       
       reldist = (int) (dist - lightcf[3]); // 13 for adressing
       reldist = (int) (dist / lightcf[4]); // for addressing each 20mm       o = *(light + reldist); 
       o = *(light + reldist); 
       o = (o + (relang * 8));
       for(ii=0;ii<8;ii++,o++)
         pind->clightsensors[ii] += (*o / (float) 1024.0);	
     }
   }


   /*
    * we check range
    */
   for(ii = 0; ii < 8; ii++)
     {
      if (pind->clightsensors[ii] > 1.0)
         pind->clightsensors[ii] = 1.0f; 
     }
   /*
    * add noise to infra-red sensors if requested
    * if ob_taken == 1 extra noise is present
    */
   if (sensornoise > 0.0)
     for(i=0; i < 8; i++)
	{
 	 pind->clightsensors[i] += rans(sensornoise);	
         if (pind->clightsensors[i] < 0.0) pind->clightsensors[i] = (float) 0.0;
         if (pind->clightsensors[i] > 1.0) pind->clightsensors[i] = (float) 1.0;
     }
   /*
    * we copy 2 or 8 sesors on the input vector
    */
   if (pind->light2sensors == 1)
    {
       input[nsensor] = pind->clightsensors[2];
       nsensor++;
       input[nsensor] = pind->clightsensors[6];   
       nsensor++;
    } 
   if (pind->lightsensors == 1)
    {
      for(ii=0;ii<8;ii++,nsensor++)
         input[nsensor] = pind->clightsensors[ii];   
    } 
  }


 /* 
  * camera (32 pixels mode)
  * 1) we set the default value (background)
  * 2) we compute the distance of each landmark (ball is the last one)
  * 3) we update the virtual camera starting with the more distant objects 
  * 4) we compute the real-camera 
  */
 if (pind->cameraunits > 0)
  {
    // background
    for(i=0;i<320;i++)
	pind->virtual_camera[i] = background_color;

    // we compute the distances (between the center of khepera and the landmarks)
    for (i=0; i <= env[cworld].nlandmarks; i++)
       env[cworld].landmark[i][5] = (float) mdist(pind->pos[0], pind->pos[1],env[cworld].landmark[i][0], env[cworld].landmark[i][1]); 

    // we look for the most distant and we update the virtual camera
    // after updating we set the distance to -1.0 (which means now ignore it)
    for (i=0; i <= env[cworld].nlandmarks; i++)
    {
      max_dist_l  = -1.0;
      most_dist_l = -1;
      for (ii=0; ii <= env[cworld].nlandmarks; ii++)
      {
        if (env[cworld].landmark[ii][5] > max_dist_l)
	 {
	   max_dist_l = env[cworld].landmark[ii][5];
	   most_dist_l = ii;
	 }
      }
      if (most_dist_l >= 0)
       {
	  // cylinder
	  // the two points are orthogonal to the line between the robot and the cylinder
          if (env[cworld].landmark[most_dist_l][3] == 0.0)
	   {
	    dist           = mdist(pind->pos[0],pind->pos[1], env[cworld].landmark[most_dist_l][0], env[cworld].landmark[most_dist_l][1]); 
            ang            = mang((int) pind->pos[0],(int) pind->pos[1], (int) env[cworld].landmark[most_dist_l][0], (int) env[cworld].landmark[most_dist_l][1], dist);
            land_first[0]  = SetX(env[cworld].landmark[most_dist_l][0],env[cworld].landmark[most_dist_l][2], (double) ang + 90.0);  
            land_first[1]  = SetY(env[cworld].landmark[most_dist_l][1],env[cworld].landmark[most_dist_l][2], (double) ang + 90.0);  
            land_second[0] = SetX(env[cworld].landmark[most_dist_l][0],env[cworld].landmark[most_dist_l][2], (double) ang + 270.0);  
            land_second[1] = SetY(env[cworld].landmark[most_dist_l][1],env[cworld].landmark[most_dist_l][2], (double) ang + 270.0);  
	   }
	  // orizontal surface
          if (env[cworld].landmark[most_dist_l][3] == 1.0)
	   {
            land_first[0]   = env[cworld].landmark[most_dist_l][0] - env[cworld].landmark[most_dist_l][2];
	    land_first[1]   = env[cworld].landmark[most_dist_l][1];
            land_second[0]  = env[cworld].landmark[most_dist_l][0] + env[cworld].landmark[most_dist_l][2];
	    land_second[1]  = env[cworld].landmark[most_dist_l][1];
	   }
	  // vertical surface
          if (env[cworld].landmark[most_dist_l][3] == 2.0)
	  {
            land_first[0]   = env[cworld].landmark[most_dist_l][0];
	    land_first[1]   = env[cworld].landmark[most_dist_l][1] - env[cworld].landmark[most_dist_l][2];
            land_second[0]  = env[cworld].landmark[most_dist_l][0] ;
	    land_second[1]  = env[cworld].landmark[most_dist_l][1] + env[cworld].landmark[most_dist_l][2];
	  }

	  dist_first      = mdist(pind->pos[0],pind->pos[1], land_first[0], land_first[1]);
	  dist_second     = mdist(pind->pos[0],pind->pos[1], land_second[0], land_second[1]);
          ang_first       = mang((int) pind->pos[0],(int) pind->pos[1], (int) land_first[0], (int) land_first[1], dist_first);
          ang_second      = mang((int) pind->pos[0],(int) pind->pos[1], (int) land_second[0], (int) land_second[1], dist_second);

	  /*
	   * we chose the smallest arc between the two possibilities
	   */
	  delta_ang1 = ang_second - ang_first;
	  if (delta_ang1 > 360.0)
	    delta_ang1 -= 360.0;
	  if (delta_ang1 < 0.0)
	    delta_ang1 += 360.0;
	  delta_ang2 = ang_first - ang_second;
	  if (delta_ang2 > 360.0)
	    delta_ang2 -= 360.0;
	  if (delta_ang2 < 0.0)
	    delta_ang2 += 360.0;
	  /*
	   * we set the real camera converting 360 degrees into 320 virtual pixels moltiplying for 0.888
	   */
	  if (delta_ang1 < delta_ang2)
	     {
	      for (ii = (int) ang_first; ii <= (int) (ang_first + delta_ang1) ;ii++)
		if (ii < 360)
	          pind->virtual_camera[(int) ((double)ii * 0.888)]     = env[cworld].landmark[most_dist_l][4];
	         else
	          pind->virtual_camera[(int) (((double) ii - 360.0) * 0.888)] = env[cworld].landmark[most_dist_l][4];
	     }
	     else
	     {
	      for (ii = (int) ang_second; ii <= (int) (ang_second + delta_ang2); ii++)
		if (ii < 360)
	          pind->virtual_camera[(int) ((double)ii * 0.888)]     = env[cworld].landmark[most_dist_l][4];
	         else
	          pind->virtual_camera[(int) (((double) ii - 360.0) * 0.888)] = env[cworld].landmark[most_dist_l][4];
	     }
	  
          env[cworld].landmark[most_dist_l][5] = - (float) 1.0;
       }
    }

    // we set the input values 
    ang_d = pind->direction - ((double) pind->camerarange / 2.0);
    if (ang_d < 0)
	   ang_d += 360.0;
    if (ang_d > 359)
	   ang_d -= 360.0;    
    for (i=0; i < pind->cameraunits; i++, nsensor++)
      {
       input[nsensor] = pind->virtual_camera[(int) (ang_d * 0.888)];
       pind->real_camera[i] = input[nsensor];
       if (input[nsensor] > 1.0)
		  input[nsensor] = 1.0f;
       if (input[nsensor] < 0.0)
		  input[nsensor] = 0.0f;
       ang_d += (pind->camerarange / pind->cameraunits);
       if (ang_d < 0)
	     ang_d += 360.0;
       if (ang_d > 359)
	     ang_d -= 360.0;
      }
  }

   /*
    * camera (the most activated sensor among 5 ranges)
    */
   if (pind->camerafive > 0)
     {    
      for(ii=nsensor;ii<(nsensor+5);ii++)
	 input[ii]= (float) 0.0;
      if (oppangle > 342)
       {
       	if (oppangle < 350)
         {
          input[nsensor+4]= (float) 1.0;
         }
         else
         {
          if (oppangle < 357)
           input[nsensor+3]= (float) 1.0;
          else
           input[nsensor+2]= (float) 1.0;
         } 
       }
      if (oppangle < 18)
       {
         if (oppangle < 3)
         {
           input[nsensor+2] = (float) 1.0;
         }
         else
         {
          if (oppangle < 10)    
            input[nsensor+1] = (float) 1.0;
           else
            input[nsensor+0] = (float) 1.0;
         }
       }
      pind->real_camera[0] = input[nsensor+0];
      pind->real_camera[1] = input[nsensor+1];
      pind->real_camera[2] = input[nsensor+2];
      pind->real_camera[3] = input[nsensor+3];
      pind->real_camera[4] = input[nsensor+4];
      nsensor += 5;
     }
   /*
    * newcamera (the most activated sensor among 5 ranges)
    * 220/240 degree of range
    */
   if (pind->newcamerafive > 0)
     {  

      for(ii=nsensor;ii<(nsensor+5);ii++)
	 input[ii] = (float) 0.0;

       if (pind->ffitness == 21)
       {
	 if ((cepoch % 2) == 1)
	  {
           dist = mdist((float)p1,(float)p2,(float) env[cworld].foodzones[0][0],(float) env[cworld].foodzones[0][1]) - 27.5; 
           ang = mang((int)p1,(int)p2,(int)env[cworld].foodzones[0][0],(int)env[cworld].foodzones[0][0],(double)dist+27.5);
           relang = (int) mangrel2((double)ang,(double)direction) / (int) 2.0;
           oppangle = relang * 2.0;
	  }
	  else
	  {
           oppangle = 180;
	  }
       }
      if (oppangle > 240)       /* 250-240 */
       {
       	if (oppangle < 288)     /* 295-288 */
         {
          input[nsensor+4] = (float) 1.0;
         }
         else
         {
          if (oppangle < 336)   /* 340-336 */
           input[nsensor+3] = (float) 1.0;
          else
           input[nsensor+2] = (float) 1.0;
         } 
       }
      if (oppangle < 120)       /* 110-120 */
       {
         if (oppangle < 24)     /*  20-24 */
         {
           input[nsensor+2] = (float) 1.0;
         }
         else
         {
          if (oppangle < 72)    /* 65-72 */
            input[nsensor+1] = (float) 1.0;
           else
            input[nsensor+0] = (float) 1.0;
         }
       }
      pind->real_camera[0] = input[nsensor+0];
      pind->real_camera[1] = input[nsensor+1];
      pind->real_camera[2] = input[nsensor+2];
      pind->real_camera[3] = input[nsensor+3];
      pind->real_camera[4] = input[nsensor+4];
      nsensor += 5;
     }     
   
   /*
    * light-barrier sensor on the gripper
    */
   if (pind->objectsensor > 0)
     {
      input[nsensor]   = (float) pind->ob_taken;
      nsensor += 1;
     }     

   /*
    * if we have a ground sensor we activate it
    */ 
   if (pind->groundsensor == 1)
     {
       g = read_ground(pind);
       if (g > 0)
         input[nsensor] = (float) 1.0;
	else
         input[nsensor] = (float) 0.0;	
       pind->bottomsensor = input[nsensor];
       nsensor += 1;
     }    

    /*
    * if we have an energy sensor we activate it
    */ 
   if (pind->battery == 1)
     {
       input[nsensor] = pind->energy;	
       nsensor += 1;
     }    

   /*
    * set motor sensors (copy of the speed motor outputs)
    */
   if (pind->motmemory > 0)
     {
       input[nsensor]   = oselect(pind, 0);
       input[nsensor+1] = oselect(pind, 1);
       nsensor += 2;
     }  

   /*
    * set elman network sensors
    */
   if (pind->elman_net == 1 && pind->nlayers > 2)
     {
      ldp = pind->layer_descp;
      ldp++;
      for (i=0;i < *ldp; i++,nsensor++)
         if (pind->lifecycle == 0)
           input[nsensor] = (float) 0.0;
          else
           input[nsensor] = pind->layerp[1][i]; 
     }  
   /*
    * copy recurrent pseudo-output into input
    */     
   if (pind->input_outputs > 0)
    {
      for (i=0;i < pind->input_outputs; i++,nsensor++)
	   input[nsensor] = oselect(pind, ((pind->wheelmotors * 2) + (pind->gripper * 2) + i));
    }
 

}


/*
 * in simulation,
 *  return in which foodzone the robot is located (starting from 1)
 *  return 0 if is located outside all foodzones
 * in the real_mode,
 *  return 1 if the background is dark
 */
int
read_ground(struct individual *pind)

{

   extern double sqrt();
   double distzone;
   int z;
   float p1; 
   float p2;

   if (sim_real == 0)
    {
     p1 = pind->pos[0];
     p2 = pind->pos[1];
     if (env[cworld].nfoodzones > 0)
      {
       for(z=0; z < env[cworld].nfoodzones; z++)
        {
         distzone = (p1 - env[cworld].foodzones[z][0])*(p1 - env[cworld].foodzones[z][0])+
                    (p2 - env[cworld].foodzones[z][1])*(p2 - env[cworld].foodzones[z][1]);
         distzone = sqrt(distzone);
         if (distzone < env[cworld].foodzones[z][2])
	        return(z+1);
        }
      }     
     return(0);
    }
    else
    {
      if (pind->bottomsensor > 0.5)
	 return(1);
        else
	 return(0);
    }
}


/*
 * receives kepera position and checks if a crash occurred
 */
int
check_crash(float p1,float p2)

{
  int i;
  float   x,y;
  double  dist;
  double  mindist;
  int     p;
  struct individual *pind;

  mindist = 9999.0;
  for(i=0;i<env[cworld].nobstacles;i++)
   {
    if (p1 >= env[cworld].obstacles[i][0] && p1 <= env[cworld].obstacles[i][2])
       x = p1;
     else
       x = env[cworld].obstacles[i][0];
    if (p2 >= env[cworld].obstacles[i][1] && p2 <= env[cworld].obstacles[i][3])
       y = p2;
     else
       y = env[cworld].obstacles[i][1];
    dist = mdist(p1,p2,x,y);
    if (dist < mindist)
      mindist = dist;
   }
  for(i=0;i<env[cworld].nroundobst;i++)
   {
    dist = mdist(p1,p2,env[cworld].roundobst[i][0],env[cworld].roundobst[i][1])-env[cworld].radius;
    if (dist < mindist)
      mindist = dist;
   }
  for(i=0;i<env[cworld].nsroundobst;i++)
    {
     if (env[cworld].sroundobst[i][2] == 0.0)	
      {
       dist = mdist(p1,p2,env[cworld].sroundobst[i][0],env[cworld].sroundobst[i][1])-env[cworld].sradius;
       if (dist < mindist)
	 mindist = dist;
      }
    }
   /*
    * check if a crash occurred with another individuals
    * note; it do not take into account individuals
    * that have the same idential positions (i.e. itself)
    */
   if (npopulations > 1)
    {
     for (p=0, pind = ind; p < npopulations; p++, pind++)
      {
	if (p1 != pind->pos[0] || p2 != pind->pos[1])
	 {
          dist = mdist(p1,p2,pind->pos[0],pind->pos[1])-27.5;
          if (dist < mindist)
	   {
	    mindist = dist;
	    if (dist < 27.5)
	     {
	      individual_crash = 1;
	     }
	   }
	 }
      }
    }
    
   if (mindist < 27.5)
     return(1);
    else
     return(0);
}



/*
 * compute distance between the two gripper's hand and the target (ta)
 */
void
check_hand(int ta, float x,float y,float  grip, double direction)

{

     double  ldirection;
     double  rdirection;     
     float   headx,heady;
     float   leftx,lefty;          
     float   rightx,righty;               
     float   fdeltax,fdeltay;
     float   flx,fly;
     float   frx,fry;     
     float   lgripper[4];
     float   rgripper[4];  

     /*
      * we compute the central points of the square
      */
     ldirection = direction + 90.0;
     if (ldirection > 359.0)
	ldirection -= 360.0;
     rdirection = direction - 90.0;
     if (rdirection < 0.0)
	rdirection += 360.0; 
    
     headx  = SetX(x,(float) 42.5,direction);
     heady  = SetY(y,(float) 42.5,direction);
     rightx = SetX(x,(float) 35.0,rdirection);
     righty = SetY(y,(float) 35.0,rdirection);
     leftx  = SetX(x,(float) 35.0,ldirection);
     lefty  = SetY(y,(float) 35.0,ldirection);          
     
     /*
      * we compute the vertices
      */     
     fdeltax = (headx - x);
     fdeltay = (heady - y);          
     flx  =  leftx + fdeltax;   
     fly  =  lefty + fdeltay;
     frx  =  rightx + fdeltax;   
     fry  =  righty + fdeltay;                  

     /*
      * we compute the gripper vertices
      * old gripper had [+ 0.5]
      */          
     grip    = (float) ((grip / 2.0) + 0.25);
     fdeltax = (float) (((flx - frx) / 2.5) * grip);
     fdeltay = (float) (((fly - fry) / 2.5) * grip);


     lgripper[0]  = flx - fdeltax;
     lgripper[1]  = fly - fdeltay;
     lgripper[2]  = SetX(lgripper[0],(float) 35,direction);
     lgripper[3]  = SetY(lgripper[1],(float) 35,direction);

     rgripper[0]  = frx + fdeltax;
     rgripper[1]  = fry + fdeltay;
     rgripper[2]  = SetX(rgripper[0],(float) 35,direction);
     rgripper[3]  = SetY(rgripper[1],(float) 35,direction);
     
     fdist = linedist(env[cworld].sroundobst[ta][0],env[cworld].sroundobst[ta][1],flx,fly,frx,fry);
     ldist = linedist(env[cworld].sroundobst[ta][0],env[cworld].sroundobst[ta][1],lgripper[0],lgripper[1],lgripper[2],lgripper[3]);
     rdist = linedist(env[cworld].sroundobst[ta][0],env[cworld].sroundobst[ta][1],rgripper[0],rgripper[1],rgripper[2],rgripper[3]);

}


/*
 *  calculate the position of the arm and return the fitness gathered 
 *  move kepera arm and gripper
 *
 *  for each target check if crashed have occurred and compute
 *  the fitness gathered
 */
float
check_arm(struct individual *pind)

{

     extern double fabs();
     float   headx,heady;     
     float   value;     
     float   fit;
     int     i,tt,ta;
     float   xx,yy;     
     int     crash;
     int     outworld;
     int     justaken=0;
     int     conta;
     int     ontop;
     float   x;
     float   y;
     float   arm;
     float   released; 
     double  direction;

     x  = pind->pos[0];
     y  = pind->pos[1];
     arm = oselect(pind, 2);
     released = oselect(pind, 3);
     direction = pind->direction;

     crash = 0;
     ob_sensed = (float) 0.0;
     fit       = (float) 0.0;
     justaken = 0;

     /*
      * move backward when the arm is going down
      */
     if (pind->p_arm < 0.75 && arm > 0.75)
	 {
          setpos(pind,(float) 0.0, (float) 0.0);
          setpos(pind,(float) 0.0, (float) 0.0);
	  x = pind->pos[0];
	  y = pind->pos[1];	  
	 }               
     headx  = SetX(x,(float) 72.5,direction);
     heady  = SetY(y,(float) 72.5,direction);	 
     /*
      * for each target we check crashes and we compute fitness
      */	 
     for (ta=0;ta<env[cworld].nsroundobst;ta++)
     {
      if (env[cworld].sroundobst[ta][2] == 0.0)
       {
        value = (float) mdist(headx,heady,env[cworld].sroundobst[ta][0],env[cworld].sroundobst[ta][1]);     
        /* 
	 * crash because kepera is moving with the arm down to the target
	 * kepera is re-positioned in this case
	 */
	if (pind->ob_taken==0 && arm>0.5 && pind->p_arm>0.5 && (value>20 && value<50))
	 {
	   crash = 1;
	 }
	/*
	 * simulated crash because the arm is going down in incorrect position
	 * kepera is re-positioned in this case
	 */
	if (pind->ob_taken == 0 && (arm > 0.75 && pind->p_arm < 0.75) && value < 50)
	 {
	   check_hand(ta,x,y,(float) 0.0,direction);
	   if (fdist < 12.5 || rdist < 12.5 || ldist < 12.5 || rdist > 40 || ldist > 40)
	   crash = 1;
	 }
	/*
	 * simulated crash because the arm is down and crash into the walls
	 */
        if (arm >= 0.75)
         {
         for(i=0;i<env[cworld].nobstacles;i++)
          {
          if (headx >= env[cworld].obstacles[i][0] && headx <= env[cworld].obstacles[i][2])
            xx = headx;
           else
            xx = env[cworld].obstacles[i][0];
          if (heady >= env[cworld].obstacles[i][1] && heady <= env[cworld].obstacles[i][3])
            yy = heady;
	   else
            yy = env[cworld].obstacles[i][1];
          if  (mdist(headx,heady,xx,yy) < 40.0)
	     crash = 2;
          }
         }
	 if (crash == 1 || crash == 2)
	 {
	     fit -= (float) 1.0;
 	     return(fit);	
	 }
	/*
         * we compute the fitness for targets not taken
         */
        if (crash == 0 && pind->ob_taken == 0)
        { 
          if (value < 50.0)
           {
            check_hand(ta,x,y,(float) 0.0,direction);  	      	     	  
            if (arm > 0.75)
	      {
	       pind->ob_taken = 1;
               ntaken++;
               justaken = 1;
	       fit += (float) 1.0;
	       /* 
		* we change a little bit obstacle position
		*/
               if (reappears == 1)
	       {
                conta = 0;		
                env[cworld].xtarget[0] = (float) 2.0;
                env[cworld].xtarget[1] = env[cworld].sroundobst[ta][0];		
                env[cworld].xtarget[2] = env[cworld].sroundobst[ta][1];				
                env[cworld].sroundobst[ta][0] += mrand(100) - 50;	       
                env[cworld].sroundobst[ta][1] += mrand(100) - 50;	       	       
                while (check_crash(x,y)== 1 && conta < 100)
	         {
                  env[cworld].sroundobst[ta][0] += mrand(100) - 50;	       
		  env[cworld].sroundobst[ta][1] += mrand(100) - 50;
		  conta++;
	         }
                env[cworld].xtarget[3] = env[cworld].sroundobst[ta][0];		
                env[cworld].xtarget[4] = env[cworld].sroundobst[ta][1];			 
	       }
	       else
	       {
                env[cworld].xtarget[0] = (float) 1.0;
                env[cworld].xtarget[1] = env[cworld].sroundobst[ta][0];		
                env[cworld].xtarget[2] = env[cworld].sroundobst[ta][1];	
                env[cworld].sroundobst[ta][2] = (float) 1.0;	       		   
	       }
	      }
          }
        } /* crash = 0 */
      }
     } /* for ta */
     /*
      * if a target is taken, we score the fitness
      * if the object is released in the arena both kepera and the
      *  released object are re-placed randomly
      */     
     if (pind->ob_taken == 1)
       {
         fit += (float) 1.0;    	  	 	    	     
	 outworld = 0;
         if (headx>(env[cworld].dx + 20) || headx< -20 || heady>(env[cworld].dy + 20) || heady < -20)
           {
	    outworld = 1;
	   }
         if (released > 0.5 && justaken == 0) 
	  {
	    pind->ob_taken = 0;
	    if (outworld == 1)	   
	     {
              fit += (float) 10000.0;	      
	      env[cworld].xtarget[0] = (float) 3.0;		
	      env[cworld].xtarget[3] = headx;
	      env[cworld].xtarget[4] = heady;			      
              nreleased++;
	     }
	     else
	     {
	      ontop = 0;
              for (ta=0;ta < env[cworld].nsroundobst;ta++)
	       {
                if (env[cworld].sroundobst[ta][2] == 0.0)
                 {		       
                  value = (float) mdist(headx,heady,env[cworld].sroundobst[ta][0],env[cworld].sroundobst[ta][1]);  	
		  if (value < 75.0)
		    {		      
		     ontop = 1;
		     fit = - (float) 1.0;
		    }
	          }
	        }
	      /* if reappears is off,  targets released ontop re-appears */
	      if (ontop == 1 && reappears == 0)
	       {
                 for (ta=0;ta < env[cworld].nsroundobst;ta++)		 
		   if (env[cworld].sroundobst[ta][2] == 1.0)
		      tt = ta;
                 env[cworld].sroundobst[tt][2] = (float) 0.0;	       	       		
	         env[cworld].xtarget[0] = (float) 3.0;	       
	         env[cworld].xtarget[3] = env[cworld].sroundobst[tt][0];
	         env[cworld].xtarget[4] = env[cworld].sroundobst[tt][1];
	       }
	     }
          }
    }
    
     pind->p_arm = arm;
     p_gripper = (float) 0.0; 
           
     return(fit);    
}


/*
 * move khepera by using sampling
 */
void
setpos(struct individual *pind,float o1, float o2)

{

  int    v1,v2;
  float  **mo;
  float  *m;
  float  ang, dist;

  
  pind->oldpos[0] = pind->pos[0];
  pind->oldpos[1] = pind->pos[1];
  pind->oldirection = pind->direction;
  
  // predator (ffitness 10) has half speed 
  if (pind->ffitness == 10)
  {      
   v1 = (int) (o1 * 10.0);
   v2 = (int) (o2 * 10.0);  
   if ( ((o1 * 100) - (v1 * 10)) > 5)
     v1++;
   if ( ((o2 * 100) - (v2 * 10)) > 5)
     v2++;
   v1 -= 5;
   v2 -= 5;
  }      
  else
  {
   // standard speed 
   v1 = (int) (o1 * 20.0);
   v2 = (int) (o2 * 20.0);  
   if ( ((o1 * 200) - (v1 * 20)) > 5)
     v1++;
   if ( ((o2 * 200) - (v2 * 20)) > 5)
     v2++;
   v1 -= 10;
   v2 -= 10;
  }

  pind->speed[0] = (float) v1;
  pind->speed[1] = (float) v2;

  mo = motor;
  m = *(mo + (v1 + 12));
  m = (m + ((v2 + 12) * 2));
  ang = *m;
  m++;
  dist = *m * (float) 10.0;

  pind->pos[0] = SetX(pind->pos[0],dist,pind->direction);
  pind->pos[1] = SetY(pind->pos[1],dist,pind->direction);

  
  pind->direction += ang;
  if (pind->direction >= 360.0)
    pind->direction -= 360;
  if (pind->direction < 0.0)
    pind->direction += 360;

}

/*
 * we replace the robot in the previous position
 */
void
reset_pos_from_crash(struct individual *pind)

{

    pind->direction = pind->oldirection;
    pind->pos[0]    = pind->oldpos[0];
    pind->pos[1]    = pind->oldpos[1];

    pind->speed[0]=0.0f;
    pind->speed[1]=0.0f;
}






/*
 * set the initial position of the robot
 */
void
set_khepera_position(struct  individual *pind)


{

    extern double fabs();
    double d,mind;
    int    volte;  
    int    startset;
    int    ta;

    startset = 0;

    /*
     * if nstarts > 0 and npopulations= 1 we use the specified positions
     */
     if (env[cworld].nstarts > 0 && startset == 0)
        {
         pind->pos[0]    = env[cworld].starts[cepoch % env[cworld].nstarts][0];
         pind->pos[1]    = env[cworld].starts[cepoch % env[cworld].nstarts][1];
         if (testsamep == 0)
           pind->direction = env[cworld].starts[cepoch % env[cworld].nstarts][2] + rans(10.0);
          else
	  {
           pind->direction = env[cworld].starts[cepoch % env[cworld].nstarts][2];
	   exit;
	  }
	 startset = 1;
        }
    /* 
     * if startonfood khepera start from the foodzone
     */
    if (startonfood == 1 && startset == 0)
      {
        pind->pos[0]    = env[cworld].foodzones[0][0];
        pind->pos[1]    = env[cworld].foodzones[0][1];
        pind->direction = (double) mrand(360);
	startset = 1;
      }

    /* 
     * if startfrontal, individual 1 and 2 start on the 
     * two side of the environment facing each other
     */ 
    if (startfrontal == 1 && startset == 0)
     {
         if (pind->population == 1)
	    {
             pind->pos[0] = (float) ((env[cworld].dx / 4.0f) * 1.0f);
             pind->direction = 0.0;
	    }
	    else
	    {
             pind->pos[0] = (float) ((env[cworld].dx / 4.0f) * 3.0f);
             pind->direction = 180.0;
	    }
         pind->pos[1]    = (float) (env[cworld].dy / 2.0f);
	 if (testsamep == 0)
           pind->direction += (double) mrand(10);
         startset = 1;
     }
    /*
     * set keperas starting position
     * we randomize the position by putting the robot
     *  not in crash and not on foodzones and more than 10cm from
     *  walls, srounds, rounds, and foodzones
     */   
     if (startfarfrom == 1 && startset == 0)
      {
          pind->pos[0]    = (float) fabs(rans((float) (env[cworld].dx - 80))) + 40;
          pind->pos[1]    = (float) fabs(rans((float) (env[cworld].dy - 80))) + 40;
          pind->direction = fabs(rans(360.0));
	  volte = 0;
          while (volte == 0 || (volte < 500 && (check_crash(pind->pos[0],pind->pos[1]) == 1 || mind < 150)))
           {
	    volte++;
            pind->pos[0]    = (float) fabs(rans((float) (env[cworld].dx - 80))) + 40;
            pind->pos[1]    = (float) fabs(rans((float) (env[cworld].dy - 80))) + 40;
            pind->direction = fabs(rans(360.0));
	    mind=9999.9;
            for (ta=0; ta< env[cworld].nsroundobst; ta++)
             {
	       d = mdist(pind->pos[0],pind->pos[1],env[cworld].sroundobst[ta][0],env[cworld].sroundobst[ta][1]);
	       if (d < mind)
	        mind = d;
             }
            for (ta=0; ta< env[cworld].nroundobst; ta++)
             {
	       d = mdist(pind->pos[0],pind->pos[1],env[cworld].roundobst[ta][0],env[cworld].roundobst[ta][1]);
	       if (d < mind)
	        mind = d;
             }
            for (ta=0; ta< env[cworld].nfoodzones; ta++)
             {
	       d = mdist(pind->pos[0],pind->pos[1],env[cworld].foodzones[ta][0],env[cworld].foodzones[ta][1]);
	       if (d < mind)
	        mind = d;
             }
           }
       startset = 1;
      }

    /*
     * set keperas starting position
     * we randomize the position by putting the robot
     *  not in crash and not on foodzones
     */
     if (startset == 0)
      {
	  volte=0;
          do
           {
	    volte++;
            pind->pos[0]    = (float) fabs(rans((float) (env[cworld].dx - 80))) + 40;
            pind->pos[1]    = (float) fabs(rans((float) (env[cworld].dy - 80))) + 40;
            pind->direction = fabs(rans(360.0));
           }
	  while (volte == 0 || (volte < 1000 && (check_crash(pind->pos[0],pind->pos[1]) == 1) ));	  
      }

}


/*
 * initialize variables at the beginning of an epoch of lifetime
 */
void
amoebastart(int gen,int pop, int nind,int epoch)


{


    extern double fabs();
    struct individual *pind;	
    int    ta,tt;    
    double d,mind;
    int    volte;    
    int    n;
    int    alsofoodzones;
    int    wdeltax, wdeltay;
    int    min_dist_ob;


    /*
     * if random_sround and not testsamep we randomize sround positions 
     *   putting objects far from walls and from themselves 
     * if we cannot find a position which satisfy the minimum distance
     *   we reduces the constraint
     * if foodzones where on sround we also randomize foodzone accordingly
     */
    if (random_sround == 1 && testsamep == 0) 
       {
	if (ind->ffitness == 1)
	 {
            random_minwd = 50;
            random_minod = 125;
	 }
	if (ind->ffitness == 6)
	 {
            random_minwd = 100;
            random_minod = 125;
	 }
        if (env[cworld].nsroundobst > 0 && env[cworld].nsroundobst == env[cworld].nfoodzones && (env[cworld].sroundobst[0][0] == env[cworld].foodzones[0][0]) && (env[cworld].sroundobst[0][1] == env[cworld].foodzones[0][1]))   	
	  alsofoodzones = 1;
	 else
          alsofoodzones = 0;
	for (ta=0; ta<env[cworld].nsroundobst; ta++)
         {
	  mind = 0.0;	     
	  volte = 0;
	  min_dist_ob = random_minod;
	  while (mind < min_dist_ob && volte < 1000)
	   {	     
	    volte++;
	    if ((volte > 199) && (volte % 10) == 0)
		min_dist_ob -= 1;
            env[cworld].sroundobst[ta][0] = (float) (fabs(rans((float) env[cworld].dx - (double) (random_minwd * 2))) + random_minwd); 
            env[cworld].sroundobst[ta][1] = (float) (fabs(rans((float) env[cworld].dy - (double) (random_minwd * 2))) + random_minwd); 
            env[cworld].sroundobst[ta][2] = (float) 0.0; 	   
            for (tt=0, mind = 9999.9; tt < ta; tt++)
             {
	      d = mdist(env[cworld].sroundobst[tt][0],env[cworld].sroundobst[tt][1],
                        env[cworld].sroundobst[ta][0],env[cworld].sroundobst[ta][1]);
	      if (d < mind)
	        mind = d;
             }	   
           }
	  if (alsofoodzones == 1)
	   {
             env[cworld].foodzones[ta][0] = env[cworld].sroundobst[ta][0];
             env[cworld].foodzones[ta][1] = env[cworld].sroundobst[ta][1];
	   }
         }
      }

    /*
     * if random_round and not testsamep we randomize round positions 
     *   putting objects far from walls, from srounds and from themselves 
     */
    if (random_round == 1 && testsamep == 0) 
       {
	for (ta=0; ta<env[cworld].nroundobst; ta++)
         {
	  mind = 0.0;	     
	  volte = 0;
	  min_dist_ob = 150;
	  while (mind < min_dist_ob && volte < 1000)
	   {	     
	    mind = 9999.9;	     
	    volte++;
	    if ((volte > 199) && (volte % 10) == 0)
		min_dist_ob -= 1;
            env[cworld].roundobst[ta][0] = (float) (fabs(rans((float) env[cworld].dx - 275)) + 132.0); 
            env[cworld].roundobst[ta][1] = (float) (fabs(rans((float) env[cworld].dy - 275)) + 132.0); 
            env[cworld].roundobst[ta][2] = (float) 0.0; 	   
            for (tt=0; tt < ta; tt++)
             {
	      d = mdist(env[cworld].roundobst[tt][0],env[cworld].roundobst[tt][1],
                        env[cworld].roundobst[ta][0],env[cworld].roundobst[ta][1]);
	      if (d < mind)
	        mind = d;
             }
            for (tt=0; tt < env[cworld].nsroundobst; tt++)
             {
	      d = mdist(env[cworld].sroundobst[tt][0],env[cworld].sroundobst[tt][1],
                        env[cworld].roundobst[ta][0],env[cworld].roundobst[ta][1]);
	      if (d < mind)
	        mind = d;
             }	   	    
           }
         }
      }
    /*
     * if random_foozone and not testsamep we randomize x foodzone positions 
     */
    if (random_foodzone == 1 && testsamep == 0) 
       {
	for (ta=0; ta<env[cworld].nfoodzones; ta++)
         {
           env[cworld].foodzones[ta][0] = (float) (fabs(rans((float) env[cworld].dx - (27.5f * 2.0f))) + 27.5f); 
           env[cworld].foodzones[ta][1] = (float) (fabs(rans((float) env[cworld].dy - (27.5f * 2.0f))) + 27.5f); 
         }
       }

     /* 
      * we set the initial position of the robot
      */
     for(n=0,pind=ind; n < npopulations;n++,pind++)
	 {
          set_khepera_position(pind);
	 }
    

    /*
     * in rat experiments we randomize the scale of the world 
     * and eventually also the proportion between short and log walls
     */
     if ((ind->ffitness == 4 || ind->ffitness == 5) && npopulations == 1)
       {

	 wdeltay = mrand(300) + 150;
	 if (ind->fitness == 4)
            wdeltax = wdeltay * 2;
	   else
            wdeltax = (int) wdeltay * (1.1 + fabs(rans(0.9)));
	 env[cworld].dx = wdeltax;
	 env[cworld].dy = wdeltay;
         env[cworld].obstacles[0][0] = (float) 0;
         env[cworld].obstacles[0][1] = (float) 0;
         env[cworld].obstacles[0][2] = (float) wdeltax;
         env[cworld].obstacles[0][3] = (float) 0;
         env[cworld].obstacles[1][0] = (float) wdeltax;
         env[cworld].obstacles[1][1] = (float) 0;
         env[cworld].obstacles[1][2] = (float) wdeltax;
         env[cworld].obstacles[1][3] = (float) wdeltay;
         env[cworld].obstacles[2][0] = (float) 0;
         env[cworld].obstacles[2][1] = (float) 0;
         env[cworld].obstacles[2][2] = (float) 0;
         env[cworld].obstacles[2][3] = (float) wdeltay;
         env[cworld].obstacles[3][0] = (float) 0;
         env[cworld].obstacles[3][1] = (float) wdeltay;
         env[cworld].obstacles[3][2] = (float) wdeltax;
         env[cworld].obstacles[3][3] = (float) wdeltay;
         
         env[cworld].foodzones[1][0] = (float) (wdeltax - 30);
         env[cworld].foodzones[1][1] = (float) (wdeltay - 15);
       

	 set_khepera_position(ind);    
       
       }

 
  
}


