/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "defs.h"

// C RunTime Header Files
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <windows.h>
#include "w32graph.h"


extern int   startx;
extern int   starty;
extern float scale;

HBITMAP hBackground[5];
int     dBackground[5][6];         
HBRUSH penbr[30];
HBRUSH fillbr[30];
extern HDC  c_hdc;

COLORREF kvgacolor(int c);


/*
 * display a string at given xy position
 */
void
kouttextxy(float x,float y, char *c)

   
{

    extern HDC c_hdc;


    TextOut(c_hdc,(int) (x*scale+startx),
		        (int) (y*scale+starty),c,strlen(c));
    

}

/*
 * display a string at given xy position (does not take into account scale)
 */
void
kouttextxy2(float x,float y, char *c)

   
{

    extern HDC c_hdc;


    TextOut(c_hdc,(int) x,(int) y,c,strlen(c));
    

}

/* 
 * display a string and a floating number at a given location
 */
void
koutfloat(float x,float y, char *c, float f)

   
{

    extern HDC c_hdc;
    char sbuffer[64];


    sprintf(sbuffer,"%s %.3f",c,f);
    TextOut(c_hdc,(int) (x*scale+startx),
		        (int) (y*scale+starty),sbuffer,strlen(sbuffer));
    

}


/*
 * display a string at the bottom of the current window
 */
void
kmessage(char *message)

{


     kdim_device();
     ksetcolor(WHITE);
     ksetfillstyle(SOLID_FILL,WHITE);
     Rectangle(c_hdc, 0, cwindowdy - 25, cwindowdx,cwindowdy); 
     kouttextxy2(0.0f,(float) (cwindowdy - 25),message);
}




void
kputpixel(float x, float y, int c)

{

    extern HDC c_hdc;

    SetPixel(c_hdc, (int) (x*scale+startx),
		    (int) (y*scale+starty),
	            kvgacolor(c));

}


/* Do not work 
int
kgetpixel(float x, float y)

{

    extern float scale;
	extern HDC c_hdc;
	COLORREF cpixel;
	int i;

    cpixel = GetPixel(c_hdc, (int) (x*scale+startx),
		                   (int) (y*scale+starty));
	for (i=0;i<16;i++)
	  if (cpixel == kvgacolor(i))
	    return i;

	return 16;

}
*/

/*
 * draw a line
 */
void
kline(float x, float y, float x2, float y2)

{

    POINT cpoint;
    int xa,ya,xb,yb;

    xa = (int)(x*scale+startx);
    ya = (int)(y*scale+starty);
    xb = (int)(x2*scale+startx);
    yb = (int)(y2*scale+starty);

    MoveToEx(c_hdc,xa,ya,&cpoint);
    LineTo(c_hdc,xb,yb);
    LineTo(c_hdc,xa,ya);
		
}


/*
 * draw a rectangle
 */
void
krectangle(float x, float y,float x2, float y2)

{

    POINT rectangle[5]; 
	rectangle[0].x = (int)(x*scale+startx);
	rectangle[0].y = (int)(y*scale+starty);
	rectangle[1].x = (int)(x2*scale+startx);
	rectangle[1].y = (int)(y*scale+starty);
	rectangle[2].x = (int)(x2*scale+startx);
	rectangle[2].y = (int)(y2*scale+starty);
	rectangle[3].x = (int)(x*scale+startx);
	rectangle[3].y = (int)(y2*scale+starty);
	rectangle[4].x = (int)(x*scale+startx);
	rectangle[4].y = (int)(y*scale+starty);
	

    Polyline(c_hdc, rectangle, 5);
		
}

/*
 * draw an ellipse
 */
void
kellipse(float x, float y,float dxx, float dyy)

{

    int xa;
    int ya;
    int xb;
    int yb;

    
    xa = (int) ((x-dxx)*scale+startx);
    ya = (int) ((y-dyy)*scale+starty);
    xb = (int) ((x+dxx)*scale+startx);
    yb = (int) ((y+dyy)*scale+starty);

    Arc(c_hdc, xa, ya, xb, yb, xa, ya, xa, ya);
		
}

/*
 * draw a circle
 */
void
kcircle(float x, float y, float r)

{

    int xa;
    int ya;
    int xb;
    int yb;

    
    xa = (int) ((x-r)*scale+startx);
    ya = (int) ((y-r)*scale+starty);
    xb = (int) ((x+r)*scale+startx);
    yb = (int) ((y+r)*scale+starty);

    Arc(c_hdc, xa, ya, xb, yb, xa, ya, xa, ya);
		
}


/*
 * draw an arrow
 */ 
void
karrow(float x0,float y0,float x,float y,float rad)

{

     double  dist;
     double  cdirection;			 
     double  idirection;
     double  ldirection;
     double  rdirection; 
     float   backx,backy;     
     float   leftx,lefty;          
     float   rightx,righty;
     float   radscale;
     POINT   cpoint;
     

     radscale = rad*scale;
     if (radscale > 2.0)
        radscale = (float) 2.0;

     dist = mdist((float)x0,(float)y0,(float)x,(float)y); 
     cdirection=mang((int)x0,(int)y0,(int)x,(int)y,dist);
     idirection = cdirection + 180.0;
     if (idirection > 359.0)
	 idirection -= 360.0;
     ldirection = cdirection + 90.0;
     if (ldirection > 359.0)
	ldirection -= 360.0;
     rdirection = cdirection - 90.0;
     if (rdirection < 0.0)
	rdirection += 360.0;

     // generate the scaled starting point 
     x0 = (float)(x0*scale+startx);
     y0 = (float)(y0*scale+starty);
     // reduce the length of the arrow
     x  = SetX(x0,(float) rad*2*scale,cdirection);
     y  = SetY(y0,(float) rad*2*scale,cdirection);

     backx  = SetX(x,(float) radscale,idirection);
     backy  = SetY(y,(float) radscale,idirection);
     rightx = SetX(backx,(float) radscale,rdirection);
     righty = SetY(backy,(float) radscale,rdirection);
     leftx  = SetX(backx,(float) radscale,ldirection);
     lefty  = SetY(backy,(float) radscale,ldirection);


     MoveToEx(c_hdc,(int) x0, (int) y0,&cpoint);
     LineTo(c_hdc,(int) x, (int) y);
     LineTo(c_hdc,(int) x0, (int) y0);
     MoveToEx(c_hdc,(int) rightx, (int) righty,&cpoint);
     LineTo(c_hdc,(int) leftx, (int) lefty);
     LineTo(c_hdc,(int) rightx,(int) righty);
     MoveToEx(c_hdc,(int) rightx, (int) righty,&cpoint);
     LineTo(c_hdc,(int) x, (int) y);
     LineTo(c_hdc,(int) rightx, (int) righty);
     MoveToEx(c_hdc,(int) leftx, (int) lefty,&cpoint);
     LineTo(c_hdc,(int) x, (int) y);
     LineTo(c_hdc,(int) leftx, (int) lefty);

     
}

/*
 * draw a filled rectangle
 */
void
kbar(float x, float y,float x2, float y2)

{

    Rectangle(c_hdc, (int)(x*scale+startx), 
		     (int)(y*scale+starty),
		     (int)(x2*scale+startx),
		     (int)(y2*scale+starty)); 		
}



/*
 * draw a filled ellipse
 */
void
kfillellipse(float x, float y,float dxx, float dyy)

{

    int xa;
    int ya;
    int xb;
    int yb;

    
    xa = (int) ((x-dxx)*scale+startx);
    ya = (int) ((y-dyy)*scale+starty);
    xb = (int) ((x+dxx)*scale+startx);
    yb = (int) ((y+dyy)*scale+starty);

    Ellipse(c_hdc, xa, ya, xb, yb);
		
}

/*
 * draw a filled circle
 */
void
kfillcircle(float x, float y,float r)

{

    int xa;
    int ya;
    int xb;
    int yb;

    
    xa = (int) ((x-r)*scale+startx);
    ya = (int) ((y-r)*scale+starty);
    xb = (int) ((x+r)*scale+startx);
    yb = (int) ((y+r)*scale+starty);

    Ellipse(c_hdc, xa, ya, xb, yb);
		
}


/*
 * return the RGB value of a color (0-15) or of a gray-scale value (16-27)
 */
COLORREF
kvgacolor(int c)

{
switch (c) 
     {
        case 0: /* BLACK */
		return(RGB(0,0,0));
		break;
        case 1: /* BLUE */
		return(RGB(0,0,200));
		break;
        case 2: /* GREEN */   
		return(RGB(0,200,0));
		break;
        case 3: /* CYAN  */
		return(RGB(0,150,150));
		break;
        case 4: /* RED */
		return(RGB(200,0,0));
		break;
        case 5: /* MAGENTA */
		return(RGB(200,50,50));
		break;
        case 6: /* BROWN */   
		return(RGB(200,100,0));
		break;
        case 7: /* LIGHTGRAY  */
		return(RGB(190,190,190));
		break;
        case 8: /* DARKGRAY */
		return(RGB(128,128,128));
		break;
        case 9: /* LIGHTBLUE */
		return(RGB(128,128,255));
		break;
        case 10: /* LIGHTGREEN */   
		return(RGB(128,255,128));
		break;
        case 11: /* LIGHTCYAN  */
		return(RGB(255,255,128));
		break;
        case 12: /* LIGHTRED */
		return(RGB(255,128,128));
		break;
        case 13: /* LIGHTMAGENTA */
		return(RGB(255,128,255));
		break;
        case 14: /* YELLOW */   
		return(RGB(255,255,128));
		break;
        case 15: /* WHITE  */
		return(RGB(255,255,255));
		break;
        case 16: /* GRAY0 */
		return(RGB(0,0,0));
		break;
        case 17: /* GRAY1 */
		return(RGB(25,25,25));
		break;
        case 18: /* GRAY2 */   
		return(RGB(50,50,50));
		break;
        case 19: /* GRAY3  */
		return(RGB(75,75,75));
		break;
        case 20: /* GRAY4 */
		return(RGB(100,100,100));
		break;
        case 21: /* GRAY5 */
		return(RGB(125,125,125));
		break;
        case 22: /* GRAY6 */   
		return(RGB(150,150,150));
		break;
        case 23: /* GRAY7 */
		return(RGB(175,175,175));
		break;
        case 24: /* GRAY8 */
		return(RGB(200,200,200));
		break;
        case 25: /* GRAY9 */
		return(RGB(225,225,225));
		break;
        case 26: /* GRAY10 */   
		return(RGB(255,255,255));
		break;
	default:
	      return(RGB(0,0,0));
          break;

 }

}

/*
 * set the color of filled objects
 */
void
ksetfillstyle(int type, int c)
{

    if (black_and_white == 1)
     {
      if (c != 15)
	c = 0;
     }
     
    
    if (c >= 0 && c < 26)
	  SelectObject(c_hdc,fillbr[c]);	
}


/*
 * set the color 
 */
void
ksetcolor(int c)
{

    if (black_and_white == 1)
     {
      if (c != 15)
	c = 0;
     }

    if (c >= 0 && c <= 26)
 	   SelectObject(c_hdc,penbr[c]);	
}


/*
 * set colors for values between 0.0 and 1.0
 */
int
kcolor_scale(float val)

 {    

    int io_type;

    if (val <= 0.5)
      if (val < 0.25)
        io_type = 9;       
       else
        io_type = 3;        
    if (val > 0.5)  
      if (val > 0.75)
        io_type = 4;       
       else
        io_type = 5;          

    return(io_type);

 }



/*
 * store a portion of main window in memory
 */
void
save_image(int n, float fx1,float fy1,float fx2, float fy2)


{
   
     int  deltax,deltay;
     HDC  hMemoryDC;
     HBITMAP hOldBitmap;
     int x1,y1,x2,y2;


     x1 = (int) (fx1*scale+startx);
     y1 = (int) (fy1*scale+starty);
     x2 = (int) (fx2*scale+startx);
     y2 = (int) (fy2*scale+starty);
     deltax = x2 - x1;
     deltay = y2 - y1;  
	 

     if (n >= 5)
	 display_error("save_image: we allocate space for 5 images only");
     if (!hBackground[n])
	 {
	   hBackground[n] = CreateCompatibleBitmap(c_hdc,deltax,deltay);
	   dBackground[n][3] = deltax;
	   dBackground[n][4] = deltay;
	 }
     if (deltax != dBackground[n][3] || deltay != dBackground[n][4])
	 {
           DeleteObject(hBackground[n]);
           hBackground[n] = CreateCompatibleBitmap(c_hdc,deltax,deltay);
	   dBackground[n][3] = deltax;
	   dBackground[n][4] = deltay;
	 }
     dBackground[n][1] = x1;
     dBackground[n][2] = y1;
     hMemoryDC = CreateCompatibleDC(c_hdc);
     hOldBitmap = SelectObject(hMemoryDC,hBackground[n]);
     BitBlt(hMemoryDC,0,0,dBackground[n][3],dBackground[n][4],c_hdc,x1,y1,SRCCOPY);
     SelectObject(hMemoryDC,hOldBitmap);
     DeleteDC(hMemoryDC);
}

/*
 * restore a portion of main window in memory
 */
void
restore_image(int n)

  
{

    HDC     hMemoryDC;
    HBITMAP hOldBitmap;

    hMemoryDC = CreateCompatibleDC(c_hdc);
    if (hBackground[n])
	 {
           hOldBitmap = SelectObject(hMemoryDC,hBackground[n]);
	   BitBlt(c_hdc,dBackground[n][1],dBackground[n][2],dBackground[n][3],dBackground[n][4],hMemoryDC,0,0,SRCCOPY); 
	   SelectObject(hMemoryDC,hOldBitmap);
	 }

    DeleteDC(hMemoryDC);	      
}



/*
 * allocate space for bitmaps, 
 * create pen colors and brush 
 */
void
kinit_graphics()

{

     int e;

     for(e=0;e<5;e++) 
	    hBackground[e] = NULL;

     for(e=0;e<27;e++) 
          penbr[e]=CreatePen(PS_SOLID,1,kvgacolor(e));
     for(e=0;e<27;e++) 
	  fillbr[e]=CreateSolidBrush(kvgacolor(e));


}



/*
 * clear current device
 */
void
kclear_device()

{

	extern HWND hwnd;
	extern HWND hwndm;
        extern int  cwindow;
	int   dx,dy;
	RECT  rc;

	if (cwindow == 0)
          GetClientRect(hwndm, &rc);
	if (cwindow == 1)
          GetClientRect(hwnd, &rc);

	if (cwindow >= 0 && cwindow <= 1)
	 {
	  dx = rc.right - rc.left;
	  dy = rc.bottom - rc.top;
	  ksetcolor(WHITE);
	  ksetfillstyle(SOLID_FILL,WHITE);
          Rectangle(c_hdc,0,0,dx,dy); 
	 }   
}


/*
 * clear current device from point x,y
 */
void
kclear_device_from(int x, int y)

{

	extern HWND hwnd;
	extern HWND hwndm;
        extern int  cwindow;
	int   dx,dy;
	RECT  rc;

	if (cwindow == 0)
          GetClientRect(hwndm, &rc);
	if (cwindow == 1)
          GetClientRect(hwnd, &rc);

	if (cwindow >= 0 && cwindow <= 1)
	 {
	  dx = rc.right - rc.left;
	  dy = rc.bottom - rc.top;
	  ksetcolor(WHITE);
	  ksetfillstyle(SOLID_FILL,WHITE);
          Rectangle(c_hdc,x,y,dx,dy); 
	 }   
}

/*
 * select the display window (0=main 1=secondary)
 */
void
select_window(int n)
{

   extern HDC  hdc;
   extern HDC  hdcm;
   extern HDC  c_hdc;
   extern int  cwindow;

   c_hdc   = hdcm;
   cwindow = 0;

   if (n == 0)
    {
     c_hdc = hdcm;
     cwindow = 0;
    }
   if (n == 1)
    {
     c_hdc = hdc;
     cwindow = 1;
    }

}


/*
 * update the dimension of the current device 
 */
void
kdim_device()

{

	extern HWND hwnd;
	extern HWND hwndm;
        extern int  cwindow;

	RECT  rc;

	if (cwindow == 0)
          GetClientRect(hwndm, &rc);
	if (cwindow == 1)
          GetClientRect(hwnd, &rc);

	if (cwindow >= 0 && cwindow <= 1)
	 {
	  cwindowdx = rc.right - rc.left;
	  cwindowdy = rc.bottom - rc.top;
	 }   
}


/*
 * set the scale so to maximize the image
 */
void
set_scale(int deltax,int deltay)

{



  kdim_device();

  scale = (float) cwindowdx / (float) (deltax + startx*2);
  if (((float) cwindowdy / (float) (deltay + starty*2)) < scale)
  scale = (float) cwindowdy / (float) (deltay + starty*2);

  startx = (int) ((float) startx * scale);
  starty = (int) ((float) starty * scale);
  
}



