/* 
 * Evorobot - A software for running evolutionary robotics experiments
 * Copyright (C) 2000 Stefano Nolfi. - Institute of Psychology - CNR - Rome 
 * nolfi@ip.rm.cnr.it - http://gral.ip.rm.cnr.it/nolfi 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "defs.h" 
#include <stdio.h>

/*
 * parse a filename .gen extracting the population number
 */
int
parsegenfile(char *filename)

{

    int lens;
    int c,cc;
    int pop;
    char endname[256];
    extern int strlen();

    lens = strlen(filename);

    c = lens - 3;
    while (filename[c] != 'P' && c > 0)
      c--;

    if (filename[c] != 'P')
     {
       sprintf(endname,"unable to extract pop. number from %s filename", filename);
       display_error(endname);
     }

    for(cc=0;c <= lens; c++,cc++)
       endname[cc] = filename[c];

    sscanf(endname,"P%d",&pop);

    return(pop);
	  
}


/*
 * receive a weights as an integer between 0 and 255
 * and return a floating point value within a given range
 * if the range is negative randomly generate the value
 * (255+1 in order to have 128=0.0)
 */

float
getbits(int v, float range)
{

   float value;
   extern double fabs();

   if (range < 0.0)
    {
     value = rans(0.0 - range);
     return(value);
    }
    else
    {
     // this is for compatibility with old weights file (which include also negative values)
     if (v < 0)
       v = 256 + v;

     value = (float) (v + 1) / (float) 256.0;
     value = range - (value * (range * (float) 2.0));
 
     return(value);
    }

}


/*
 * receives an integer (8bits) 
 * and set a weight(randomly generated), 
 *  a learning rule (two bits = hebb, presynaptic, postynaptic, covariance)
 *  a learning rate (two bits = 0.0, 0.33, 0.66, 1.0)
 *  and the sign of the weight (one bit)
 * weights are initialized between -0.1 and 0.1 although they might reach value between -1.0 and 1.0 
 * (i.e. weights' range should be 1.0) 
 */ 
void
getlrule(int g, float *wt, int *rule, float *rate)


{
    
      extern double fabs();
      int b[8];
      int val;
      int i;

      val = g;
      for(i=0;i < 8;i++)
	 {
	    b[i] = val % 2;
	    val  = val / 2;
	 }    

      *rule = b[0] + (b[1] * 2);
      val   = b[2] + (b[3] * 2);
      if (val == 0)
       *rate = (float) 0.0;
      if (val == 1)
       *rate = (float) 0.33;
      if (val == 2)
       *rate = (float) 0.66;
      if (val == 3)
       *rate = (float) 0.99;
      if (b[4] == 0)
        *wt = (float) fabs(rans(0.1));
       else
        *wt = (float) (0.0 - fabs(rans((float) 0.1)));

}



/*
 * return the mutated version of an integer
 */ 
int
mutate(int w)


{
    
      int b[8];
      int val;
      int ii;


      val = w;
      for(ii=0;ii < 8;ii++)
	 {
	    b[ii] = val % 2;
	    val  = val / 2;
	 }    
      for(ii=0;ii < 8;ii++)
        {
	 if (mrand(100) < percmut)
	 {
	   b[ii] = mrand(2);
	 }
	}
      w = 0;
      w += b[0] * 1;
      w += b[1] * 2;
      w += b[2] * 4;
      w += b[3] * 8;
      w += b[4] * 16;
      w += b[5] * 32;
      w += b[6] * 64;      
      w += b[7] * 128;   
 
      return(w);
	 
}


/*
 * generate the phenotype of individual n of population p
 * if type 0 we gen the individual n. from the current generation 
 * if type 1 we get the individual from the bests of the corresponding n. previous generation
 * if type 2 we get the individual from the bests of the corresponding n. previous generation of the same population
 */
void
getgenome(int n, int p, int type)

{
	
	extern  double fabs();
        struct  individual *pind;	
	int     i;
	int     j;
	int     k;
	int	*gi;	
	float	**wp;
	float	**bp;
	float   **wrate;
	float   **brate;
	int     **wrule;
	int     **brule;
	float   *w;
        float   *b;
	float   *wrt;
	float   *brt;
        int     *wru;
	int     *bru;
	int	*bldp;
	int	*tldp;	
	int	*ldp;
	float   range;

	/* we set weights range */
	 pind = (ind + p);
	 range = pind->wrange;

	/* we get the right population and genotype */
	if (type == 0)
	{
	 pind = (ind + p);
         //gi = *(genome + (n + (p * nindividuals)));
         gi = *(genome + (n + (p * (nindividuals + opp_from_prev))));
	}
	if (type == 1)
	{
	 pind = (ind + p);
         //gi = *(genome + n + (npopulations * nindividuals) + (p * opp_from_prev));
	 gi = *(genome + n + (p * (nindividuals + opp_from_prev)) + nindividuals);
	}
	if (type == 2)
	{
	 pind = (ind + p);
         //gi = *(genome + n + (npopulations * nindividuals));
         gi = *(genome + n + nindividuals);
	}

	if (pind->hebbian_learning == 0)
	 {
	   // standard
	   for (i=1, bldp=(pind->layer_descp), tldp=(pind->layer_descp+1), wp=pind->weightp; i < pind->nlayers; i++, bldp++, tldp++, wp++) 
	     {
	       for (j=0, w = *wp; j < *bldp; j++) 
		  {
		      for (k=0; k < *tldp; k++) 
	                  {
                                *(w + j + (k * *bldp)) = getbits((int) *gi, range);
				gi++;
			  }
		    }
	      }
           for (i=0, ldp=pind->layer_descp, bp=pind->biasp; i<pind->nlayers; i++, ldp++, bp++)
	       {
		for (j=0, b = *bp; j < *ldp; j++) 
		       {
                        *b = getbits((int) *gi, range);
			gi++;
                        b++; 
		        }
	       }
	}
	else
	{
	   // hebbian learning
	   // biases are set to 0.0 with rate 0.0 (i.e. they are ignored)
	   for (i=1, bldp=(pind->layer_descp), tldp=(pind->layer_descp+1), wp=pind->weightp, wrule=pind->wrules, wrate=pind->wrates; i < pind->nlayers; i++, bldp++, tldp++, wp++, wrate++, wrule++) 
	     {
	       for (j=0, w = *wp, wru = *wrule, wrt = *wrate; j < *bldp; j++) 
		  {
		      for (k=0; k < *tldp; k++) 
	                  {
			      getlrule((int) *gi, (w + j + (k * *bldp)), (wru + j + (k * *bldp)), (wrt + j + (k * *bldp))); 
			      gi++;
			    }
		    }
	      }
           for (i=0, ldp=pind->layer_descp, bp=pind->biasp, brule=pind->brules, brate=pind->brates; i<pind->nlayers; i++, ldp++, bp++, brule++, brate++)
	       {
		for (j=0, b = *bp, bru = *brule, brt = *brate; j < *ldp; j++) 
		       {
			getlrule((int) *gi, b, bru, brt);
			*b = 0.0f;
			*brt = 0.0f;
			gi++;
                        b++; 
			bru++;
			brt++;
		       }
	       }
	}	
       if (pind->autoteach == 1 || pind->modularity == 2)
        {
	 for (i=1, bldp=(pind->layer_descp), tldp=(pind->layer_descp+1), wp=pind->aweightp; i < pind->nlayers; i++, bldp++, tldp++, wp++) 
	     {
		for (j=0, w = *wp; j < *bldp; j++) 
		    {
			for (k=0; k < *tldp; k++) 
			    {
                                *(w + j + (k * *bldp)) = getbits((int) *gi, range);
				gi++;
			    }
		    }
	      }
         for (i=0, ldp=pind->layer_descp, bp=pind->abiasp; i<pind->nlayers; i++, ldp++, bp++)
	       {
		for (j=0, b = *bp; j < *ldp; j++) 
		       {
                        *b = getbits((int) *gi, range);
			gi++;
                        b++; 
		        }
	       }
       }


}



/*
 * save the net n on the fp file 
 */ 
void
saveanet(FILE *fp, int n, int p)

{
	int	i;
	int	j;
	int	k;

        struct  individual *pind;	
	int     *gi;
	int	*ldp;
	int	*bldp;
	int	*tldp;

	/* we get the right population and genotype */
	pind = (ind + p);
        // gi = *(genome + (n + (p * nindividuals)));
        gi = *(genome + (n + (p * (nindividuals + opp_from_prev))));

	fprintf(fp, "NETWORK CONFIGURED BY RULE\n");
	fprintf(fp, "# weights after %d (fit = %f) runs\n",0,0.0);
	fprintf(fp, "# WEIGHTS\n");
	for (i=1, bldp=(pind->layer_descp), tldp=(pind->layer_descp+1); i < pind->nlayers; i++, bldp++, tldp++) 
	      {
		fprintf(fp, "# from layer %d\n", i-1);
		for (j=0; j < *bldp; j++) 
		       {
			for (k=0; k < *tldp; k++) 
			    {
			     fprintf(fp, "%d\n", *gi);
			     gi++;
			    }
		        }
	      }
	fprintf(fp, "# BIASES\n");
  	for (i=0, ldp = pind->layer_descp; i<pind->nlayers; i++, ldp++) 
	      {
		fprintf(fp, "# from layer %d\n", i);
		for (j=0; j < *ldp; j++) 
		      {
			fprintf(fp, "%d\n", *gi);
			gi++;
		      }
	      }
     if (pind->autoteach == 1 || pind->modularity == 2)
	{
  	fprintf(fp, "NETWORK CONFIGURED BY RULE\n");
	fprintf(fp, "# weights after %d (fit = %f) runs\n",0,0.0);
	fprintf(fp, "# WEIGHTS\n");
	for (i=1, bldp=(pind->layer_descp), tldp=(pind->layer_descp+1); i < pind->nlayers; i++, bldp++, tldp++) 
	      {
		fprintf(fp, "# from layer %d\n", i-1);
		for (j=0; j < *bldp; j++) 
		       {
			for (k=0; k < *tldp; k++) 
			    {
			     fprintf(fp, "%d\n", *gi);
			     gi++;
			    }
		        }
	      }
	fprintf(fp, "# BIASES\n");
  	for (i=0, ldp = pind->layer_descp; i<pind->nlayers; i++, ldp++) 
	      {
		fprintf(fp, "# from layer %d\n", i);
		for (j=0; j < *ldp; j++) 
		       {
			fprintf(fp, "%d\n", *gi);
			gi++;
		       }
	      }
	}
     fprintf(fp, "END\n");

}



/*
 * load the net n of population p from the fp file 
 * mode=1 load the net into opp_from_prev
 */ 
void
loadanet(FILE *fp, int n, int p, int mode)


{
 
        struct  individual *pind;	
	extern int strcmp();
	int	*gi;
	int	*bldp;
	int	*tldp;
	int	*ldp;
	float   buffer;
	int     bufferi;
	char    stringa[64];
	int	i;
	int	j;
	int	k;

	pind = (ind + p);
	// we check if we are asking for an individual of previous generations 
	//if (n >= (nindividuals * npopulations) && n < ((opp_from_prev + nindividuals) * npopulations))
        //gi = *(genome + n);
        //else
        //gi = *(genome + (n + (p * nindividuals)));
	if (mode == 0)
          gi = *(genome + (n + (p * (nindividuals + opp_from_prev))));
	 else
          gi = *(genome + (n + (p * (nindividuals + opp_from_prev)) + nindividuals));

	fscanf(fp, "NETWORK CONFIGURED BY %s\n",stringa);
	if (strcmp(stringa,"RULE") != 0)
          display_error("loading weight's file: network not found");
	fscanf(fp, "# weights after %d (fit = %f) runs\n",&bufferi,&buffer);
	fscanf(fp, "# WEIGHTS\n");
	for (i=1, bldp=(pind->layer_descp), tldp=(pind->layer_descp+1); i < pind->nlayers; i++, bldp++, tldp++) 
	      {
		fscanf(fp, "# from layer %d\n",&bufferi);
		for (j=0; j < *bldp; j++) 
		       {
			for (k=0; k < *tldp; k++) 
			    {
			      fscanf(fp, "%d\n",gi);
			      gi++;
			    }
		        }
	      }
	printf("\n");
	fscanf(fp, "# BIASES\n");
  	for (i=0, ldp = pind->layer_descp; i<pind->nlayers; i++, ldp++) 
	      {
		fscanf(fp, "# from layer %d\n", &bufferi);
		for (j=0; j < *ldp; j++) 
		        {
			 fscanf(fp, "%d\n", gi);
			 gi++;
		        }
	      }
     if (pind->autoteach == 1 || pind->modularity == 2)
	{
  	fscanf(fp, "NETWORK CONFIGURED BY RULE\n");
	fscanf(fp, "# weights after %d (fit = %f) runs\n",&bufferi,&buffer);
	fscanf(fp, "# WEIGHTS\n");
	for (i=1, bldp=(pind->layer_descp), tldp=(pind->layer_descp+1); i < pind->nlayers; i++, bldp++, tldp++) 
	      {
		fscanf(fp, "# from layer %d\n", &bufferi);
		for (j=0; j < *bldp; j++) 
		       {
			for (k=0; k < *tldp; k++) 
			    {
			     fscanf(fp, "%d\n", gi);
			     gi++;
			    }
		        }
	      }
	fscanf(fp, "# BIASES\n");
  	for (i=0, ldp = pind->layer_descp; i<pind->nlayers; i++, ldp++) 
	      {
		fscanf(fp, "# from layer %d\n",&bufferi);
		for (j=0; j < *ldp; j++) 
		        {
			fscanf(fp, "%d\n", gi);
			gi++;
		        }
	      }
	}
      if (oldstylewts == 0)
      {
        fscanf(fp, "%s\n",stringa);
        if (strcmp(stringa,"END") != 0)
	   display_error("loading weight file: neural architecture may be inconsistent");
      }
}





/*
 * save the population genome
 */ 
void
saveallw(int gen)

{

	FILE	*fopen();
	FILE	*fp;
        char    filename[64];
        char    errorm[64];	
        int     n;
	int     p;

	for(p=0;p<npopulations;p++)
	 {
 	  sprintf(filename,"G%dP%dS%d.gen",gen,p,seed);
	  if ((fp=fopen(filename, "w+")) == NULL) 
	       {
		sprintf(errorm,"cannot open %s file",filename);
		display_error(errorm);
	       }	    
	  for(n=0;n<nindividuals;n++)	   
	       {
	        fprintf(fp,"**NET : %d_%d_%d.wts\n",gen,p,n);		
	        saveanet(fp,n,p);
	       } 
	  fflush(fp);	       
	  fclose(fp);
	 }
}



/*
 * load the populations genome
 * if called a number >=0 load all populations
 * if called with a filename load only the population indicated in the filename
 *  in this case it can load up to (individuals+opp_from_prev) individuals 
 */ 
void
loadallw(int gen, char *filew)

{

        struct  individual *pind;
	FILE	*fopen();
	FILE	*fp;
        char    filename[64];
        char    errorm[64];
	char    flag[64];
        int     n;
        int     p;
	int     nloaded;
	int     max;

	for(p=0, pind=ind; p<npopulations; p++, pind++)
	 {
	  max = nindividuals;
	  if (gen == -1)
	     max = nindividuals + opp_from_prev;
	  if (pind->fixed == 0 && pind->randomweights == 0)
	  {
	   if (gen >= 0)
 	      sprintf(filename,"G%dP%dS%d.gen",gen,p,seed);
	     else
	     {
 	      sprintf(filename,"%s",filew);
	     }
	   if (gen >= 0 || p == parsegenfile(filew))
	    if ((fp=fopen(filename, "r")) != NULL) 
	       {
	         for(n=0,nloaded=0;n<max;n++)	   
	            {
		       *flag = NULL;
	               fscanf(fp, "%s : %s\n", flag, errorm);
		       if (strcmp(flag,"**NET") == 0)
		         {
	                  loadanet(fp,nloaded,p,0);
			  pind->dindividual = nloaded;
			  sprintf(errorm,"loading pop: %d ind: %d",p,n);
			  kmessage(errorm);
			  nloaded++;
		         }
		         else
		         {
			   break;
		         }
	            }
	         fclose(fp);
	       }
             else
               {
		 sprintf(errorm,"cannot open file %s\n",filename); 
                 display_error(errorm);
               }
	  }
        }

}




/*
 * load [opp_from_prev] from the best file
 */ 
void
load_previous(int gen)

{
	
        struct  individual *pind;	
	FILE	*fopen();
	FILE	*fp;
        char    filename[64];
        char    errorm[64];	
        int     n;
        int     p;
	int     nprevious;
   
	for(p=0, pind=ind; p<npopulations; p++, pind++)
            {
	      if (pind->fixed == 0)
	      {
	       sprintf(filename,"B%dP%dS%d.gen",1,p,seed);
	       nprevious = 0;
	       if ((fp=fopen(filename, "r")) != NULL) 
	        {
	        for(n=0;n<gen;n++)	   
	            {
	             fscanf(fp, "**NET : %s\n", errorm);
	             loadanet(fp,nprevious,p,1);
 		     if (n >= (gen - opp_from_prev))
		       nprevious++;
	            }
	        fclose(fp);
	        }
	        else
	        {
	 	 sprintf(errorm,"cannot open file %s \n",filename);
		 display_error(errorm);
	        }
	      }
	    }
}



/*
 * copy the bestgenome nn into genome n making mutations
 */
void
getbestweights(int n, int nn, int p)

{

        struct individual *pind;	
	int     *gi;
	int     *bgi;	
        int     i;

	/* we get the right population and genotype */
	pind = (ind + p);
        gi = *(genome + (n + (p * (nindividuals + opp_from_prev))));
        bgi = *(bestgenome + (nn + (p * bestfathers)));
	
	for (i=0;i < pind->numchars;i++,gi++,bgi++)
              *gi = mutate(*bgi);

}





/*
 * copy genome n in the bestgenome nn 
 */
void
putbestweights(int n, int nn, int p)

{


        struct individual *pind;	
	int    *gi;
	int    *bgi;	
        int     i;

	/* we get the right population and genotype */
	pind = (ind + p);
        gi = *(genome + (n + (p * (nindividuals + opp_from_prev))));
        bgi = *(bestgenome + (nn + (p * bestfathers)));
	
	for (i=0;i < pind->numchars;i++,gi++,bgi++)
              *bgi = *gi;	      

}


/*
 * set to 0.0 the activation value of all units
 * set to 'dont care' the teaching
 */
void
reset_net(struct  individual *pind)
{

  float	 **layp;
  float	 *laypi;
  int    *laydp;  
  int    i,ii;



  for (i=0,laydp=pind->layer_descp,layp=pind->layerp; i < pind->nlayers ; i++,laydp++,layp++)
      {
        for (ii=0,laypi = *layp;ii <= *laydp;ii++,laypi++)
          {
           *laypi = (float) 0.0;
	   if (i == (pind->nlayers - 1))
	     pind->amoebat[ii] = (float) -999.0;
          }
      } 
  if (pind->autoteach == 1 || pind->modularity == 2)
   {
    for (i=0,laydp=pind->layer_descp,layp=pind->alayerp; i < pind->nlayers ; i++,laydp++,layp++)
      {
       for (ii=0,laypi = *layp;ii <= *laydp;ii++,laypi++)
         {
          *laypi = (float) 0.0;
         }
      } 
   }



}
 




/*
 * load statistics
 */ 
void
loadstatistics(char *files)

{


     extern  double atof();
     FILE	*fopen();
     FILE	*fp;
     char       line[128];
     char       word[128];
     int n;
     int nc;

     if ((fp=fopen(files, "r")) != NULL) 
	{
	  n = 0;
	  while (fgets(line,128,fp) != NULL && n < 1000)
	  {
            nc = getword(word,line,' ');
	    if (nc > 0)
	      statfit[n][0] = atof(word);
	     else
	      statfit[n][0] = -9999.0;
            nc = getword(word,line,' ');
	    if (nc > 0)
	      statfit[n][1] = atof(word);
	     else
	      statfit[n][0] = -9999.0;
	     if (npopulations == 2)
	     {
               nc = getword(word,line,' ');
	       if (nc > 0)
	         statfit[n][2] = atof(word);
	        else
	         statfit[n][0] = -9999.0;
               nc = getword(word,line,' ');
	       if (nc > 0)
	         statfit[n][3] = atof(word);
	        else
	         statfit[n][0] = -9999.0;
	     }
	    n++;
	  }
	  statfit[n][0] = -9999.0;
	  fclose(fp);
        }
       else
       {
	 display_warning("error loading statistics file");
       }

}



